@@ -1,5 +1,5 @@
 /* This file should be included in the multipack module */
-/* $Revision: 4953 $ */
+/* $Revision$ */
 /*  module_methods:
  {"odeint", (PyCFunction) odepack_odeint, METH_VARARGS|METH_KEYWORDS, doc_odeint},
  */
@@ -14,10 +14,18 @@
  */
 
 
-#if defined(NO_APPEND_FORTRAN)
-#define LSODA  lsoda
+#if defined(UPPERCASE_FORTRAN)
+	#if defined(NO_APPEND_FORTRAN)
+		/* nothing to do here */
+	#else
+		#define LSODA  LSODA_
+	#endif
 #else
-#define LSODA  lsoda_
+	#if defined(NO_APPEND_FORTRAN)
+		#define LSODA  lsoda
+	#else
+		#define LSODA  lsoda_
+	#endif
 #endif
 
 void LSODA();
