@@ -1,11 +1,13 @@
 # Author: Travis Oliphant 2001
 
-__all__ = ['quad', 'dblquad', 'tplquad', 'quad_explain', 'Inf','inf']
 import _quadpack
 import sys
 import numpy
 from numpy import inf, Inf
 
+__all__ = ['quad', 'dblquad', 'tplquad', 'quad_explain']
+
+
 error = _quadpack.error
 
 def quad_explain(output=sys.stdout):
@@ -114,17 +116,17 @@ Weighting the integrand:
   information dictionary contains the following entries instead of 'last',
   'alist', 'blist', 'rlist', and 'elist':
 
-  'lst' -- The number of subintervals needed for the integration (call it K_f).
-  'rslst' -- A rank-1 array of length M_f=limlst, whose first K_f elements
+  'lst'    : The number of subintervals needed for the integration (call it K_f).
+  'rslst'  : A rank-1 array of length M_f=limlst, whose first K_f elements
              contain the integral contribution over the interval (a+(k-1)c,
              a+kc) where c = (2*floor(|w|) + 1) * pi / |w| and k=1,2,...,K_f.
-  'erlst' -- A rank-1 array of length M_f containing the error estimate
+  'erlst'  : A rank-1 array of length M_f containing the error estimate
              corresponding to the interval in the same position in
              infodict['rslist'].
-  'ierlst' -- A rank-1 integer array of length M_f containing an error flag
-              corresponding to the interval in the same position in
-              infodict['rslist'].  See the explanation dictionary (last entry
-              in the output tuple) for the meaning of the codes.
+  'ierlst' : A rank-1 integer array of length M_f containing an error flag
+             corresponding to the interval in the same position in
+             infodict['rslist'].  See the explanation dictionary (last entry
+             in the output tuple) for the meaning of the codes.
 """)
     return
 
@@ -207,21 +209,20 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
 
     See Also
     --------
-        dblquad, tplquad - double and triple integrals
-        fixed_quad - fixed-order Gaussian quadrature
-        quadrature - adaptive Gaussian quadrature
-        odeint, ode - ODE integrators
-        simps, trapz, romb - integrators for sampled data
-        scipy.special - for coefficients and roots of orthogonal polynomials
+    dblquad, tplquad : double and triple integrals
+    fixed_quad : fixed-order Gaussian quadrature
+    quadrature : adaptive Gaussian quadrature
+    odeint, ode : ODE integrators
+    simps, trapz, romb : integrators for sampled data
+    scipy.special : for coefficients and roots of orthogonal polynomials
 
     Examples
     --------
-
     Calculate :math:`\\int^4_0 x^2 dx` and compare with an analytic result
 
     >>> from scipy import integrate
     >>> x2 = lambda x: x**2
-    >>> integrate.quad(x,0.,4.)
+    >>> integrate.quad(x2,0.,4.)
     (21.333333333333332, 2.3684757858670003e-13)
     >> print 4.**3/3
     21.3333333333
@@ -232,7 +233,6 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
     >>> integrate.quad(invexp,0,inf)
     (0.99999999999999989, 5.8426061711142159e-11)
 
-
     >>> f = lambda x,a : a*x
     >>> y, err = integrate.quad(f, 0, 1, args=(1,))
     >>> y
@@ -254,7 +254,7 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
 
     msgs = {80: "A Python error occurred possibly while calling the function.",
              1: "The maximum number of subdivisions (%d) has been achieved.\n  If increasing the limit yields no improvement it is advised to analyze \n  the integrand in order to determine the difficulties.  If the position of a \n  local difficulty can be determined (singularity, discontinuity) one will \n  probably gain from splitting up the interval and calling the integrator \n  on the subranges.  Perhaps a special-purpose integrator should be used." % limit,
-             2: "The ocurrence of roundoff error is detected, which prevents \n  the requested tolerance from being achieved.  The error may be \n  underestimated.",
+             2: "The occurrence of roundoff error is detected, which prevents \n  the requested tolerance from being achieved.  The error may be \n  underestimated.",
              3: "Extremely bad integrand behavior occurs at some points of the\n  integration interval.",
              4: "The algorithm does not converge.  Roundoff error is detected\n  in the extrapolation table.  It is assumed that the requested tolerance\n  cannot be achieved, and that the returned result (if full_output = 1) is \n  the best which can be obtained.",
              5: "The integral is probably divergent, or slowly convergent.",
@@ -284,10 +284,11 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
             else:
                 return retval[:-1] + (msg,)
         else:
-            print "Warning: " + msg
+            import warnings
+            warnings.warn(msg)
             return retval[:-1]
     else:
-        raise ValueError, msg
+        raise ValueError(msg)
 
 
 def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
@@ -304,7 +305,7 @@ def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
         infbounds = -1
         bound = b
     else:
-        raise RuntimeError, "Infinity comparisons don't work for you."
+        raise RuntimeError("Infinity comparisons don't work for you.")
 
     if points is None:
         if infbounds == 0:
@@ -313,7 +314,7 @@ def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
             return _quadpack._qagie(func,bound,infbounds,args,full_output,epsabs,epsrel,limit)
     else:
         if infbounds !=0:
-            raise ValueError, "Infinity inputs cannot be used with break points."
+            raise ValueError("Infinity inputs cannot be used with break points.")
         else:
             nl = len(points)
             the_points = numpy.zeros((nl+2,), float)
@@ -324,7 +325,7 @@ def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
 def _quad_weight(func,a,b,args,full_output,epsabs,epsrel,limlst,limit,maxp1,weight,wvar,wopts):
 
     if weight not in ['cos','sin','alg','alg-loga','alg-logb','alg-log','cauchy']:
-        raise ValueError, "%s not a recognized weighting function." % weight
+        raise ValueError("%s not a recognized weighting function." % weight)
 
     strdict = {'cos':1,'sin':2,'alg':1,'alg-loga':2,'alg-logb':3,'alg-log':4}
 
@@ -356,10 +357,10 @@ def _quad_weight(func,a,b,args,full_output,epsabs,epsrel,limlst,limit,maxp1,weig
             args = (func,) + args
             return _quadpack._qawfe(thefunc,-b,wvar,integr,args,full_output,epsabs,limlst,limit,maxp1)
         else:
-            raise ValueError, "Cannot integrate with this weight from -Inf to +Inf."
+            raise ValueError("Cannot integrate with this weight from -Inf to +Inf.")
     else:
         if a in [-Inf,Inf] or b in [-Inf,Inf]:
-            raise ValueError, "Cannot integrate with this weight over an infinite interval."
+            raise ValueError("Cannot integrate with this weight over an infinite interval.")
 
         if weight[:3] == 'alg':
             integr = strdict[weight]
@@ -375,46 +376,48 @@ def _infunc(x,func,gfun,hfun,more_args):
 
 def dblquad(func, a, b, gfun, hfun, args=(), epsabs=1.49e-8, epsrel=1.49e-8):
     """
-    Compute the double integral of func2d(y,x)
-    from x=a..b and y=gfun(x)..hfun(x).
+    Compute a double integral.
+
+    Return the double (definite) integral of func(y,x) from x=a..b and
+    y=gfun(x)..hfun(x).
 
     Parameters
     -----------
-      func2d : function
-             a Python function or method of at least two variables: y must be
-             the first argument and x the second argument.
-      (a,b)  : tuple
-             the limits of integration in x: a < b
-      gfun   : function
-             the lower boundary curve in y which is a function taking a single
-             floating point argument (x) and returning a floating point result:
-             a lambda function can be useful here.
-      hfun   : function
-             the upper boundary curve in y (same requirements as gfun).
-      args   :
-             extra arguments to pass to func2d.
-      epsabs : float
-             absolute tolerance passed directly to the inner 1-D quadrature
-             integration.
-      epsrel : float
-             relative tolerance of the inner 1-D integrals.
-
+    func : callable
+        A Python function or method of at least two variables: y must be the
+        first argument and x the second argument.
+    (a,b) : tuple
+        The limits of integration in x: a < b
+    gfun : callable
+        The lower boundary curve in y which is a function taking a single
+        floating point argument (x) and returning a floating point result: a
+        lambda function can be useful here.
+    hfun : callable
+        The upper boundary curve in y (same requirements as `gfun`).
+    args : sequence, optional
+        Extra arguments to pass to `func2d`.
+    epsabs : float, optional
+        Absolute tolerance passed directly to the inner 1-D quadrature
+        integration. Default is 1.49e-8.
+    epsrel : float
+        Relative tolerance of the inner 1-D integrals. Default is 1.49e-8.
 
     Returns
-    -----------
-      y      : float
-             the resultant integral.
-      abserr : float
-             an estimate of the error.
+    -------
+    y : float
+        The resultant integral.
+    abserr : float
+        An estimate of the error.
 
-    See also:
-        quad - single integral
-        tplquad - triple integral
-        fixed_quad - fixed-order Gaussian quadrature
-        quadrature - adaptive Gaussian quadrature
-        odeint, ode - ODE integrators
-        simps, trapz, romb - integrators for sampled data
-        scipy.special - for coefficients and roots of orthogonal polynomials
+    See also
+    --------
+    quad : single integral
+    tplquad : triple integral
+    fixed_quad : fixed-order Gaussian quadrature
+    quadrature : adaptive Gaussian quadrature
+    odeint, ode : ODE integrators
+    simps, trapz, romb : integrators for sampled data
+    scipy.special : for coefficients and roots of orthogonal polynomials
 
     """
     return quad(_infunc,a,b,(func,gfun,hfun,args),epsabs=epsabs,epsrel=epsrel)
@@ -430,12 +433,12 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
     """
     Compute a triple (definite) integral.
 
-    Return the triple integral of func3d(z, y,x) from
+    Return the triple integral of func(z, y, x) from
     x=a..b, y=gfun(x)..hfun(x), and z=qfun(x,y)..rfun(x,y)
 
     Parameters
     ----------
-    func3d : function
+    func : function
         A Python function or method of at least three variables in the
         order (z, y, x).
     (a,b) : tuple
@@ -453,11 +456,11 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
         The upper boundary surface in z. (Same requirements as qfun.)
     args : Arguments
         Extra arguments to pass to func3d.
-    epsabs : float
+    epsabs : float, optional
         Absolute tolerance passed directly to the innermost 1-D quadrature
-        integration.
-    epsrel : float
-        Relative tolerance of the innermost 1-D integrals.
+        integration. Default is 1.49e-8.
+    epsrel : float, optional
+        Relative tolerance of the innermost 1-D integrals. Default is 1.49e-8.
 
     Returns
     -------
