@@ -8,6 +8,10 @@ from numpy import sum, ones, add, diff, isinf, isscalar, \
      asarray, real, trapz, arange, empty
 import numpy as np
 import math
+import warnings
+
+class AccuracyWarning(Warning):
+    pass
 
 def fixed_quad(func,a,b,args=(),n=5):
     """
@@ -48,8 +52,8 @@ def fixed_quad(func,a,b,args=(),n=5):
     x = real(x)
     ainf, binf = map(isinf,(a,b))
     if ainf or binf:
-        raise ValueError, "Gaussian quadrature is only available for " \
-              "finite limits."
+        raise ValueError("Gaussian quadrature is only available for "
+                "finite limits.")
     y = (b-a)*(x+1)/2.0 + a
     return (b-a)/2.0*sum(w*func(y,*args),0), None
 
@@ -99,7 +103,8 @@ def vectorize1(func, args=(), vec_func=False):
             return output
     return vfunc
 
-def quadrature(func,a,b,args=(),tol=1.49e-8,maxiter=50, vec_func=True):
+def quadrature(func, a, b, args=(), tol=1.49e-8, rtol=1.49e-8, maxiter=50,
+               vec_func=True):
     """
     Compute a definite integral using fixed-tolerance Gaussian quadrature.
 
@@ -116,9 +121,9 @@ def quadrature(func,a,b,args=(),tol=1.49e-8,maxiter=50, vec_func=True):
         Upper limit of integration.
     args : tuple, optional
         Extra arguments to pass to function.
-    tol : float, optional
+    tol, rol : float, optional
         Iteration stops when error between last two iterates is less than
-        tolerance.
+        `tol` OR the relative change is less than `rtol`.
     maxiter : int, optional
         Maximum number of iterations.
     vec_func : bool, optional
@@ -147,17 +152,20 @@ def quadrature(func,a,b,args=(),tol=1.49e-8,maxiter=50, vec_func=True):
     odeint: ODE integrator
 
     """
-    err = 100.0
-    val = err
-    n = 1
     vfunc = vectorize1(func, args, vec_func=vec_func)
-    while (err > tol) and (n < maxiter):
+    val = np.inf
+    err = np.inf
+    for n in xrange(1, maxiter+1):
         newval = fixed_quad(vfunc, a, b, (), n)[0]
         err = abs(newval-val)
         val = newval
-        n = n + 1
-    if n == maxiter:
-        print "maxiter (%d) exceeded. Latest difference = %e" % (n,err)
+
+        if err < tol or err < rtol*abs(val):
+            break
+    else:
+        warnings.warn(
+            "maxiter (%d) exceeded. Latest difference = %e" % (maxiter, err),
+            AccuracyWarning)
     return val, err
 
 def tupleset(t, i, value):
@@ -168,7 +176,7 @@ def tupleset(t, i, value):
 def cumtrapz(y, x=None, dx=1.0, axis=-1):
     """
     Cumulatively integrate y(x) using samples along the given axis
-    and the composite trapezoidal rule.  If x is None, spacing given by dx
+    and the composite trapezoidal rule.  If x is None, spacing given by `dx`
     is assumed.
 
     Parameters
@@ -188,18 +196,17 @@ def cumtrapz(y, x=None, dx=1.0, axis=-1):
 
     See Also
     --------
-
-      quad: adaptive quadrature using QUADPACK
-      romberg: adaptive Romberg quadrature
-      quadrature: adaptive Gaussian quadrature
-      fixed_quad: fixed-order Gaussian quadrature
-      dblquad: double integrals
-      tplquad: triple integrals
-      romb: integrators for sampled data
-      trapz: integrators for sampled data
-      cumtrapz: cumulative integration for sampled data
-      ode: ODE integrators
-      odeint: ODE integrators
+    quad: adaptive quadrature using QUADPACK
+    romberg: adaptive Romberg quadrature
+    quadrature: adaptive Gaussian quadrature
+    fixed_quad: fixed-order Gaussian quadrature
+    dblquad: double integrals
+    tplquad: triple integrals
+    romb: integrators for sampled data
+    trapz: integrators for sampled data
+    cumtrapz: cumulative integration for sampled data
+    ode: ODE integrators
+    odeint: ODE integrators
 
     """
     y = asarray(y)
@@ -310,19 +317,18 @@ def simps(y, x=None, dx=1, axis=-1, even='avg'):
             returnshape = 1
             x=x.reshape(tuple(shapex))
         elif len(x.shape) != len(y.shape):
-            raise ValueError, "If given, shape of x must be 1-d or the " \
-                  "same as y."
+            raise ValueError("If given, shape of x must be 1-d or the "
+                    "same as y.")
         if x.shape[axis] != N:
-            raise ValueError, "If given, length of x along axis must be the " \
-                  "same as y."
+            raise ValueError("If given, length of x along axis must be the "
+                    "same as y.")
     if N % 2 == 0:
         val = 0.0
         result = 0.0
         slice1 = (slice(None),)*nd
         slice2 = (slice(None),)*nd
         if not even in ['avg', 'last', 'first']:
-            raise ValueError, \
-                  "Parameter 'even' must be 'avg', 'last', or 'first'."
+            raise ValueError("Parameter 'even' must be 'avg', 'last', or 'first'.")
         # Compute using Simpson's rule on first intervals
         if even in ['avg', 'first']:
             slice1 = tupleset(slice1, axis, -1)
@@ -351,40 +357,36 @@ def simps(y, x=None, dx=1, axis=-1, even='avg'):
 
 def romb(y, dx=1.0, axis=-1, show=False):
     """
-    Romberg integration using samples of a function
+    Romberg integration using samples of a function.
 
     Parameters
     -----------
-      y : array like
-           a vector of 2**k + 1 equally-spaced samples of a function
-
-      dx : array like
-           the sample spacing.
-
-      axis : array like?
-           the axis along which to integrate
-
-      show : Boolean
-           When y is a single 1-d array, then if this argument is True
+    y : array_like
+        A vector of ``2**k + 1`` equally-spaced samples of a function.
+    dx : array_like, optional
+        The sample spacing. Default is 1.
+    axis : array_like?, optional
+        The axis along which to integrate. Default is -1 (last axis).
+    show : bool, optional
+           When y is a single 1-D array, then if this argument is True
            print the table showing Richardson extrapolation from the
-           samples.
+           samples. Default is False.
 
     Returns
-    -----------
+    -------
+    ret : array_like?
+        The integrated result for each axis.
 
-       ret : array_like?
-          The integrated result for each axis.
-
-    See also:
-
-      quad - adaptive quadrature using QUADPACK
-      romberg - adaptive Romberg quadrature
-      quadrature - adaptive Gaussian quadrature
-      fixed_quad - fixed-order Gaussian quadrature
-      dblquad, tplquad - double and triple integrals
-      simps, trapz - integrators for sampled data
-      cumtrapz - cumulative integration for sampled data
-      ode, odeint - ODE integrators
+    See also
+    --------
+    quad - adaptive quadrature using QUADPACK
+    romberg - adaptive Romberg quadrature
+    quadrature - adaptive Gaussian quadrature
+    fixed_quad - fixed-order Gaussian quadrature
+    dblquad, tplquad - double and triple integrals
+    simps, trapz - integrators for sampled data
+    cumtrapz - cumulative integration for sampled data
+    ode, odeint - ODE integrators
 
     """
     y = asarray(y)
@@ -397,8 +399,8 @@ def romb(y, dx=1.0, axis=-1, show=False):
         n <<= 1
         k += 1
     if n != Ninterv:
-        raise ValueError, \
-              "Number of samples must be one plus a non-negative power of 2."
+        raise ValueError("Number of samples must be one plus a "
+                "non-negative power of 2.")
 
     R = {}
     all = (slice(None),) * nd
@@ -505,7 +507,7 @@ def _printresmat(function, interval, resmat):
     print 'The final result is', resmat[i][j],
     print 'after', 2**(len(resmat)-1)+1, 'function evaluations.'
 
-def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
+def romberg(function, a, b, args=(), tol=1.48e-8, rtol=1.48e-8, show=False,
             divmax=10, vec_func=False):
     """
     Romberg integration of a callable function or method.
@@ -537,12 +539,12 @@ def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
         Extra arguments to pass to function. Each element of `args` will
         be passed as a single argument to `func`. Default is to pass no
         extra arguments.
-    tol : float, optional
-        The desired tolerance. Default is 1.48e-8.
+    tol, rtol : float, optional
+        The desired absolute and relative tolerances. Defaults are 1.48e-8.
     show : bool, optional
         Whether to print the results. Default is False.
     divmax : int, optional
-        ?? Default is 10.
+        Maximum order of extrapolation. Default is 10.
     vec_func : bool, optional
         Whether `func` handles arrays as arguments (i.e whether it is a
         "vector" function). Default is False.
@@ -562,7 +564,7 @@ def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
 
     Examples
     --------
-    Integrate a gaussian from 0,1 and compare to the error function.
+    Integrate a gaussian from 0 to 1 and compare to the error function.
 
     >>> from scipy.special import erf
     >>> gaussian = lambda x: 1/np.sqrt(np.pi) * np.exp(-x**2)
@@ -588,14 +590,14 @@ def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
     if isinf(a) or isinf(b):
         raise ValueError("Romberg integration only available for finite limits.")
     vfunc = vectorize1(function, args, vec_func=vec_func)
-    i = n = 1
+    n = 1
     interval = [a,b]
     intrange = b-a
     ordsum = _difftrap(vfunc, interval, n)
     result = intrange * ordsum
     resmat = [[result]]
-    lastresult = result + tol * 2.0
-    while (abs(result - lastresult) > tol) and (i <= divmax):
+    err = np.inf
+    for i in xrange(1, divmax+1):
         n = n * 2
         ordsum = ordsum + _difftrap(vfunc, interval, n)
         resmat.append([])
@@ -604,7 +606,15 @@ def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
             resmat[i].append(_romberg_diff(resmat[i-1][k], resmat[i][k], k+1))
         result = resmat[i][i]
         lastresult = resmat[i-1][i-1]
-        i = i + 1
+
+        err = abs(result - lastresult)
+        if err < tol or err < rtol*abs(result):
+            break
+    else:
+        warnings.warn(
+            "divmax (%d) exceeded. Latest difference = %e" % (divmax, err),
+            AccuracyWarning)
+
     if show:
         _printresmat(vfunc, interval, resmat)
     return result
@@ -670,7 +680,7 @@ _builtincoeffs = {
         1275983280000)
     }
 
-def newton_cotes(rn,equal=0):
+def newton_cotes(rn, equal=0):
     """
     Return weights and error coefficient for Newton-Cotes integration.
 
@@ -689,22 +699,20 @@ def newton_cotes(rn,equal=0):
 
     Parameters
     ----------
-
     rn : int
-        The integer order for equally-spaced data
-        or the relative positions of the samples with
-        the first sample at 0 and the last at N, where
-        N+1 is the length of rn.  N is the order of the Newton
+        The integer order for equally-spaced data or the relative positions of
+        the samples with the first sample at 0 and the last at N, where N+1 is
+        the length of `rn`.  N is the order of the Newton-Cotes integration.
     equal: int, optional
-        Set to 1 to enforce equally spaced data
+        Set to 1 to enforce equally spaced data.
 
     Returns
     -------
-    an : array
-        1-d array of weights to apply to the function at
-        the provided sample positions.
-    B  : float
-        error coefficient
+    an : ndarray
+        1-D array of weights to apply to the function at the provided sample
+        positions.
+    B : float
+        Error coefficient.
 
     Notes
     -----
@@ -728,8 +736,8 @@ def newton_cotes(rn,equal=0):
         return na*np.array(vi,float)/da, float(nb)/db
 
     if (rn[0] != 0) or (rn[-1] != N):
-        raise ValueError, "The sample positions must start at 0"\
-              " and end at N"
+        raise ValueError("The sample positions must start at 0"
+                " and end at N")
     yi = rn / float(N)
     ti = 2.0*yi - 1
     nvec = np.arange(0,N+1)
