@@ -18,8 +18,10 @@ __all__ = [
     'RectBivariateSpline']
 
 import warnings
-from numpy import zeros, concatenate, alltrue, ravel, all, diff
+from numpy import zeros, concatenate, alltrue, ravel, all, diff, array
+import numpy as np
 
+import fitpack
 import dfitpack
 
 ################ Univariate spline ####################
@@ -55,21 +57,21 @@ class UnivariateSpline(object):
     """
     One-dimensional smoothing spline fit to a given set of data points.
 
-    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data.  `s`
+    Fits a spline y=s(x) of degree `k` to the provided `x`, `y` data.  `s`
     specifies the number of knots by specifying a smoothing condition.
 
     Parameters
     ----------
-    x : sequence
-        input dimension of data points -- must be increasing
-    y : sequence
-         input dimension of data points
-    w : sequence or None, optional
-        weights for spline fitting.  Must be positive.  If None (default),
+    x : array_like
+        1-D array of independent input data. Must be increasing.
+    y : array_like
+        1-D array of dependent input data, of the same length as `x`.
+    w : array_like, optional
+        Weights for spline fitting.  Must be positive.  If None (default),
         weights are all equal.
-    bbox : sequence or None, optional
+    bbox : array_like, optional
         2-sequence specifying the boundary of the approximation interval. If
-        None (default), bbox=[x[0],x[-1]].
+        None (default), ``bbox=[x[0], x[-1]]``.
     k : int, optional
         Degree of the smoothing spline.  Must be <= 5.
     s : float or None, optional
@@ -82,7 +84,6 @@ class UnivariateSpline(object):
         an estimate of the standard deviation of y[i].  If 0, spline will
         interpolate through all data points.
 
-
     See Also
     --------
     InterpolatedUnivariateSpline : Subclass with smoothing forced to 0
@@ -92,20 +93,22 @@ class UnivariateSpline(object):
     splev, sproot, splint, spalde
     BivariateSpline : A similar class for two-dimensional spline interpolation
 
-
+    Notes
+    -----
+    The number of data points must be larger than the spline degree `k`.
 
     Examples
     --------
     >>> from numpy import linspace,exp
     >>> from numpy.random import randn
     >>> from scipy.interpolate import UnivariateSpline
-    >>> x = linspace(-3,3,100)
+    >>> x = linspace(-3, 3, 100)
     >>> y = exp(-x**2) + randn(100)/10
-    >>> s = UnivariateSpline(x,y,s=1)
-    >>> xs = linspace(-3,3,1000)
+    >>> s = UnivariateSpline(x, y, s=1)
+    >>> xs = linspace(-3, 3, 1000)
     >>> ys = s(xs)
 
-    xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y
+    xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y.
 
     """
 
@@ -176,13 +179,10 @@ class UnivariateSpline(object):
             k,m = data[5],len(data[0])
             nest = m+k+1 # this is the maximum bound for nest
         else:
-            assert n<=nest,"nest can only be increased"
-        t,c,fpint,nrdata = data[8].copy(),data[9].copy(),\
-                           data[11].copy(),data[12].copy()
-        t.resize(nest)
-        c.resize(nest)
-        fpint.resize(nest)
-        nrdata.resize(nest)
+            if not n <= nest:
+                raise ValueError("`nest` can only be increased")
+        t, c, fpint, nrdata = [np.resize(data[n], nest) for n in [8,9,11,12]]
+
         args = data[:8] + (t,c,n,fpint,nrdata,data[13])
         data = dfitpack.fpcurf1(*args)
         return data
@@ -205,15 +205,20 @@ class UnivariateSpline(object):
         self._data = data
         self._reset_class()
 
-    def __call__(self, x, nu=None):
+    def __call__(self, x, nu=0):
         """ Evaluate spline (or its nu-th derivative) at positions x.
         Note: x can be unordered but the evaluation is more efficient
         if x is (partially) ordered.
 
         """
-        if nu is None:
-            return dfitpack.splev(*(self._eval_args+(x,)))
-        return dfitpack.splder(nu=nu,*(self._eval_args+(x,)))
+        x = np.asarray(x)
+        # empty input yields empty output
+        if x.size == 0:
+            return array([])
+#        if nu is None:
+#            return dfitpack.splev(*(self._eval_args+(x,)))
+#        return dfitpack.splder(nu=nu,*(self._eval_args+(x,)))
+        return fitpack.splev(x, self._eval_args, der=nu)
 
     def get_knots(self):
         """ Return the positions of (boundary and interior)
@@ -245,7 +250,8 @@ class UnivariateSpline(object):
     def derivatives(self, x):
         """ Return all derivatives of the spline at the point x."""
         d,ier = dfitpack.spalde(*(self._eval_args+(x,)))
-        assert ier==0,`ier`
+        if not ier == 0:
+            raise ValueError("Error code returned by spalde: %s" % ier)
         return d
 
     def roots(self):
@@ -256,35 +262,35 @@ class UnivariateSpline(object):
         k = self._data[5]
         if k==3:
             z,m,ier = dfitpack.sproot(*self._eval_args[:2])
-            assert ier==0,`ier`
+            if not ier == 0:
+                raise ValueError("Error code returned by spalde: %s" % ier)
             return z[:m]
-        raise NotImplementedError,\
-              'finding roots unsupported for non-cubic splines'
+        raise NotImplementedError('finding roots unsupported for '
+                                    'non-cubic splines')
 
 class InterpolatedUnivariateSpline(UnivariateSpline):
     """
     One-dimensional interpolating spline for a given set of data points.
 
-    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data. Spline
+    Fits a spline y=s(x) of degree `k` to the provided `x`, `y` data. Spline
     function passes through all provided points. Equivalent to
     `UnivariateSpline` with  s=0.
 
     Parameters
     ----------
-    x : sequence
+    x : array_like
         input dimension of data points -- must be increasing
-    y : sequence
-         input dimension of data points
-    w : sequence or None, optional
-        weights for spline fitting.  Must be positive.  If None (default),
+    y : array_like
+        input dimension of data points
+    w : array_like, optional
+        Weights for spline fitting.  Must be positive.  If None (default),
         weights are all equal.
-    bbox : sequence or None, optional
+    bbox : array_like, optional
         2-sequence specifying the boundary of the approximation interval. If
         None (default), bbox=[x[0],x[-1]].
     k : int, optional
         Degree of the smoothing spline.  Must be <= 5.
 
-
     See Also
     --------
     UnivariateSpline : Superclass -- allows knots to be selected by a
@@ -294,17 +300,19 @@ class InterpolatedUnivariateSpline(UnivariateSpline):
     splev, sproot, splint, spalde
     BivariateSpline : A similar class for two-dimensional spline interpolation
 
-
+    Notes
+    -----
+    The number of data points must be larger than the spline degree `k`.
 
     Examples
     --------
     >>> from numpy import linspace,exp
     >>> from numpy.random import randn
     >>> from scipy.interpolate import UnivariateSpline
-    >>> x = linspace(-3,3,100)
+    >>> x = linspace(-3, 3, 100)
     >>> y = exp(-x**2) + randn(100)/10
-    >>> s = UnivariateSpline(x,y,s=1)
-    >>> xs = linspace(-3,3,1000)
+    >>> s = UnivariateSpline(x, y, s=1)
+    >>> xs = linspace(-3, 3, 1000)
     >>> ys = s(xs)
 
     xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y
@@ -333,22 +341,22 @@ class LSQUnivariateSpline(UnivariateSpline):
     """
     One-dimensional spline with explicit internal knots.
 
-    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data.  `t`
+    Fits a spline y=s(x) of degree `k` to the provided `x`, `y` data.  `t`
     specifies the internal knots of the spline
 
     Parameters
     ----------
-    x : sequence
+    x : array_like
         input dimension of data points -- must be increasing
-    y : sequence
+    y : array_like
         input dimension of data points
-    t: sequence
+    t: array_like
         interior knots of the spline.  Must be in ascending order
         and bbox[0]<t[0]<...<t[-1]<bbox[-1]
-    w : sequence or None, optional
+    w : array_like, optional
         weights for spline fitting.  Must be positive.  If None (default),
         weights are all equal.
-    bbox : sequence or None, optional
+    bbox : array_like, optional
         2-sequence specifying the boundary of the approximation interval. If
         None (default), bbox=[x[0],x[-1]].
     k : int, optional
@@ -356,7 +364,6 @@ class LSQUnivariateSpline(UnivariateSpline):
 
     Raises
     ------
-
     ValueError
         If the interior knots do not satisfy the Schoenberg-Whitney conditions
 
@@ -369,7 +376,9 @@ class LSQUnivariateSpline(UnivariateSpline):
     splev, sproot, splint, spalde
     BivariateSpline : A similar class for two-dimensional spline interpolation
 
-
+    Notes
+    -----
+    The number of data points must be larger than the spline degree `k`.
 
     Examples
     --------
@@ -412,8 +421,8 @@ class LSQUnivariateSpline(UnivariateSpline):
         t = concatenate(([xb]*(k+1),t,[xe]*(k+1)))
         n = len(t)
         if not alltrue(t[k+1:n-k]-t[k:n-k-1] > 0,axis=0):
-            raise ValueError,\
-                  'Interior knots t must satisfy Schoenberg-Whitney conditions'
+            raise ValueError('Interior knots t must satisfy '
+                            'Schoenberg-Whitney conditions')
         data = dfitpack.fpcurfm1(x,y,k,t,w=w,xb=xb,xe=xe)
         self._data = data[:-3] + (None,None,data[-1])
         self._reset_class()
@@ -468,14 +477,14 @@ class BivariateSpline(object):
     [xb,xe] x [yb, ye] calculated from a given set of data points
     (x,y,z).
 
-    See also:
-
-    bisplrep, bisplev - an older wrapping of FITPACK
-    UnivariateSpline - a similar class for univariate spline interpolation
-    SmoothUnivariateSpline - to create a BivariateSpline through the
-                             given points
-    LSQUnivariateSpline - to create a BivariateSpline using weighted
-                          least-squares fitting
+    See Also
+    --------
+    bisplrep, bisplev : an older wrapping of FITPACK
+    UnivariateSpline : a similar class for univariate spline interpolation
+    SmoothUnivariateSpline :
+        to create a BivariateSpline through the given points
+    LSQUnivariateSpline :
+        to create a BivariateSpline using weighted least-squares fitting
     """
 
     def get_residual(self):
@@ -483,6 +492,7 @@ class BivariateSpline(object):
         approximation: sum ((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0)
         """
         return self.fp
+
     def get_knots(self):
         """ Return a tuple (tx,ty) where tx,ty contain knots positions
         of the spline with respect to x-, y-variable, respectively.
@@ -490,18 +500,27 @@ class BivariateSpline(object):
           t[k+1:-k-1] and t[:k+1]=b, t[-k-1:]=e, respectively.
         """
         return self.tck[:2]
+
     def get_coeffs(self):
         """ Return spline coefficients."""
         return self.tck[2]
-    def __call__(self,x,y,mth='array'):
+
+    def __call__(self, x, y, mth='array'):
         """ Evaluate spline at positions x,y."""
+        x = np.asarray(x)
+        y = np.asarray(y)
+        # empty input yields empty output
+        if (x.size == 0) and (y.size == 0):
+            return array([])
+
         if mth=='array':
             tx,ty,c = self.tck[:3]
             kx,ky = self.degrees
             z,ier = dfitpack.bispev(tx,ty,c,kx,ky,x,y)
-            assert ier==0,'Invalid input: ier='+`ier`
+            if not ier == 0:
+                raise ValueError("Error code returned by bispev: %s" % ier)
             return z
-        raise NotImplementedError
+        raise NotImplementedError('unknown method mth=%s' % mth)
 
     def ev(self, xi, yi):
         """
@@ -510,7 +529,8 @@ class BivariateSpline(object):
         tx,ty,c = self.tck[:3]
         kx,ky = self.degrees
         zi,ier = dfitpack.bispeu(tx,ty,c,kx,ky,xi,yi)
-        assert ier==0, 'Invalid input: ier='+`ier`
+        if not ier == 0:
+            raise ValueError("Error code returned by bispeu: %s" % ier)
         return zi
 
     def integral(self, xa, xb, ya, yb):
@@ -537,37 +557,42 @@ class BivariateSpline(object):
 class SmoothBivariateSpline(BivariateSpline):
     """ Smooth bivariate spline approximation.
 
-    See also:
+    Parameters
+    ----------
+    x, y, z : array_like
+        1-D sequences of data points (order is not important).
+    w : array_lie, optional
+        Positive 1-D sequence of weights.
+    bbox : array_like, optional
+        Sequence of length 4 specifying the boundary of the rectangular
+        approximation domain.  By default,
+        ``bbox=[min(x,tx),max(x,tx), min(y,ty),max(y,ty)]``.
+    kx, ky : ints, optional
+        Degrees of the bivariate spline. Default is 3.
+    s : float, optional
+        Positive smoothing factor defined for estimation condition:
+        ``sum((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0) <= s``
+        Default ``s=len(w)`` which should be a good value if 1/w[i] is an
+        estimate of the standard deviation of z[i].
+    eps : float, optional
+        A threshold for determining the effective rank of an over-determined
+        linear system of equations. `eps` should have a value between 0 and 1,
+        the default is 1e-16.
+
+    Notes
+    -----
+    The length of `x`, `y` and `z` should be at least ``(kx+1) * (ky+1)``.
+
+    See Also
+    --------
+    bisplrep, bisplev : an older wrapping of FITPACK
+    UnivariateSpline : a similar class for univariate spline interpolation
+    LSQUnivariateSpline : to create a BivariateSpline using weighted
 
-    bisplrep, bisplev - an older wrapping of FITPACK
-    UnivariateSpline - a similar class for univariate spline interpolation
-    LSQUnivariateSpline - to create a BivariateSpline using weighted
-                          least-squares fitting
     """
 
-    def __init__(self, x, y, z, w=None,
-                 bbox = [None]*4, kx=3, ky=3, s=None, eps=None):
-        """
-        Input:
-          x,y,z  - 1-d sequences of data points (order is not
-                   important)
-        Optional input:
-          w          - positive 1-d sequence of weights
-          bbox       - 4-sequence specifying the boundary of
-                       the rectangular approximation domain.
-                       By default, bbox=[min(x,tx),max(x,tx),
-                                         min(y,ty),max(y,ty)]
-          kx,ky=3,3  - degrees of the bivariate spline.
-          s          - positive smoothing factor defined for
-                       estimation condition:
-                         sum((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0) <= s
-                       Default s=len(w) which should be a good value
-                       if 1/w[i] is an estimate of the standard
-                       deviation of z[i].
-          eps        - a threshold for determining the effective rank
-                       of an over-determined linear system of
-                       equations. 0 < eps < 1, default is 1e-16.
-        """
+    def __init__(self, x, y, z, w=None, bbox = [None]*4, kx=3, ky=3, s=None,
+                 eps=None):
         xb,xe,yb,ye = bbox
         nx,tx,ny,ty,c,fp,wrk1,ier = dfitpack.surfit_smth(x,y,z,w,
                                                          xb,xe,yb,ye,
@@ -584,35 +609,45 @@ class SmoothBivariateSpline(BivariateSpline):
         self.degrees = kx,ky
 
 class LSQBivariateSpline(BivariateSpline):
-    """ Weighted least-squares spline approximation.
-    See also:
+    """ Weighted least-squares bivariate spline approximation.
 
-    bisplrep, bisplev - an older wrapping of FITPACK
-    UnivariateSpline - a similar class for univariate spline interpolation
-    SmoothUnivariateSpline - to create a BivariateSpline through the
-                             given points
+    Parameters
+    ----------
+    x, y, z : array_like
+        1-D sequences of data points (order is not important).
+    tx, ty : array_like
+        Strictly ordered 1-D sequences of knots coordinates.
+    w : array_lie, optional
+        Positive 1-D sequence of weights.
+    bbox : array_like, optional
+        Sequence of length 4 specifying the boundary of the rectangular
+        approximation domain.  By default,
+        ``bbox=[min(x,tx),max(x,tx), min(y,ty),max(y,ty)]``.
+    kx, ky : ints, optional
+        Degrees of the bivariate spline. Default is 3.
+    s : float, optional
+        Positive smoothing factor defined for estimation condition:
+        ``sum((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0) <= s``
+        Default ``s=len(w)`` which should be a good value if 1/w[i] is an
+        estimate of the standard deviation of z[i].
+    eps : float, optional
+        A threshold for determining the effective rank of an over-determined
+        linear system of equations. `eps` should have a value between 0 and 1,
+        the default is 1e-16.
+
+    Notes
+    -----
+    The length of `x`, `y` and `z` should be at least ``(kx+1) * (ky+1)``.
+
+    See Also
+    --------
+    bisplrep, bisplev : an older wrapping of FITPACK
+    UnivariateSpline : a similar class for univariate spline interpolation
+    SmoothUnivariateSpline : To create a BivariateSpline through the given points
     """
 
-    def __init__(self, x, y, z, tx, ty, w=None,
-                 bbox = [None]*4,
-                 kx=3, ky=3, eps=None):
-        """
-        Input:
-          x,y,z  - 1-d sequences of data points (order is not
-                   important)
-          tx,ty  - strictly ordered 1-d sequences of knots
-                   coordinates.
-        Optional input:
-          w          - positive 1-d sequence of weights
-          bbox       - 4-sequence specifying the boundary of
-                       the rectangular approximation domain.
-                       By default, bbox=[min(x,tx),max(x,tx),
-                                         min(y,ty),max(y,ty)]
-          kx,ky=3,3  - degrees of the bivariate spline.
-          eps        - a threshold for determining the effective rank
-                       of an over-determined linear system of
-                       equations. 0 < eps < 1, default is 1e-16.
-        """
+    def __init__(self, x, y, z, tx, ty, w=None, bbox=[None]*4, kx=3, ky=3,
+                 eps=None):
         nx = 2*kx+2+len(tx)
         ny = 2*ky+2+len(ty)
         tx1 = zeros((nx,),float)
@@ -644,47 +679,48 @@ class LSQBivariateSpline(BivariateSpline):
 class RectBivariateSpline(BivariateSpline):
     """ Bivariate spline approximation over a rectangular mesh.
 
-    Can be used for both smoothing or interpolating data.
+    Can be used for both smoothing and interpolating data.
 
-    See also:
+    Parameters
+    ----------
+    x,y : array_like
+        1-D arrays of coordinates in strictly ascending order.
+    z : array_like
+        2-D array of data with shape (x.size,y.size).
+    bbox : array_like, optional
+        Sequence of length 4 specifying the boundary of the rectangular
+        approximation domain.  By default,
+        ``bbox=[min(x,tx),max(x,tx), min(y,ty),max(y,ty)]``.
+    kx, ky : ints, optional
+        Degrees of the bivariate spline. Default is 3.
+    s : float, optional
+        Positive smoothing factor defined for estimation condition:
+        ``sum((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0) <= s``
+        Default is s=0, which is for interpolation.
 
-    SmoothBivariateSpline - a smoothing bivariate spline for scattered data
-    bisplrep, bisplev - an older wrapping of FITPACK
-    UnivariateSpline - a similar class for univariate spline interpolation
+    See Also
+    --------
+    SmoothBivariateSpline : a smoothing bivariate spline for scattered data
+    bisplrep, bisplev : an older wrapping of FITPACK
+    UnivariateSpline : a similar class for univariate spline interpolation
     """
 
-    def __init__(self, x, y, z,
-                 bbox = [None]*4, kx=3, ky=3, s=0):
-        """
-        Input:
-          x,y  - 1-d sequences of coordinates in strictly ascending order
-            z  - 2-d array of data with shape (x.size,y.size)
-        Optional input:
-          bbox       - 4-sequence specifying the boundary of
-                       the rectangular approximation domain.
-                       By default, bbox=[min(x,tx),max(x,tx),
-                                         min(y,ty),max(y,ty)]
-          kx,ky=3,3  - degrees of the bivariate spline.
-          s          - positive smoothing factor defined for
-                       estimation condition:
-                         sum((w[i]*(z[i]-s(x[i],y[i])))**2,axis=0) <= s
-                       Default s=0 which is for interpolation
-        """
+    def __init__(self, x, y, z, bbox = [None]*4, kx=3, ky=3, s=0):
         x,y = ravel(x),ravel(y)
         if not all(diff(x) > 0.0):
-            raise TypeError,'x must be strictly increasing'
+            raise TypeError('x must be strictly increasing')
         if not all(diff(y) > 0.0):
-            raise TypeError,'y must be strictly increasing'
+            raise TypeError('y must be strictly increasing')
         if not ((x.min() == x[0]) and (x.max() == x[-1])):
-            raise TypeError, 'x must be strictly ascending'
+            raise TypeError('x must be strictly ascending')
         if not ((y.min() == y[0]) and (y.max() == y[-1])):
-            raise TypeError, 'y must be strictly ascending'
+            raise TypeError('y must be strictly ascending')
         if not x.size == z.shape[0]:
-            raise TypeError,\
-                  'x dimension of z must have same number of elements as x'
+            raise TypeError('x dimension of z must have same number of '
+                            'elements as x')
         if not y.size == z.shape[1]:
-            raise TypeError,\
-                  'y dimension of z must have same number of elements as y'
+            raise TypeError('y dimension of z must have same number of '
+                            'elements as y')
         z = ravel(z)
         xb,xe,yb,ye = bbox
         nx,tx,ny,ty,c,fp,ier = dfitpack.regrid_smth(x,y,z,
