@@ -91,7 +91,11 @@ def expm2(A):
         t = 'd'
     s,vr = eig(A)
     vri = inv(vr)
-    return dot(dot(vr,diag(exp(s))),vri).astype(t)
+    r = dot(dot(vr,diag(exp(s))),vri)
+    if t in ['f', 'd']:
+        return r.real.astype(t)
+    else:
+        return r.astype(t)
 
 def expm3(A, q=20):
     """Compute the matrix exponential using Taylor series.
@@ -302,7 +306,7 @@ def funm(A, func, disp=True):
     # Perform Shur decomposition (lapack ?gees)
     A = asarray(A)
     if len(A.shape)!=2:
-        raise ValueError, "Non-matrix input to matrix function."
+        raise ValueError("Non-matrix input to matrix function.")
     if A.dtype.char in ['F', 'D', 'G']:
         cmplx_type = 1
     else:
@@ -502,7 +506,7 @@ def sqrtm(A, disp=True):
     """
     A = asarray(A)
     if len(A.shape)!=2:
-        raise ValueError, "Non-matrix input to matrix function."
+        raise ValueError("Non-matrix input to matrix function.")
     T, Z = schur(A)
     T, Z = rsf2csf(T,Z)
     n,n = T.shape
