@@ -1,7 +1,3 @@
-#!/usr/bin/env python
-#
-# Created by: Pearu Peterson, March 2002
-#
 """ Test functions for linalg.decomp module
 
 """
@@ -16,11 +12,11 @@ Run tests if linalg is not installed:
 
 import numpy as np
 from numpy.testing import TestCase, assert_equal, assert_array_almost_equal, \
-        assert_array_equal, assert_raises, run_module_suite, dec
+        assert_array_equal, assert_raises, assert_, run_module_suite, dec
 
 from scipy.linalg import eig, eigvals, lu, svd, svdvals, cholesky, qr, \
      schur, rsf2csf, lu_solve, lu_factor, solve, diagsvd, hessenberg, rq, \
-     eig_banded, eigvals_banded, eigh
+     eig_banded, eigvals_banded, eigh, eigvalsh, LinAlgError
 from scipy.linalg.flapack import dgbtrf, dgbtrs, zgbtrf, zgbtrs, \
      dsbev, dsbevd, dsbevx, zhbevd, zhbevx
 
@@ -30,7 +26,9 @@ from numpy import array, transpose, sometrue, diag, ones, linalg, \
      asarray, matrix, isfinite, all, ndarray, outer, eye, dtype, empty,\
      triu, tril
 
-from numpy.random import rand, normal
+from numpy.random import rand, normal, seed
+
+from scipy.linalg._testutils import assert_no_overwrite
 
 # digit precision to use in asserts for different types
 DIGITS = {'d':11, 'D':11, 'f':4, 'F':4}
@@ -47,7 +45,7 @@ def assert_dtype_equal(act, des):
     else:
         des = dtype(des)
 
-    assert act == des, 'dtype mismatch: "%s" (should be "%s") '%(act, des)
+    assert_(act == des, 'dtype mismatch: "%s" (should be "%s") ' % (act, des))
 
 # XXX: This function should not be defined here, but somewhere in
 #      scipy.linalg namespace
@@ -132,7 +130,7 @@ class TestEigVals(TestCase):
         assert_array_almost_equal(w,exact_w)
 
 
-class TestEig(TestCase):
+class TestEig(object):
 
     def test_simple(self):
         a = [[1,2,3],[1,2,3],[2,5,6]]
@@ -154,6 +152,16 @@ class TestEig(TestCase):
         for i in range(3):
             assert_array_almost_equal(dot(transpose(a),v[:,i]),w[i]*v[:,i])
 
+    def test_simple_complex_eig(self):
+        a = [[1,2],[-2,1]]
+        w,vl,vr = eig(a,left=1,right=1)
+        assert_array_almost_equal(w, array([1+2j, 1-2j]))
+        for i in range(2):
+            assert_array_almost_equal(dot(a,vr[:,i]),w[i]*vr[:,i])
+        for i in range(2):
+            assert_array_almost_equal(dot(conjugate(transpose(a)),vl[:,i]),
+                                      conjugate(w[i])*vl[:,i])
+
     def test_simple_complex(self):
         a = [[1,2,3],[1,2,3],[2,5,6+1j]]
         w,vl,vr = eig(a,left=1,right=1)
@@ -163,16 +171,9 @@ class TestEig(TestCase):
             assert_array_almost_equal(dot(conjugate(transpose(a)),vl[:,i]),
                                       conjugate(w[i])*vl[:,i])
 
-    def test_singular(self):
-        """Test singular pair"""
-        # Example taken from
-        # http://www.cs.umu.se/research/nla/singular_pairs/guptri/matlab.html
-        A = array(( [22,34,31,31,17], [45,45,42,19,29], [39,47,49,26,34],
-            [27,31,26,21,15], [38,44,44,24,30]))
-
-        B = array(( [13,26,25,17,24], [31,46,40,26,37], [26,40,19,25,25],
-            [16,25,27,14,23], [24,35,18,21,22]))
-
+    def _check_gen_eig(self, A, B):
+        A, B = asarray(A), asarray(B)
+        msg = "\n%r\n%r" % (A, B)
         w, vr = eig(A,B)
         wt = eigvals(A,B)
         val1 = dot(A, vr)
@@ -180,11 +181,25 @@ class TestEig(TestCase):
         res = val1 - val2
         for i in range(res.shape[1]):
             if all(isfinite(res[:, i])):
-                assert_array_almost_equal(res[:, i], 0)
+                assert_array_almost_equal(res[:, i], 0, err_msg=msg)
 
-        # Disable this test, which fails now, and is not really necessary if the above
-        # succeeds ?
-        #assert_array_almost_equal(w[isfinite(w)], wt[isfinite(w)])
+        assert_array_almost_equal(sort(w[isfinite(w)]), sort(wt[isfinite(wt)]),
+                                  err_msg=msg)
+
+    def test_singular(self):
+        """Test singular pair"""
+        # Example taken from
+        # http://www.cs.umu.se/research/nla/singular_pairs/guptri/matlab.html
+        A = array(( [22,34,31,31,17], [45,45,42,19,29], [39,47,49,26,34],
+            [27,31,26,21,15], [38,44,44,24,30]))
+        B = array(( [13,26,25,17,24], [31,46,40,26,37], [26,40,19,25,25],
+            [16,25,27,14,23], [24,35,18,21,22]))
+
+        olderr = np.seterr(all='ignore')
+        try:
+            self._check_gen_eig(A, B)
+        finally:
+            np.seterr(**olderr)
 
     def test_falker(self):
         """Test matrices giving some Nan generalized eigen values."""
@@ -195,16 +210,38 @@ class TestEig(TestCase):
         I = identity(3)
         A = bmat([[I,Z],[Z,-K]])
         B = bmat([[Z,I],[M,D]])
-        A = asarray(A)
-        B = asarray(B)
 
-        w, vr = eig(A,B)
-        val1 = dot(A, vr)
-        val2 = dot(B, vr) * w
-        res = val1 - val2
-        for i in range(res.shape[1]):
-            if all(isfinite(res[:, i])):
-                assert_array_almost_equal(res[:, i], 0)
+        olderr = np.seterr(all='ignore')
+        try:
+            self._check_gen_eig(A, B)
+        finally:
+            np.seterr(**olderr)
+
+    def test_bad_geneig(self):
+        # Ticket #709 (strange return values from DGGEV)
+
+        def matrices(omega):
+            c1 = -9 + omega**2
+            c2 = 2*omega
+            A = [[1, 0,  0,  0],
+                 [0, 1,  0,  0],
+                 [0, 0,  c1, 0],
+                 [0, 0,  0, c1]]
+            B = [[0, 0,  1,   0],
+                 [0, 0,  0,   1],
+                 [1, 0,  0, -c2],
+                 [0, 1, c2,   0]]
+            return A, B
+
+        # With a buggy LAPACK, this can fail for different omega on different
+        # machines -- so we need to test several values
+        olderr = np.seterr(all='ignore')
+        try:
+            for k in xrange(100):
+                A, B = matrices(omega=k*5./100)
+                self._check_gen_eig(A, B)
+        finally:
+            np.seterr(**olderr)
 
     def test_not_square_error(self):
         """Check that passing a non-square array raises a ValueError."""
@@ -644,6 +681,15 @@ class TestLU(TestCase):
         """Check lu decomposition on medium size, rectangular matrix."""
         self._test_common(self.cmed)
 
+    def test_simple_known(self):
+        # Ticket #1458
+        for order in ['C', 'F']:
+            A = np.array([[2, 1],[0, 1.]], order=order)
+            LU, P = lu_factor(A)
+            assert_array_almost_equal(LU, np.array([[2, 1], [0, 1]]))
+            assert_array_equal(P, np.array([0, 1]))
+
+
 class TestLUSingle(TestLU):
     """LU testers for single precision, real and double"""
     def __init__(self, *args, **kw):
@@ -664,127 +710,141 @@ class TestLUSingle(TestLU):
         self.cmed = self.vrect.astype(complex64)
 
 class TestLUSolve(TestCase):
+    def setUp(self):
+        seed(1234)
+
     def test_lu(self):
-        a = random((10,10))
+        a0 = random((10,10))
         b = random((10,))
 
-        x1 = solve(a,b)
+        for order in ['C', 'F']:
+            a = np.array(a0, order=order)
 
-        lu_a = lu_factor(a)
-        x2 = lu_solve(lu_a,b)
+            x1 = solve(a,b)
 
-        assert_array_equal(x1,x2)
+            lu_a = lu_factor(a)
+            x2 = lu_solve(lu_a,b)
+
+            assert_array_almost_equal(x1,x2)
 
 class TestSVD(TestCase):
+    def setUp(self):
+        seed(1234)
 
     def test_simple(self):
         a = [[1,2,3],[1,20,3],[2,5,6]]
-        u,s,vh = svd(a)
-        assert_array_almost_equal(dot(transpose(u),u),identity(3))
-        assert_array_almost_equal(dot(transpose(vh),vh),identity(3))
-        sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-        for i in range(len(s)): sigma[i,i] = s[i]
-        assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+        for full_matrices in (True, False):
+            u,s,vh = svd(a, full_matrices=full_matrices)
+            assert_array_almost_equal(dot(transpose(u),u),identity(3))
+            assert_array_almost_equal(dot(transpose(vh),vh),identity(3))
+            sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
+            for i in range(len(s)): sigma[i,i] = s[i]
+            assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_simple_singular(self):
         a = [[1,2,3],[1,2,3],[2,5,6]]
-        u,s,vh = svd(a)
-        assert_array_almost_equal(dot(transpose(u),u),identity(3))
-        assert_array_almost_equal(dot(transpose(vh),vh),identity(3))
-        sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-        for i in range(len(s)): sigma[i,i] = s[i]
-        assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+        for full_matrices in (True, False):
+            u,s,vh = svd(a, full_matrices=full_matrices)
+            assert_array_almost_equal(dot(transpose(u),u),identity(3))
+            assert_array_almost_equal(dot(transpose(vh),vh),identity(3))
+            sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
+            for i in range(len(s)): sigma[i,i] = s[i]
+            assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_simple_underdet(self):
         a = [[1,2,3],[4,5,6]]
-        u,s,vh = svd(a)
-        assert_array_almost_equal(dot(transpose(u),u),identity(2))
-        assert_array_almost_equal(dot(transpose(vh),vh),identity(3))
-        sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-        for i in range(len(s)): sigma[i,i] = s[i]
-        assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+        for full_matrices in (True, False):
+            u,s,vh = svd(a, full_matrices=full_matrices)
+            assert_array_almost_equal(dot(transpose(u),u),identity(u.shape[0]))
+            sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
+            for i in range(len(s)): sigma[i,i] = s[i]
+            assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_simple_overdet(self):
         a = [[1,2],[4,5],[3,4]]
-        u,s,vh = svd(a)
-        assert_array_almost_equal(dot(transpose(u),u),identity(3))
-        assert_array_almost_equal(dot(transpose(vh),vh),identity(2))
-        sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-        for i in range(len(s)): sigma[i,i] = s[i]
-        assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+        for full_matrices in (True, False):
+            u,s,vh = svd(a, full_matrices=full_matrices)
+            assert_array_almost_equal(dot(transpose(u),u), identity(u.shape[1]))
+            assert_array_almost_equal(dot(transpose(vh),vh),identity(2))
+            sigma = zeros((u.shape[1],vh.shape[0]),s.dtype.char)
+            for i in range(len(s)): sigma[i,i] = s[i]
+            assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_random(self):
         n = 20
         m = 15
         for i in range(3):
             for a in [random([n,m]),random([m,n])]:
-                u,s,vh = svd(a)
-                assert_array_almost_equal(dot(transpose(u),u),identity(len(u)))
-                assert_array_almost_equal(dot(transpose(vh),vh),identity(len(vh)))
-                sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-                for i in range(len(s)): sigma[i,i] = s[i]
-                assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+                for full_matrices in (True, False):
+                    u,s,vh = svd(a, full_matrices=full_matrices)
+                    assert_array_almost_equal(dot(transpose(u),u),identity(u.shape[1]))
+                    assert_array_almost_equal(dot(vh, transpose(vh)),identity(vh.shape[0]))
+                    sigma = zeros((u.shape[1],vh.shape[0]),s.dtype.char)
+                    for i in range(len(s)): sigma[i,i] = s[i]
+                    assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_simple_complex(self):
         a = [[1,2,3],[1,2j,3],[2,5,6]]
-        u,s,vh = svd(a)
-        assert_array_almost_equal(dot(conj(transpose(u)),u),identity(3))
-        assert_array_almost_equal(dot(conj(transpose(vh)),vh),identity(3))
-        sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-        for i in range(len(s)): sigma[i,i] = s[i]
-        assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+        for full_matrices in (True, False):
+            u,s,vh = svd(a, full_matrices=full_matrices)
+            assert_array_almost_equal(dot(conj(transpose(u)),u),identity(u.shape[1]))
+            assert_array_almost_equal(dot(conj(transpose(vh)),vh),identity(vh.shape[0]))
+            sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
+            for i in range(len(s)): sigma[i,i] = s[i]
+            assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
     def test_random_complex(self):
         n = 20
         m = 15
         for i in range(3):
-            for a in [random([n,m]),random([m,n])]:
-                a = a + 1j*random(list(a.shape))
-                u,s,vh = svd(a)
-                assert_array_almost_equal(dot(conj(transpose(u)),u),identity(len(u)))
-                # This fails when [m,n]
-                #assert_array_almost_equal(dot(conj(transpose(vh)),vh),identity(len(vh),dtype=vh.dtype.char))
-                sigma = zeros((u.shape[0],vh.shape[0]),s.dtype.char)
-                for i in range(len(s)): sigma[i,i] = s[i]
-                assert_array_almost_equal(dot(dot(u,sigma),vh),a)
+            for full_matrices in (True, False):
+                for a in [random([n,m]),random([m,n])]:
+                    a = a + 1j*random(list(a.shape))
+                    u,s,vh = svd(a, full_matrices=full_matrices)
+                    assert_array_almost_equal(dot(conj(transpose(u)),u),identity(u.shape[1]))
+                    # This fails when [m,n]
+                    #assert_array_almost_equal(dot(conj(transpose(vh)),vh),identity(len(vh),dtype=vh.dtype.char))
+                    sigma = zeros((u.shape[1],vh.shape[0]),s.dtype.char)
+                    for i in range(len(s)): sigma[i,i] = s[i]
+                    assert_array_almost_equal(dot(dot(u,sigma),vh),a)
 
 class TestSVDVals(TestCase):
 
     def test_simple(self):
         a = [[1,2,3],[1,2,3],[2,5,6]]
         s = svdvals(a)
-        assert len(s)==3
-        assert s[0]>=s[1]>=s[2]
+        assert_(len(s) == 3)
+        assert_(s[0] >= s[1] >= s[2])
 
     def test_simple_underdet(self):
         a = [[1,2,3],[4,5,6]]
         s = svdvals(a)
-        assert len(s)==2
-        assert s[0]>=s[1]
+        assert_(len(s) == 2)
+        assert_(s[0] >= s[1])
 
     def test_simple_overdet(self):
         a = [[1,2],[4,5],[3,4]]
         s = svdvals(a)
-        assert len(s)==2
-        assert s[0]>=s[1]
+        assert_(len(s) == 2)
+        assert_(s[0] >= s[1])
 
     def test_simple_complex(self):
         a = [[1,2,3],[1,20,3j],[2,5,6]]
         s = svdvals(a)
-        assert len(s)==3
-        assert s[0]>=s[1]>=s[2]
+        assert_(len(s) == 3)
+        assert_(s[0] >= s[1] >= s[2])
 
     def test_simple_underdet_complex(self):
         a = [[1,2,3],[4,5j,6]]
         s = svdvals(a)
-        assert len(s)==2
-        assert s[0]>=s[1]
+        assert_(len(s) == 2)
+        assert_(s[0] >= s[1])
 
     def test_simple_overdet_complex(self):
         a = [[1,2],[4,5],[3j,4]]
         s = svdvals(a)
-        assert len(s)==2
-        assert s[0]>=s[1]
+        assert_(len(s) == 2)
+        assert_(s[0] >= s[1])
 
 class TestDiagSVD(TestCase):
 
@@ -794,18 +854,43 @@ class TestDiagSVD(TestCase):
 
 class TestQR(TestCase):
 
+    def setUp(self):
+        seed(1234)
+
     def test_simple(self):
         a = [[8,2,3],[2,9,3],[5,3,6]]
         q,r = qr(a)
         assert_array_almost_equal(dot(transpose(q),q),identity(3))
         assert_array_almost_equal(dot(q,r),a)
 
+    def test_simple_pivoting(self):
+        a = np.asarray([[8,2,3],[2,9,3],[5,3,6]])
+        q,r,p = qr(a, pivoting=True)
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(3))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        q2,r2 = qr(a[:,p])
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
     def test_simple_trap(self):
         a = [[8,2,3],[2,9,3]]
         q,r = qr(a)
         assert_array_almost_equal(dot(transpose(q),q),identity(2))
         assert_array_almost_equal(dot(q,r),a)
 
+    def test_simple_trap_pivoting(self):
+        a = np.asarray([[8,2,3],[2,9,3]])
+        q,r,p = qr(a, pivoting=True)
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        q2,r2 = qr(a[:,p])
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
     def test_simple_tall(self):
         # full version
         a = [[8,2],[2,9],[5,3]]
@@ -813,21 +898,102 @@ class TestQR(TestCase):
         assert_array_almost_equal(dot(transpose(q),q),identity(3))
         assert_array_almost_equal(dot(q,r),a)
 
+    def test_simple_tall_pivoting(self):
+        # full version pivoting
+        a = np.asarray([[8,2],[2,9],[5,3]])
+        q,r,p = qr(a, pivoting=True)
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(3))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        q2,r2 = qr(a[:,p])
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
     def test_simple_tall_e(self):
         # economy version
         a = [[8,2],[2,9],[5,3]]
-        q,r = qr(a,econ=True)
+        q,r = qr(a, mode='economic')
         assert_array_almost_equal(dot(transpose(q),q),identity(2))
         assert_array_almost_equal(dot(q,r),a)
         assert_equal(q.shape, (3,2))
         assert_equal(r.shape, (2,2))
 
+    def test_simple_tall_e_pivoting(self):
+        # economy version pivoting
+        a = np.asarray([[8,2],[2,9],[5,3]])
+        q,r,p = qr(a, pivoting=True, mode='economic')
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        q2,r2 = qr(a[:,p], mode='economic')
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
+    def test_simple_fat(self):
+        # full version
+        a = [[8,2,5],[2,9,3]]
+        q,r = qr(a)
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a)
+        assert_equal(q.shape, (2,2))
+        assert_equal(r.shape, (2,3))
+
+    def test_simple_fat_pivoting(self):
+        # full version pivoting
+        a = np.asarray([[8,2,5],[2,9,3]])
+        q,r,p = qr(a, pivoting=True)
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        assert_equal(q.shape, (2,2))
+        assert_equal(r.shape, (2,3))
+        q2,r2 = qr(a[:,p])
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
+    def test_simple_fat_e(self):
+        # economy version
+        a = [[8,2,3],[2,9,5]]
+        q,r = qr(a, mode='economic')
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a)
+        assert_equal(q.shape, (2,2))
+        assert_equal(r.shape, (2,3))
+
+    def test_simple_fat_e_pivoting(self):
+        # economy version pivoting
+        a = np.asarray([[8,2,3],[2,9,5]])
+        q,r,p = qr(a, pivoting=True, mode='economic')
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        assert_equal(q.shape, (2,2))
+        assert_equal(r.shape, (2,3))
+        q2,r2 = qr(a[:,p], mode='economic')
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
     def test_simple_complex(self):
         a = [[3,3+4j,5],[5,2,2+7j],[3,2,7]]
         q,r = qr(a)
         assert_array_almost_equal(dot(conj(transpose(q)),q),identity(3))
         assert_array_almost_equal(dot(q,r),a)
 
+    def test_simple_complex_pivoting(self):
+        a = np.asarray([[3,3+4j,5],[5,2,2+7j],[3,2,7]])
+        q,r,p = qr(a, pivoting=True)
+        d = abs(diag(r))
+        assert_(all(d[1:] <= d[:-1]))
+        assert_array_almost_equal(dot(conj(transpose(q)),q),identity(3))
+        assert_array_almost_equal(dot(q,r),a[:,p])
+        q2,r2 = qr(a[:,p])
+        assert_array_almost_equal(q,q2)
+        assert_array_almost_equal(r,r2)
+
     def test_random(self):
         n = 20
         for k in range(2):
@@ -836,6 +1002,19 @@ class TestQR(TestCase):
             assert_array_almost_equal(dot(transpose(q),q),identity(n))
             assert_array_almost_equal(dot(q,r),a)
 
+    def test_random_pivoting(self):
+        n = 20
+        for k in range(2):
+            a = random([n,n])
+            q,r,p = qr(a, pivoting=True)
+            d = abs(diag(r))
+            assert_(all(d[1:] <= d[:-1]))
+            assert_array_almost_equal(dot(transpose(q),q),identity(n))
+            assert_array_almost_equal(dot(q,r),a[:,p])
+            q2,r2 = qr(a[:,p])
+            assert_array_almost_equal(q,q2)
+            assert_array_almost_equal(r,r2)
+
     def test_random_tall(self):
         # full version
         m = 200
@@ -846,18 +1025,50 @@ class TestQR(TestCase):
             assert_array_almost_equal(dot(transpose(q),q),identity(m))
             assert_array_almost_equal(dot(q,r),a)
 
+    def test_random_tall_pivoting(self):
+        # full version pivoting
+        m = 200
+        n = 100
+        for k in range(2):
+            a = random([m,n])
+            q,r,p = qr(a, pivoting=True)
+            d = abs(diag(r))
+            assert_(all(d[1:] <= d[:-1]))
+            assert_array_almost_equal(dot(transpose(q),q),identity(m))
+            assert_array_almost_equal(dot(q,r),a[:,p])
+            q2,r2 = qr(a[:,p])
+            assert_array_almost_equal(q,q2)
+            assert_array_almost_equal(r,r2)
+
     def test_random_tall_e(self):
         # economy version
         m = 200
         n = 100
         for k in range(2):
             a = random([m,n])
-            q,r = qr(a,econ=True)
+            q,r = qr(a, mode='economic')
             assert_array_almost_equal(dot(transpose(q),q),identity(n))
             assert_array_almost_equal(dot(q,r),a)
             assert_equal(q.shape, (m,n))
             assert_equal(r.shape, (n,n))
 
+    def test_random_tall_e_pivoting(self):
+        # economy version pivoting
+        m = 200
+        n = 100
+        for k in range(2):
+            a = random([m,n])
+            q,r,p = qr(a, pivoting=True, mode='economic')
+            d = abs(diag(r))
+            assert_(all(d[1:] <= d[:-1]))
+            assert_array_almost_equal(dot(transpose(q),q),identity(n))
+            assert_array_almost_equal(dot(q,r),a[:,p])
+            assert_equal(q.shape, (m,n))
+            assert_equal(r.shape, (n,n))
+            q2,r2 = qr(a[:,p], mode='economic')
+            assert_array_almost_equal(q,q2)
+            assert_array_almost_equal(r,r2)
+
     def test_random_trap(self):
         m = 100
         n = 200
@@ -867,6 +1078,20 @@ class TestQR(TestCase):
             assert_array_almost_equal(dot(transpose(q),q),identity(m))
             assert_array_almost_equal(dot(q,r),a)
 
+    def test_random_trap_pivoting(self):
+        m = 100
+        n = 200
+        for k in range(2):
+            a = random([m,n])
+            q,r,p = qr(a, pivoting=True)
+            d = abs(diag(r))
+            assert_(all(d[1:] <= d[:-1]))
+            assert_array_almost_equal(dot(transpose(q),q),identity(m))
+            assert_array_almost_equal(dot(q,r),a[:,p])
+            q2,r2 = qr(a[:,p])
+            assert_array_almost_equal(q,q2)
+            assert_array_almost_equal(r,r2)
+
     def test_random_complex(self):
         n = 20
         for k in range(2):
@@ -875,67 +1100,115 @@ class TestQR(TestCase):
             assert_array_almost_equal(dot(conj(transpose(q)),q),identity(n))
             assert_array_almost_equal(dot(q,r),a)
 
+    def test_random_complex_pivoting(self):
+        n = 20
+        for k in range(2):
+            a = random([n,n])+1j*random([n,n])
+            q,r,p = qr(a, pivoting=True)
+            d = abs(diag(r))
+            assert_(all(d[1:] <= d[:-1]))
+            assert_array_almost_equal(dot(conj(transpose(q)),q),identity(n))
+            assert_array_almost_equal(dot(q,r),a[:,p])
+            q2,r2 = qr(a[:,p])
+            assert_array_almost_equal(q,q2)
+            assert_array_almost_equal(r,r2)
+
 class TestRQ(TestCase):
 
+    def setUp(self):
+        seed(1234)
+
     def test_simple(self):
         a = [[8,2,3],[2,9,3],[5,3,6]]
         r,q = rq(a)
-        assert_array_almost_equal(dot(transpose(q),q),identity(3))
+        assert_array_almost_equal(dot(q, transpose(q)),identity(3))
         assert_array_almost_equal(dot(r,q),a)
 
+    def test_r(self):
+        a = [[8,2,3],[2,9,3],[5,3,6]]
+        r,q = rq(a)
+        r2 = rq(a, mode='r')
+        assert_array_almost_equal(r, r2)
+
     def test_random(self):
         n = 20
         for k in range(2):
             a = random([n,n])
             r,q = rq(a)
-            assert_array_almost_equal(dot(transpose(q),q),identity(n))
+            assert_array_almost_equal(dot(q, transpose(q)),identity(n))
             assert_array_almost_equal(dot(r,q),a)
 
-# TODO: implement support for non-square and complex arrays
+    def test_simple_trap(self):
+        a = [[8,2,3],[2,9,3]]
+        r,q = rq(a)
+        assert_array_almost_equal(dot(transpose(q),q),identity(3))
+        assert_array_almost_equal(dot(r,q),a)
 
-##    def test_simple_trap(self):
-##        a = [[8,2,3],[2,9,3]]
-##        r,q = rq(a)
-##        assert_array_almost_equal(dot(transpose(q),q),identity(2))
-##        assert_array_almost_equal(dot(r,q),a)
+    def test_simple_tall(self):
+        a = [[8,2],[2,9],[5,3]]
+        r,q = rq(a)
+        assert_array_almost_equal(dot(transpose(q),q),identity(2))
+        assert_array_almost_equal(dot(r,q),a)
 
-##    def test_simple_tall(self):
-##        a = [[8,2],[2,9],[5,3]]
-##        r,q = rq(a)
-##        assert_array_almost_equal(dot(transpose(q),q),identity(3))
-##        assert_array_almost_equal(dot(r,q),a)
+    def test_simple_fat(self):
+        a = [[8,2,5],[2,9,3]]
+        r,q = rq(a)
+        assert_array_almost_equal(dot(transpose(q),q),identity(3))
+        assert_array_almost_equal(dot(r,q),a)
 
-##    def test_simple_complex(self):
-##        a = [[3,3+4j,5],[5,2,2+7j],[3,2,7]]
-##        r,q = rq(a)
-##        assert_array_almost_equal(dot(conj(transpose(q)),q),identity(3))
-##        assert_array_almost_equal(dot(r,q),a)
+    def test_simple_complex(self):
+        a = [[3,3+4j,5],[5,2,2+7j],[3,2,7]]
+        r,q = rq(a)
+        assert_array_almost_equal(dot(q, conj(transpose(q))),identity(3))
+        assert_array_almost_equal(dot(r,q),a)
 
-##    def test_random_tall(self):
-##        m = 200
-##        n = 100
-##        for k in range(2):
-##            a = random([m,n])
-##            r,q = rq(a)
-##            assert_array_almost_equal(dot(transpose(q),q),identity(m))
-##            assert_array_almost_equal(dot(r,q),a)
+    def test_random_tall(self):
+        m = 200
+        n = 100
+        for k in range(2):
+            a = random([m,n])
+            r,q = rq(a)
+            assert_array_almost_equal(dot(q, transpose(q)),identity(n))
+            assert_array_almost_equal(dot(r,q),a)
 
-##    def test_random_trap(self):
-##        m = 100
-##        n = 200
-##        for k in range(2):
-##            a = random([m,n])
-##            r,q = rq(a)
-##            assert_array_almost_equal(dot(transpose(q),q),identity(m))
-##            assert_array_almost_equal(dot(r,q),a)
+    def test_random_trap(self):
+        m = 100
+        n = 200
+        for k in range(2):
+            a = random([m,n])
+            r,q = rq(a)
+            assert_array_almost_equal(dot(q, transpose(q)),identity(n))
+            assert_array_almost_equal(dot(r,q),a)
 
-##    def test_random_complex(self):
-##        n = 20
-##        for k in range(2):
-##            a = random([n,n])+1j*random([n,n])
-##            r,q = rq(a)
-##            assert_array_almost_equal(dot(conj(transpose(q)),q),identity(n))
-##            assert_array_almost_equal(dot(r,q),a)
+    def test_random_trap_economic(self):
+        m = 100
+        n = 200
+        for k in range(2):
+            a = random([m,n])
+            r,q = rq(a, mode='economic')
+            assert_array_almost_equal(dot(q,transpose(q)),identity(m))
+            assert_array_almost_equal(dot(r,q),a)
+            assert_equal(q.shape, (m, n))
+            assert_equal(r.shape, (m, m))
+
+    def test_random_complex(self):
+        n = 20
+        for k in range(2):
+            a = random([n,n])+1j*random([n,n])
+            r,q = rq(a)
+            assert_array_almost_equal(dot(q, conj(transpose(q))),identity(n))
+            assert_array_almost_equal(dot(r,q),a)
+
+    def test_random_complex_economic(self):
+        m = 100
+        n = 200
+        for k in range(2):
+            a = random([m,n])+1j*random([m,n])
+            r,q = rq(a, mode='economic')
+            assert_array_almost_equal(dot(q,conj(transpose(q))),identity(m))
+            assert_array_almost_equal(dot(r,q),a)
+            assert_equal(q.shape, (m, n))
+            assert_equal(r.shape, (m, m))
 
 transp = transpose
 any = sometrue
@@ -947,11 +1220,84 @@ class TestSchur(TestCase):
         t,z = schur(a)
         assert_array_almost_equal(dot(dot(z,t),transp(conj(z))),a)
         tc,zc = schur(a,'complex')
-        assert(any(ravel(iscomplex(zc))) and any(ravel(iscomplex(tc))))
+        assert_(any(ravel(iscomplex(zc))) and any(ravel(iscomplex(tc))))
         assert_array_almost_equal(dot(dot(zc,tc),transp(conj(zc))),a)
         tc2,zc2 = rsf2csf(tc,zc)
         assert_array_almost_equal(dot(dot(zc2,tc2),transp(conj(zc2))),a)
 
+    def test_sort(self):
+        a = [[4.,3.,1.,-1.],[-4.5,-3.5,-1.,1.],[9.,6.,-4.,4.5],[6.,4.,-3.,3.5]]
+        s,u,sdim = schur(a,sort='lhp')
+        assert_array_almost_equal([[0.1134,0.5436,0.8316,0.],
+                                   [-0.1134,-0.8245,0.5544,0.],
+                                   [-0.8213,0.1308,0.0265,-0.5547],
+                                   [-0.5475,0.0872,0.0177,0.8321]],
+                                  u,3)
+        assert_array_almost_equal([[-1.4142,0.1456,-11.5816,-7.7174],
+                                   [0.,-0.5000,9.4472,-0.7184],
+                                   [0.,0.,1.4142,-0.1456],
+                                   [0.,0.,0.,0.5]],
+                                  s,3)
+        assert_equal(2,sdim)
+
+        s,u,sdim = schur(a,sort='rhp')
+        assert_array_almost_equal([[0.4862,-0.4930,0.1434,-0.7071],
+                                   [-0.4862,0.4930,-0.1434,-0.7071],
+                                   [0.6042,0.3944,-0.6924,0.],
+                                   [0.4028,0.5986,0.6924,0.]],
+                                  u,3)
+        assert_array_almost_equal([[1.4142,-0.9270,4.5368,-14.4130],
+                                   [0.,0.5,6.5809,-3.1870],
+                                   [0.,0.,-1.4142,0.9270],
+                                   [0.,0.,0.,-0.5]],
+                                  s,3)
+        assert_equal(2,sdim)
+
+        s,u,sdim = schur(a,sort='iuc')
+        assert_array_almost_equal([[0.5547,0.,-0.5721,-0.6042],
+                                   [-0.8321,0.,-0.3814,-0.4028],
+                                   [0.,0.7071,-0.5134,0.4862],
+                                   [0.,0.7071,0.5134,-0.4862]],
+                                  u,3)
+        assert_array_almost_equal([[-0.5000,0.0000,-6.5809,-4.0974],
+                                   [0.,0.5000,-3.3191,-14.4130],
+                                   [0.,0.,1.4142,2.1573],
+                                   [0.,0.,0.,-1.4142]],
+                                  s,3)
+        assert_equal(2,sdim)
+
+        s,u,sdim = schur(a,sort='ouc')
+        assert_array_almost_equal([[0.4862,-0.5134,0.7071,0.],
+                                   [-0.4862,0.5134,0.7071,0.],
+                                   [0.6042,0.5721,0.,-0.5547],
+                                   [0.4028,0.3814,0.,0.8321]],
+                                  u,3)
+        assert_array_almost_equal([[1.4142,-2.1573,14.4130,4.0974],
+                                   [0.,-1.4142,3.3191,6.5809],
+                                   [0.,0.,-0.5000,0.],
+                                   [0.,0.,0.,0.5000]],
+                                  s,3)
+        assert_equal(2,sdim)
+
+        rhp_function = lambda x: x >= 0.0
+        s,u,sdim = schur(a,sort=rhp_function)
+        assert_array_almost_equal([[0.4862,-0.4930,0.1434,-0.7071],
+                                   [-0.4862,0.4930,-0.1434,-0.7071],
+                                   [0.6042,0.3944,-0.6924,0.],
+                                   [0.4028,0.5986,0.6924,0.]],
+                                  u,3)
+        assert_array_almost_equal([[1.4142,-0.9270,4.5368,-14.4130],
+                                   [0.,0.5,6.5809,-3.1870],
+                                   [0.,0.,-1.4142,0.9270],
+                                   [0.,0.,0.,-0.5]],
+                                  s,3)
+        assert_equal(2,sdim)
+
+    def test_sort_errors(self):
+        a = [[4.,3.,1.,-1.],[-4.5,-3.5,-1.,1.],[9.,6.,-4.,4.5],[6.,4.,-3.,3.5]]
+        assert_raises(ValueError, schur, a, sort='unsupported')
+        assert_raises(ValueError, schur, a, sort=1)
+
 class TestHessenberg(TestCase):
 
     def test_simple(self):
@@ -1001,22 +1347,34 @@ class TestHessenberg(TestCase):
 
 
 
-class TestDataNotShared(TestCase):
+class TestDatacopied(TestCase):
 
-    def test_datanotshared(self):
-        from scipy.linalg.decomp import _datanotshared
+    def test_datacopied(self):
+        from scipy.linalg.decomp import _datacopied
 
         M = matrix([[0,1],[2,3]])
         A = asarray(M)
         L = M.tolist()
         M2 = M.copy()
 
-        assert_equal(_datanotshared(M,M),False)
-        assert_equal(_datanotshared(M,A),False)
+        class Fake1:
+            def __array__(self):
+                return A
 
-        assert_equal(_datanotshared(M,L),True)
-        assert_equal(_datanotshared(M,M2),True)
-        assert_equal(_datanotshared(A,M2),True)
+        class Fake2:
+            __array_interface__ = A.__array_interface__
+
+        F1 = Fake1()
+        F2 = Fake2()
+
+        AF1 = asarray(F1)
+        AF2 = asarray(F2)
+
+        for item, status in [(M, False), (A, False), (L, True),
+                             (M2, False), (F1, False), (F2, False)]:
+            arr = asarray(item)
+            assert_equal(_datacopied(arr, item), status,
+                         err_msg=repr(item))
 
 
 def test_aligned_mem_float():
@@ -1107,5 +1465,45 @@ def test_lapack_misaligned():
 # not properly tested
 # cholesky, rsf2csf, lu_solve, solve, eig_banded, eigvals_banded, eigh, diagsvd
 
+
+class TestOverwrite(object):
+    def test_eig(self):
+        assert_no_overwrite(eig, [(3,3)])
+        assert_no_overwrite(eig, [(3,3), (3,3)])
+    def test_eigh(self):
+        assert_no_overwrite(eigh, [(3,3)])
+        assert_no_overwrite(eigh, [(3,3), (3,3)])
+    def test_eig_banded(self):
+        assert_no_overwrite(eig_banded, [(3,2)])
+    def test_eigvals(self):
+        assert_no_overwrite(eigvals, [(3,3)])
+    def test_eigvalsh(self):
+        assert_no_overwrite(eigvalsh, [(3,3)])
+    def test_eigvals_banded(self):
+        assert_no_overwrite(eigvals_banded, [(3,2)])
+    def test_hessenberg(self):
+        assert_no_overwrite(hessenberg, [(3,3)])
+    def test_lu_factor(self):
+        assert_no_overwrite(lu_factor, [(3,3)])
+    def test_lu_solve(self):
+        x = np.array([[1,2,3], [4,5,6], [7,8,8]])
+        xlu = lu_factor(x)
+        assert_no_overwrite(lambda b: lu_solve(xlu, b), [(3,)])
+    def test_lu(self):
+        assert_no_overwrite(lu, [(3,3)])
+    def test_qr(self):
+        assert_no_overwrite(qr, [(3,3)])
+    def test_rq(self):
+        assert_no_overwrite(rq, [(3,3)])
+    def test_schur(self):
+        assert_no_overwrite(schur, [(3,3)])
+    def test_schur_complex(self):
+        assert_no_overwrite(lambda a: schur(a, 'complex'), [(3,3)],
+                            dtypes=[np.float32, np.float64])
+    def test_svd(self):
+        assert_no_overwrite(svd, [(3,3)])
+    def test_svdvals(self):
+        assert_no_overwrite(svdvals, [(3,3)])
+
 if __name__ == "__main__":
     run_module_suite()
