@@ -1,3 +1,10 @@
+"""
+A collection of image utilities using the Python Imaging Library (PIL).
+
+Note that PIL is not a dependency of SciPy and this module is not
+available on systems that don't have PIL installed.
+
+"""
 # Functions which need the PIL
 
 import numpy
@@ -7,14 +14,59 @@ from numpy import amin, amax, ravel, asarray, cast, arange, \
      ones, newaxis, transpose, mgrid, iscomplexobj, sum, zeros, uint8, \
      issubdtype, array
 
-import Image
-import ImageFilter
+try:
+    from PIL import Image, ImageFilter
+except ImportError:
+    import Image
+    import ImageFilter
+
 
 __all__ = ['fromimage','toimage','imsave','imread','bytescale',
            'imrotate','imresize','imshow','imfilter','radon']
 
+
 # Returns a byte-scaled image
 def bytescale(data, cmin=None, cmax=None, high=255, low=0):
+    """
+    Byte scales an array (image).
+
+    Parameters
+    ----------
+    data : ndarray
+        PIL image data array.
+    cmin :  Scalar
+        Bias scaling of small values, Default is data.min().
+    cmax : scalar
+        Bias scaling of large values, Default is data.max().
+    high : scalar
+        Scale max value to `high`.
+    low : scalar
+        Scale min value to `low`.
+
+    Returns
+    -------
+    img_array : ndarray
+        Bytescaled array.
+
+    Examples
+    --------
+    >>> img = array([[ 91.06794177,   3.39058326,  84.4221549 ],
+                     [ 73.88003259,  80.91433048,   4.88878881],
+                     [ 51.53875334,  34.45808177,  27.5873488 ]])
+    >>> bytescale(img)
+    array([[255,   0, 236],
+           [205, 225,   4],
+           [140,  90,  70]], dtype=uint8)
+    >>> bytescale(img, high=200, low=100)
+    array([[200, 100, 192],
+           [180, 188, 102],
+           [155, 135, 128]], dtype=uint8)
+    >>> bytescale(img, cmin=0, cmax=255)
+    array([[91,  3, 84],
+           [74, 81,  5],
+           [52, 34, 28]], dtype=uint8)
+
+    """
     if data.dtype == uint8:
         return data
     high = high - low
@@ -25,39 +77,84 @@ def bytescale(data, cmin=None, cmax=None, high=255, low=0):
     return bytedata + cast[uint8](low)
 
 def imread(name,flatten=0):
-    """Read an image file from a filename.
+    """
+    Read an image file from a filename.
 
-    Optional arguments:
+    Parameters
+    ----------
+    name : str
+        The file name to be read.
+    flatten : bool, optional
+        If True, flattens the color layers into a single gray-scale layer.
+
+    Returns
+    -------
+    imread : ndarray
+        The array obtained by reading image from file `name`.
+
+    Notes
+    -----
+    The image is flattened by calling convert('F') on
+    the resulting image object.
 
-     - flatten (0): if true, the image is flattened by calling convert('F') on
-     the resulting image object.  This flattens the color layers into a single
-     grayscale layer.
     """
 
     im = Image.open(name)
     return fromimage(im,flatten=flatten)
 
 def imsave(name, arr):
-    """Save an array to an image file.
+    """
+    Save an array as an image.
+
+    Parameters
+    ----------
+    filename : str
+        Output filename.
+    image : ndarray, MxN or MxNx3 or MxNx4
+        Array containing image values.  If the shape is ``MxN``, the array
+        represents a grey-level image.  Shape ``MxNx3`` stores the red, green
+        and blue bands along the last dimension.  An alpha layer may be
+        included, specified as the last colour band of an ``MxNx4`` array.
+
+    Examples
+    --------
+    Construct an array of gradient intensity values and save to file:
+
+    >>> x = np.zeros((255, 255))
+    >>> x = np.zeros((255, 255), dtype=np.uint8)
+    >>> x[:] = np.arange(255)
+    >>> imsave('/tmp/gradient.png', x)
+
+    Construct an array with three colour bands (R, G, B) and store to file:
+
+    >>> rgb = np.zeros((255, 255, 3), dtype=np.uint8)
+    >>> rgb[..., 0] = np.arange(255)
+    >>> rgb[..., 1] = 55
+    >>> rgb[..., 2] = 1 - np.arange(255)
+    >>> imsave('/tmp/rgb_gradient.png', rgb)
+
     """
     im = toimage(arr)
     im.save(name)
     return
 
 def fromimage(im, flatten=0):
-    """Return a copy of a PIL image as a numpy array.
+    """
+    Return a copy of a PIL image as a numpy array.
 
-    :Parameters:
-        im : PIL image
-            Input image.
-        flatten : bool
-            If true, convert the output to grey-scale.
+    Parameters
+    ----------
+    im : PIL image
+        Input image.
+    flatten : bool
+        If true, convert the output to grey-scale.
 
-    :Returns:
-        img_array : ndarray
-            The different colour bands/channels are stored in the
-            third dimension, such that a grey-image is MxN, an
-            RGB-image MxNx3 and an RGBA-image MxNx4.
+    Returns
+    -------
+    fromimage : ndarray
+        The different colour bands/channels are stored in the
+        third dimension, such that a grey-image is MxN, an
+        RGB-image MxNx3 and an RGBA-image MxNx4.
 
     """
     if not Image.isImageType(im):
@@ -67,8 +164,9 @@ def fromimage(im, flatten=0):
     return array(im)
 
 _errstr = "Mode is unknown or incompatible with input array shape."
-def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
-            mode=None,channel_axis=None):
+
+def toimage(arr, high=255, low=0, cmin=None, cmax=None, pal=None,
+            mode=None, channel_axis=None):
     """Takes a numpy array and returns a PIL image.  The mode of the
     PIL image depends on the array shape, the pal keyword, and the mode
     keyword.
@@ -87,11 +185,12 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
     """
     data = asarray(arr)
     if iscomplexobj(data):
-        raise ValueError, "Cannot convert a complex-valued array."
+        raise ValueError("Cannot convert a complex-valued array.")
     shape = list(data.shape)
     valid = len(shape)==2 or ((len(shape)==3) and \
                               ((3 in shape) or (4 in shape)))
-    assert valid, "Not a suitable array shape for any mode."
+    if not valid:
+        raise ValueError("'arr' does not have a suitable array shape for any mode.")
     if len(shape) == 2:
         shape = (shape[1],shape[0]) # columns show up first
         if mode == 'F':
@@ -122,7 +221,7 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
             data32 = data.astype(numpy.uint32)
             image = Image.fromstring(mode,shape,data32.tostring())
         else:
-            raise ValueError, _errstr
+            raise ValueError(_errstr)
         return image
 
     # if here then 3-d array with a 3 or a 4 in the shape length.
@@ -135,13 +234,13 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
             if len(ca):
                 ca = ca[0]
             else:
-                raise ValueError, "Could not find channel dimension."
+                raise ValueError("Could not find channel dimension.")
     else:
         ca = channel_axis
 
     numch = shape[ca]
     if numch not in [3,4]:
-        raise ValueError, "Channel axis dimension is not valid."
+        raise ValueError("Channel axis dimension is not valid.")
 
     bytedata = bytescale(data,high=high,low=low,cmin=cmin,cmax=cmax)
     if ca == 2:
@@ -159,24 +258,47 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
 
 
     if mode not in ['RGB','RGBA','YCbCr','CMYK']:
-        raise ValueError, _errstr
+        raise ValueError(_errstr)
 
     if mode in ['RGB', 'YCbCr']:
-        assert numch == 3, "Invalid array shape for mode."
+        if numch != 3:
+            raise ValueError("Invalid array shape for mode.")
     if mode in ['RGBA', 'CMYK']:
-        assert numch == 4, "Invalid array shape for mode."
+        if numch != 4:
+            raise ValueError("Invalid array shape for mode.")
 
-    # Here we know data and mode is coorect
+    # Here we know data and mode is correct
     image = Image.fromstring(mode, shape, strdata)
     return image
 
 def imrotate(arr,angle,interp='bilinear'):
-    """Rotate an image counter-clockwise by angle degrees.
+    """
+    Rotate an image counter-clockwise by angle degrees.
+
+    Parameters
+    ----------
+    arr : nd_array
+        Input array of image to be rotated.
+    angle : float
+        The angle of rotation.
+    interp : str, optional
+        Interpolation
+
+
+    Returns
+    -------
+    imrotate : nd_array
+        The rotated array of image.
+
+    Notes
+    -----
 
     Interpolation methods can be:
-        'nearest' :  for nearest neighbor
-        'bilinear' : for bilinear
-        'cubic' or 'bicubic' : for bicubic
+     * 'nearest' :  for nearest neighbor
+     * 'bilinear' : for bilinear
+     * 'cubic' : cubic
+     * 'bicubic' : for bicubic
+
     """
     arr = asarray(arr)
     func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
@@ -184,18 +306,29 @@ def imrotate(arr,angle,interp='bilinear'):
     im = im.rotate(angle,resample=func[interp])
     return fromimage(im)
 
-def imresize(arr,newsize,interp='bilinear',mode=None):
-    newsize=list(newsize)
-    newsize.reverse()
-    newsize = tuple(newsize)
-    arr = asarray(arr)
-    func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
-    im = toimage(arr,mode=mode)
-    im = im.resize(newsize,resample=func[interp])
-    return fromimage(im)
-
 def imshow(arr):
-    """Simple showing of an image through an external viewer.
+    """
+    Simple showing of an image through an external viewer.
+
+    Uses the image viewer specified by the environment variable
+    SCIPY_PIL_IMAGE_VIEWER, or if that is not defined then `see`,
+    to view a temporary file generated from array data.
+
+    Parameters
+    ----------
+    arr : ndarray
+        Array of image data to show.
+
+    Returns
+    -------
+    None
+
+    Examples
+    --------
+    >>> a = np.tile(np.arange(255), (255,1))
+    >>> from scipy import misc
+    >>> misc.pilutil.imshow(a)
+
     """
     im = toimage(arr)
     fnum,fname = tempfile.mkstemp('.png')
@@ -214,14 +347,34 @@ def imshow(arr):
     if status != 0:
         raise RuntimeError('Could not execute image viewer.')
 
-def imresize(arr,size):
-    """Resize an image.
-
-    If size is an integer it is a percentage of current size.
-    If size is a float it is a fraction of current size.
-    If size is a tuple it is the size of the output image.
+def imresize(arr, size, interp='bilinear', mode=None):
     """
-    im = toimage(arr)
+    Resize an image.
+
+    Parameters
+    ----------
+    arr : nd_array
+        The array of image to be resized.
+
+    size : int, float or tuple
+        * int   - Percentage of current size.
+        * float - Fraction of current size.
+        * tuple - Size of the output image.
+
+    interp : str
+        Interpolation to use for re-sizing ('nearest', 'bilinear', 'bicubic'
+        or 'cubic').
+
+    mode : str
+        The PIL image mode ('P', 'L', etc.).
+
+    Returns
+    -------
+    imresize : ndarray
+        The resized array of image.
+
+    """
+    im = toimage(arr, mode=mode)
     ts = type(size)
     if issubdtype(ts,int):
         size = size / 100.0
@@ -229,16 +382,35 @@ def imresize(arr,size):
         size = (array(im.size)*size).astype(int)
     else:
         size = (size[1],size[0])
-    imnew = im.resize(size)
+    func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
+    imnew = im.resize(size, resample=func[interp])
     return fromimage(imnew)
 
 
 def imfilter(arr,ftype):
-    """Simple filtering of an image.
+    """
+    Simple filtering of an image.
+
+    Parameters
+    ----------
+    arr : ndarray
+        The array of Image in which the filter is to be applied.
+    ftype : str
+        The filter that has to be applied. Legal values are:
+        'blur', 'contour', 'detail', 'edge_enhance', 'edge_enhance_more',
+        'emboss', 'find_edges', 'smooth', 'smooth_more', 'sharpen'.
+
+    Returns
+    -------
+    imfilter : ndarray
+        The array with filter applied.
+
+    Raises
+    ------
+    ValueError
+        *Unknown filter type.* . If the filter you are trying
+        to apply is unsupported.
 
-    type can be:
-            'blur', 'contour', 'detail', 'edge_enhance', 'edge_enhance_more',
-            'emboss', 'find_edges', 'smooth', 'smooth_more', 'sharpen'
     """
     _tdict = {'blur':ImageFilter.BLUR,
               'contour':ImageFilter.CONTOUR,
@@ -254,7 +426,7 @@ def imfilter(arr,ftype):
 
     im = toimage(arr)
     if ftype not in _tdict.keys():
-        raise ValueError, "Unknown filter type."
+        raise ValueError("Unknown filter type.")
     return fromimage(im.filter(_tdict[ftype]))
 
 
