@@ -33,12 +33,15 @@ import numpy
 import _ni_support
 import _nd_image
 
+__all__ = ['spline_filter1d', 'spline_filter', 'geometric_transform',
+           'map_coordinates', 'affine_transform', 'shift', 'zoom', 'rotate']
+
+
 def _extend_mode_to_code(mode):
     mode = _ni_support._extend_mode_to_code(mode)
     return mode
 
-def spline_filter1d(input, order = 3, axis = -1, output = numpy.float64,
-                    output_type = None):
+def spline_filter1d(input, order=3, axis=-1, output=numpy.float64):
     """
     Calculates a one-dimensional spline filter along the given axis.
 
@@ -57,8 +60,6 @@ def spline_filter1d(input, order = 3, axis = -1, output = numpy.float64,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array. Default is `numpy.float64`.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
 
     Returns
     -------
@@ -68,12 +69,11 @@ def spline_filter1d(input, order = 3, axis = -1, output = numpy.float64,
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
-    output, return_value = _ni_support._get_output(output, input,
-                                                    output_type)
+        raise TypeError('Complex type not supported')
+    output, return_value = _ni_support._get_output(output, input)
     if order in [0, 1]:
         output[...] = numpy.array(input)
     else:
@@ -82,8 +82,7 @@ def spline_filter1d(input, order = 3, axis = -1, output = numpy.float64,
     return return_value
 
 
-def spline_filter(input, order = 3, output = numpy.float64,
-                  output_type = None):
+def spline_filter(input, order=3, output = numpy.float64):
     """
     Multi-dimensional spline filter.
 
@@ -103,12 +102,11 @@ def spline_filter(input, order = 3, output = numpy.float64,
 
     """
     if order < 2 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
-    output, return_value = _ni_support._get_output(output, input,
-                                                    output_type)
+        raise TypeError('Complex type not supported')
+    output, return_value = _ni_support._get_output(output, input)
     if order not in [0, 1] and input.ndim > 0:
         for axis in range(input.ndim):
             spline_filter1d(input, order, axis, output = output)
@@ -117,10 +115,10 @@ def spline_filter(input, order = 3, output = numpy.float64,
         output[...] = input[...]
     return return_value
 
-def geometric_transform(input, mapping, output_shape = None,
-                        output_type = None, output = None, order = 3,
-                        mode = 'constant', cval = 0.0, prefilter = True,
-                        extra_arguments = (), extra_keywords = {}):
+def geometric_transform(input, mapping, output_shape=None,
+                        output=None, order=3,
+                        mode='constant', cval=0.0, prefilter=True,
+                        extra_arguments=(), extra_keywords={}):
     """
     Apply an arbritrary geometric transform.
 
@@ -142,8 +140,6 @@ def geometric_transform(input, mapping, output_shape = None,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -188,28 +184,28 @@ def geometric_transform(input, mapping, output_shape = None,
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     if output_shape is None:
         output_shape = input.shape
     if input.ndim < 1 or len(output_shape) < 1:
-        raise RuntimeError, 'input and output rank must be > 0'
+        raise RuntimeError('input and output rank must be > 0')
     mode = _extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output = numpy.float64)
     else:
         filtered = input
     output, return_value = _ni_support._get_output(output, input,
-                                        output_type, shape = output_shape)
+                                                   shape=output_shape)
     _nd_image.geometric_transform(filtered, mapping, None, None, None,
                output, order, mode, cval, extra_arguments, extra_keywords)
     return return_value
 
 
-def map_coordinates(input, coordinates, output_type = None, output = None,
-                order = 3, mode = 'constant', cval = 0.0, prefilter = True):
+def map_coordinates(input, coordinates, output=None, order=3,
+                    mode='constant', cval=0.0, prefilter=True):
     """
     Map the input array to new coordinates by interpolation.
 
@@ -232,8 +228,6 @@ def map_coordinates(input, coordinates, output_type = None, output = None,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -262,56 +256,56 @@ def map_coordinates(input, coordinates, output_type = None, output = None,
 
     Examples
     --------
-    >>> import scipy.ndimage
+    >>> from scipy import ndimage
     >>> a = np.arange(12.).reshape((4, 3))
     >>> a
     array([[  0.,   1.,   2.],
            [  3.,   4.,   5.],
            [  6.,   7.,   8.],
            [  9.,  10.,  11.]])
-    >>> sp.ndimage.map_coordinates(a, [[0.5, 2], [0.5, 1]], order=1)
+    >>> ndimage.map_coordinates(a, [[0.5, 2], [0.5, 1]], order=1)
     [ 2.  7.]
 
     Above, the interpolated value of a[0.5, 0.5] gives output[0], while
     a[2, 1] is output[1].
 
     >>> inds = np.array([[0.5, 2], [0.5, 4]])
-    >>> sp.ndimage.map_coordinates(a, inds, order=1, cval=-33.3)
+    >>> ndimage.map_coordinates(a, inds, order=1, cval=-33.3)
     array([  2. , -33.3])
-    >>> sp.ndimage.map_coordinates(a, inds, order=1, mode='nearest')
+    >>> ndimage.map_coordinates(a, inds, order=1, mode='nearest')
     array([ 2.,  8.])
-    >>> sp.ndimage.map_coordinates(a, inds, order=1, cval=0, output=bool)
+    >>> ndimage.map_coordinates(a, inds, order=1, cval=0, output=bool)
     array([ True, False], dtype=bool
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     coordinates = numpy.asarray(coordinates)
     if numpy.iscomplexobj(coordinates):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     output_shape = coordinates.shape[1:]
     if input.ndim < 1 or len(output_shape) < 1:
-        raise RuntimeError, 'input and output rank must be > 0'
+        raise RuntimeError('input and output rank must be > 0')
     if coordinates.shape[0] != input.ndim:
-        raise RuntimeError, 'invalid shape for coordinate array'
+        raise RuntimeError('invalid shape for coordinate array')
     mode = _extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output = numpy.float64)
     else:
         filtered = input
     output, return_value = _ni_support._get_output(output, input,
-                                        output_type, shape = output_shape)
+                                                   shape=output_shape)
     _nd_image.geometric_transform(filtered, None, coordinates, None, None,
                output, order, mode, cval, None, None)
     return return_value
 
 
-def affine_transform(input, matrix, offset = 0.0, output_shape = None,
-                     output_type = None, output = None, order = 3,
-                     mode = 'constant', cval = 0.0, prefilter = True):
+def affine_transform(input, matrix, offset=0.0, output_shape=None,
+                     output=None, order=3,
+                     mode='constant', cval=0.0, prefilter=True):
     """
     Apply an affine transformation.
 
@@ -340,8 +334,6 @@ def affine_transform(input, matrix, offset = 0.0, output_shape = None,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -366,34 +358,34 @@ def affine_transform(input, matrix, offset = 0.0, output_shape = None,
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     if output_shape is None:
         output_shape = input.shape
     if input.ndim < 1 or len(output_shape) < 1:
-        raise RuntimeError, 'input and output rank must be > 0'
+        raise RuntimeError('input and output rank must be > 0')
     mode = _extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output = numpy.float64)
     else:
         filtered = input
     output, return_value = _ni_support._get_output(output, input,
-                                        output_type, shape = output_shape)
+                                                   shape=output_shape)
     matrix = numpy.asarray(matrix, dtype = numpy.float64)
     if matrix.ndim not in [1, 2] or matrix.shape[0] < 1:
-        raise RuntimeError, 'no proper affine matrix provided'
+        raise RuntimeError('no proper affine matrix provided')
     if matrix.shape[0] != input.ndim:
-        raise RuntimeError, 'affine matrix has wrong number of rows'
+        raise RuntimeError('affine matrix has wrong number of rows')
     if matrix.ndim == 2 and matrix.shape[1] != output.ndim:
-        raise RuntimeError, 'affine matrix has wrong number of columns'
+        raise RuntimeError('affine matrix has wrong number of columns')
     if not matrix.flags.contiguous:
         matrix = matrix.copy()
     offset = _ni_support._normalize_sequence(offset, input.ndim)
     offset = numpy.asarray(offset, dtype = numpy.float64)
     if offset.ndim != 1 or offset.shape[0] < 1:
-        raise RuntimeError, 'no proper offset provided'
+        raise RuntimeError('no proper offset provided')
     if not offset.flags.contiguous:
         offset = offset.copy()
     if matrix.ndim == 1:
@@ -405,8 +397,8 @@ def affine_transform(input, matrix, offset = 0.0, output_shape = None,
     return return_value
 
 
-def shift(input, shift, output_type = None, output = None, order = 3,
-          mode = 'constant', cval = 0.0, prefilter = True):
+def shift(input, shift, output=None, order=3, mode='constant', cval=0.0,
+          prefilter=True):
     """
     Shift an array.
 
@@ -424,8 +416,6 @@ def shift(input, shift, output_type = None, output = None, order = 3,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -450,19 +440,18 @@ def shift(input, shift, output_type = None, output = None, order = 3,
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     if input.ndim < 1:
-        raise RuntimeError, 'input and output rank must be > 0'
+        raise RuntimeError('input and output rank must be > 0')
     mode = _extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output = numpy.float64)
     else:
         filtered = input
-    output, return_value = _ni_support._get_output(output, input,
-                                                    output_type)
+    output, return_value = _ni_support._get_output(output, input)
     shift = _ni_support._normalize_sequence(shift, input.ndim)
     shift = [-ii for ii in shift]
     shift = numpy.asarray(shift, dtype = numpy.float64)
@@ -472,8 +461,8 @@ def shift(input, shift, output_type = None, output = None, order = 3,
     return return_value
 
 
-def zoom(input, zoom, output_type = None, output = None, order = 3,
-         mode = 'constant', cval = 0.0, prefilter = True):
+def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
+         prefilter=True):
     """
     Zoom an array.
 
@@ -489,8 +478,6 @@ def zoom(input, zoom, output_type = None, output = None, order = 3,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -515,12 +502,12 @@ def zoom(input, zoom, output_type = None, output = None, order = 3,
 
     """
     if order < 0 or order > 5:
-        raise RuntimeError, 'spline order not supported'
+        raise RuntimeError('spline order not supported')
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
-        raise TypeError, 'Complex type not supported'
+        raise TypeError('Complex type not supported')
     if input.ndim < 1:
-        raise RuntimeError, 'input and output rank must be > 0'
+        raise RuntimeError('input and output rank must be > 0')
     mode = _extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output = numpy.float64)
@@ -528,9 +515,16 @@ def zoom(input, zoom, output_type = None, output = None, order = 3,
         filtered = input
     zoom = _ni_support._normalize_sequence(zoom, input.ndim)
     output_shape = tuple([int(ii * jj) for ii, jj in zip(input.shape, zoom)])
-    zoom = (numpy.array(input.shape)-1)/(numpy.array(output_shape,float)-1)
+
+    zoom_div = numpy.array(output_shape, float) - 1
+    zoom = (numpy.array(input.shape) - 1) / zoom_div
+
+    # Zooming to infinity is unpredictable, so just choose
+    # zoom factor 1 instead
+    zoom[numpy.isinf(zoom)] = 1
+
     output, return_value = _ni_support._get_output(output, input,
-                                        output_type, shape = output_shape)
+                                                   shape=output_shape)
     zoom = numpy.asarray(zoom, dtype = numpy.float64)
     zoom = numpy.ascontiguousarray(zoom)
     _nd_image.zoom_shift(filtered, zoom, None, output, order, mode, cval)
@@ -547,9 +541,9 @@ def _minmax(coor, minc, maxc):
         maxc[1] = coor[1]
     return minc, maxc
 
-def rotate(input, angle, axes = (1, 0), reshape = True,
-           output_type = None, output = None, order = 3,
-           mode = 'constant', cval = 0.0, prefilter = True):
+def rotate(input, angle, axes=(1, 0), reshape=True,
+           output=None, order=3,
+           mode='constant', cval=0.0, prefilter=True):
     """
     Rotate an array.
 
@@ -571,8 +565,6 @@ def rotate(input, angle, axes = (1, 0), reshape = True,
     output : ndarray or dtype, optional
         The array in which to place the output, or the dtype of the returned
         array.
-    output_type : dtype, optional
-        DEPRECATED, DO NOT USE. If used, a RuntimeError is raised.
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
@@ -604,7 +596,7 @@ def rotate(input, angle, axes = (1, 0), reshape = True,
     if axes[1] < 0:
         axes[1] += rank
     if axes[0] < 0 or axes[1] < 0 or axes[0] > rank or axes[1] > rank:
-        raise RuntimeError, 'invalid rotation plane specified'
+        raise RuntimeError('invalid rotation plane specified')
     if axes[0] > axes[1]:
         axes = axes[1], axes[0]
     angle = numpy.pi / 180 * angle
@@ -645,15 +637,15 @@ def rotate(input, angle, axes = (1, 0), reshape = True,
     output_shape[axes[1]] = ox
     output_shape = tuple(output_shape)
     output, return_value = _ni_support._get_output(output, input,
-                                        output_type, shape = output_shape)
+                                                   shape=output_shape)
     if input.ndim <= 2:
-        affine_transform(input, matrix, offset, output_shape, None, output,
+        affine_transform(input, matrix, offset, output_shape, output,
                          order, mode, cval, prefilter)
     else:
         coordinates = []
         size = numpy.product(input.shape,axis=0)
-        size /= input.shape[axes[0]]
-        size /= input.shape[axes[1]]
+        size //= input.shape[axes[0]]
+        size //= input.shape[axes[1]]
         for ii in range(input.ndim):
             if ii not in axes:
                 coordinates.append(0)
@@ -667,7 +659,7 @@ def rotate(input, angle, axes = (1, 0), reshape = True,
         for ii in range(size):
             ia = input[tuple(coordinates)]
             oa = output[tuple(coordinates)]
-            affine_transform(ia, matrix, offset, os, None, oa, order, mode,
+            affine_transform(ia, matrix, offset, os, oa, order, mode,
                              cval, prefilter)
             for jj in iter_axes:
                 if coordinates[jj] < input.shape[jj] - 1:
