@@ -63,8 +63,7 @@ typedef enum
          tComplex128=PyArray_COMPLEX128,
          tObject=PyArray_OBJECT,        /* placeholder... does nothing */
          tMaxType=PyArray_NTYPES,
-         tDefault=PyArray_FLOAT64,
-         tLong=PyArray_LONG,
+         tDefault=PyArray_FLOAT64
 } NumarrayType;
 
 #define NI_MAXDIM NPY_MAXDIMS
@@ -74,7 +73,6 @@ typedef enum
 #define HAS_UINT64 1
 
 
-
 #ifdef ND_IMPORT_ARRAY
 
 /* Numarray Helper Functions */
