@@ -113,7 +113,7 @@ spline_coefficients(double x, int order, double *result)
 /* map a coordinate outside the borders, according to the requested
      boundary condition: */
 static double
-map_coordinate(double in, maybelong len, int mode)
+map_coordinate(double in, npy_intp len, int mode)
 {
     if (in < 0) {
         switch (mode) {
@@ -121,8 +121,8 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz2 = 2 * len - 2;
-                in = sz2 * (maybelong)(-in / sz2) + in;
+                npy_intp sz2 = 2 * len - 2;
+                in = sz2 * (npy_intp)(-in / sz2) + in;
                 in = in <= 1 - len ? in + sz2 : -in;
             }
             break;
@@ -130,9 +130,9 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz2 = 2 * len;
+                npy_intp sz2 = 2 * len;
                 if (in < -sz2)
-                    in = sz2 * (maybelong)(-in / sz2) + in;
+                    in = sz2 * (npy_intp)(-in / sz2) + in;
                 in = in < -len ? in + sz2 : -in - 1;
             }
             break;
@@ -140,10 +140,10 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz = len - 1;
+                npy_intp sz = len - 1;
                 // Integer division of -in/sz gives (-in mod sz)
                 // Note that 'in' is negative
-                in += sz * ((maybelong)(-in / sz) + 1);
+                in += sz * ((npy_intp)(-in / sz) + 1);
             }
             break;
         case NI_EXTEND_NEAREST:
@@ -159,8 +159,8 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz2 = 2 * len - 2;
-                in -= sz2 * (maybelong)(in / sz2);
+                npy_intp sz2 = 2 * len - 2;
+                in -= sz2 * (npy_intp)(in / sz2);
                 if (in >= len)
                     in = sz2 - in;
             }
@@ -169,8 +169,8 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz2 = 2 * len;
-                in -= sz2 * (maybelong)(in / sz2);
+                npy_intp sz2 = 2 * len;
+                in -= sz2 * (npy_intp)(in / sz2);
                 if (in >= len)
                     in = sz2 - in - 1;
             }
@@ -179,8 +179,8 @@ map_coordinate(double in, maybelong len, int mode)
             if (len <= 1) {
                 in = 0;
             } else {
-                maybelong sz = len - 1;
-                in -= sz * (maybelong)(in / sz);
+                npy_intp sz = len - 1;
+                in -= sz * (npy_intp)(in / sz);
             }
             break;
         case NI_EXTEND_NEAREST:
@@ -203,7 +203,7 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
                                             PyArrayObject *output)
 {
     int hh, npoles = 0, more;
-    maybelong kk, ll, lines, len;
+    npy_intp kk, ll, lines, len;
     double *buffer = NULL, weight, pole[2];
     NI_LineBuffer iline_buffer, oline_buffer;
 
@@ -307,10 +307,11 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
     return PyErr_Occurred() ? 0 : 1;
 }
 
+/* copy row of coordinate array from location at _p to _coor */
 #define CASE_MAP_COORDINATES(_p, _coor, _rank, _stride, _type) \
 case t ## _type:                                                    \
 {                                                              \
-    int _hh;                                                     \
+    npy_intp _hh;                                               \
     for(_hh = 0; _hh < _rank; _hh++) {                           \
         _coor[_hh] = *(_type*)_p;                                  \
         _p += _stride;                                             \
@@ -345,18 +346,18 @@ case t ## _type:                            \
     break;
 
 int
-NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
+NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                 int, int, void*), void* map_data, PyArrayObject* matrix_ar,
                 PyArrayObject* shift_ar, PyArrayObject *coordinates,
                 PyArrayObject *output, int order, int mode, double cval)
 {
     char *po, *pi, *pc = NULL;
-    maybelong **edge_offsets = NULL, **data_offsets = NULL, filter_size;
-    maybelong ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
-    maybelong cstride = 0, kk, hh, ll, jj, *idxs = NULL;
-    maybelong size;
+    npy_intp **edge_offsets = NULL, **data_offsets = NULL, filter_size;
+    npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
+    npy_intp cstride = 0, kk, hh, ll, jj, *idxs = NULL;
+    npy_intp size;
     double **splvals = NULL, icoor[MAXDIM];
-    double idimensions[MAXDIM], istrides[MAXDIM];
+    npy_intp idimensions[MAXDIM], istrides[MAXDIM];
     NI_Iterator io, ic;
     Float64 *matrix = matrix_ar ? (Float64*)PyArray_DATA(matrix_ar) : NULL;
     Float64 *shift = shift_ar ? (Float64*)PyArray_DATA(shift_ar) : NULL;
@@ -381,8 +382,8 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
     }
 
     /* offsets used at the borders: */
-    edge_offsets = (maybelong**)malloc(irank * sizeof(maybelong*));
-    data_offsets = (maybelong**)malloc(irank * sizeof(maybelong*));
+    edge_offsets = (npy_intp**)malloc(irank * sizeof(npy_intp*));
+    data_offsets = (npy_intp**)malloc(irank * sizeof(npy_intp*));
     if (!edge_offsets || !data_offsets) {
         PyErr_NoMemory();
         goto exit;
@@ -390,7 +391,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
     for(jj = 0; jj < irank; jj++)
         data_offsets[jj] = NULL;
     for(jj = 0; jj < irank; jj++) {
-        data_offsets[jj] = (maybelong*)malloc((order + 1) * sizeof(maybelong));
+        data_offsets[jj] = (npy_intp*)malloc((order + 1) * sizeof(npy_intp));
         if (!data_offsets[jj]) {
             PyErr_NoMemory();
             goto exit;
@@ -415,7 +416,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
     filter_size = 1;
     for(jj = 0; jj < irank; jj++)
         filter_size *= order + 1;
-    idxs = (maybelong*)malloc(filter_size * sizeof(idxs));
+    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
     if (!idxs) {
         PyErr_NoMemory();
         goto exit;
@@ -430,9 +431,9 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
     po = (void *)PyArray_DATA(output);
 
     /* make a table of all possible coordinates within the spline filter: */
-    fcoordinates = (maybelong*)malloc(irank * filter_size * sizeof(maybelong));
+    fcoordinates = (npy_intp*)malloc(irank * filter_size * sizeof(npy_intp));
     /* make a table of all offsets within the spline filter: */
-    foffsets = (maybelong*)malloc(filter_size * sizeof(maybelong));
+    foffsets = (npy_intp*)malloc(filter_size * sizeof(npy_intp));
     if (!fcoordinates || !foffsets) {
         PyErr_NoMemory();
         goto exit;
@@ -482,7 +483,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
         } else if (coordinates) {
             /* mapping is from an coordinates array: */
             char *p = pc;
-            switch(coordinates->descr->type_num) {
+            switch (NI_NormalizeType(coordinates->descr->type_num)) {
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, Bool);
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, UInt8);
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, UInt16);
@@ -552,7 +553,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
         }
 
         if (!constant) {
-            maybelong *ff = fcoordinates;
+            npy_intp *ff = fcoordinates;
             for(hh = 0; hh < filter_size; hh++) {
                 int idx = 0;
                 if (edge) {
@@ -571,11 +572,11 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
             }
         }
         if (!constant) {
-            maybelong *ff = fcoordinates;
+            npy_intp *ff = fcoordinates;
             t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
                 double coeff = 0.0;
-                switch(input->descr->type_num) {
+                switch (NI_NormalizeType(input->descr->type_num)) {
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
@@ -590,7 +591,8 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
                 default:
-                    PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+                    PyErr_SetString(PyExc_RuntimeError,
+                                                    "data type not supported");
                     goto exit;
                 }
                 /* calculate the interpolated value: */
@@ -604,14 +606,17 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(maybelong*, double*,
             t = cval;
         }
         /* store output value: */
-        switch (output->descr->type_num) {
+        switch (NI_NormalizeType(output->descr->type_num)) {
             CASE_INTERP_OUT(po, t, Bool);
             CASE_INTERP_OUT_UINT(po, t, UInt8, 0, MAX_UINT8);
             CASE_INTERP_OUT_UINT(po, t, UInt16, 0, MAX_UINT16);
             CASE_INTERP_OUT_UINT(po, t, UInt32, 0, MAX_UINT32);
 #if HAS_UINT64
-            /* FIXME */
-            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, MAX_UINT32);
+            /* There was a bug in numpy as of (at least) <= 1.6.1 such that
+             * MAX_UINT64 was incorrectly defined, leading to a compiler error.
+             * NPY_MAX_UINT64 is correctly defined
+             */
+            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, NPY_MAX_UINT64);
 #endif
             CASE_INTERP_OUT_INT(po, t, Int8, MIN_INT8, MAX_INT8);
             CASE_INTERP_OUT_INT(po, t, Int16, MIN_INT16, MAX_INT16);
@@ -657,11 +662,11 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                                  int order, int mode, double cval)
 {
     char *po, *pi;
-    maybelong **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
-    maybelong ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
-    maybelong jj, hh, kk, filter_size, odimensions[MAXDIM];
-    maybelong idimensions[MAXDIM], istrides[MAXDIM], *idxs = NULL;
-    maybelong size;
+    npy_intp **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
+    npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
+    npy_intp jj, hh, kk, filter_size, odimensions[MAXDIM];
+    npy_intp idimensions[MAXDIM], istrides[MAXDIM], *idxs = NULL;
+    npy_intp size;
     double ***splvals = NULL;
     NI_Iterator io;
     Float64 *zooms = zoom_ar ? (Float64*)PyArray_DATA(zoom_ar) : NULL;
@@ -677,7 +682,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
 
     /* if the mode is 'constant' we need some temps later: */
     if (mode == NI_EXTEND_CONSTANT) {
-        zeros = (maybelong**)malloc(rank * sizeof(maybelong*));
+        zeros = (npy_intp**)malloc(rank * sizeof(npy_intp*));
         if (!zeros) {
             PyErr_NoMemory();
             goto exit;
@@ -685,7 +690,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         for(jj = 0; jj < rank; jj++)
             zeros[jj] = NULL;
         for(jj = 0; jj < rank; jj++) {
-            zeros[jj] = (maybelong*)malloc(odimensions[jj] * sizeof(maybelong));
+            zeros[jj] = (npy_intp*)malloc(odimensions[jj] * sizeof(npy_intp));
             if(!zeros[jj]) {
                 PyErr_NoMemory();
                 goto exit;
@@ -694,11 +699,11 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     }
 
     /* store offsets, along each axis: */
-    offsets = (maybelong**)malloc(rank * sizeof(maybelong*));
+    offsets = (npy_intp**)malloc(rank * sizeof(npy_intp*));
     /* store spline coefficients, along each axis: */
     splvals = (double***)malloc(rank * sizeof(double**));
     /* store offsets at all edges: */
-    edge_offsets = (maybelong***)malloc(rank * sizeof(maybelong**));
+    edge_offsets = (npy_intp***)malloc(rank * sizeof(npy_intp**));
     if (!offsets || !splvals || !edge_offsets) {
         PyErr_NoMemory();
         goto exit;
@@ -709,9 +714,9 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         edge_offsets[jj] = NULL;
     }
     for(jj = 0; jj < rank; jj++) {
-        offsets[jj] = (maybelong*)malloc(odimensions[jj] * sizeof(maybelong));
+        offsets[jj] = (npy_intp*)malloc(odimensions[jj] * sizeof(npy_intp));
         splvals[jj] = (double**)malloc(odimensions[jj] * sizeof(double*));
-        edge_offsets[jj] = (maybelong**)malloc(odimensions[jj] * sizeof(maybelong*));
+        edge_offsets[jj] = (npy_intp**)malloc(odimensions[jj] * sizeof(npy_intp*));
         if (!offsets[jj] || !splvals[jj] || !edge_offsets[jj]) {
             PyErr_NoMemory();
             goto exit;
@@ -747,7 +752,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                 }
                 offsets[jj][kk] = istrides[jj] * start;
                 if (start < 0 || start + order >= idimensions[jj]) {
-                    edge_offsets[jj][kk] = (maybelong*)malloc((order + 1) * sizeof(maybelong));
+                    edge_offsets[jj][kk] = (npy_intp*)malloc((order + 1) * sizeof(npy_intp));
                     if (!edge_offsets[jj][kk]) {
                         PyErr_NoMemory();
                         goto exit;
@@ -788,7 +793,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     filter_size = 1;
     for(jj = 0; jj < rank; jj++)
         filter_size *= order + 1;
-    idxs = (maybelong*)malloc(filter_size * sizeof(idxs));
+    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
     if (!idxs) {
         PyErr_NoMemory();
         goto exit;
@@ -801,8 +806,8 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     po = (void *)PyArray_DATA(output);
 
     /* store all coordinates and offsets with filter: */
-    fcoordinates = (maybelong*)malloc(rank * filter_size * sizeof(maybelong));
-    foffsets = (maybelong*)malloc(filter_size * sizeof(maybelong));
+    fcoordinates = (npy_intp*)malloc(rank * filter_size * sizeof(npy_intp));
+    foffsets = (npy_intp*)malloc(filter_size * sizeof(npy_intp));
     if (!fcoordinates || !foffsets) {
         PyErr_NoMemory();
         goto exit;
@@ -845,7 +850,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         }
 
         if (!zero) {
-            maybelong *ff = fcoordinates;
+            npy_intp *ff = fcoordinates;
             for(hh = 0; hh < filter_size; hh++) {
                 int idx = 0;
                 if (edge) {
@@ -866,11 +871,11 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
             }
         }
         if (!zero) {
-            maybelong *ff = fcoordinates;
+            npy_intp *ff = fcoordinates;
             t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
                 double coeff = 0.0;
-                switch(input->descr->type_num) {
+                switch (NI_NormalizeType(input->descr->type_num)) {
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
@@ -885,7 +890,8 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
                     CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
                 default:
-                    PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+                    PyErr_SetString(PyExc_RuntimeError,
+                                                    "data type not supported");
                     goto exit;
                 }
                 /* calculate interpolated value: */
@@ -899,14 +905,17 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
             t = cval;
         }
         /* store output: */
-        switch (output->descr->type_num) {
+        switch (NI_NormalizeType(output->descr->type_num)) {
             CASE_INTERP_OUT(po, t, Bool);
             CASE_INTERP_OUT_UINT(po, t, UInt8, 0, MAX_UINT8);
             CASE_INTERP_OUT_UINT(po, t, UInt16, 0, MAX_UINT16);
             CASE_INTERP_OUT_UINT(po, t, UInt32, 0, MAX_UINT32);
 #if HAS_UINT64
-            /* FIXME */
-            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, MAX_UINT32);
+            /* There was a bug in numpy as of (at least) <= 1.6.1 such that
+             * MAX_UINT64 was incorrectly defined, leading to a compiler error.
+             * NPY_MAX_UINT64 is correctly defined
+             */
+            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, NPY_MAX_UINT64);
 #endif
             CASE_INTERP_OUT_INT(po, t, Int8, MIN_INT8, MAX_INT8);
             CASE_INTERP_OUT_INT(po, t, Int16, MIN_INT16, MAX_INT16);
