@@ -17,16 +17,9 @@ from sputils import upcast, to_native, isdense, isshape, getdtype, \
 class _cs_matrix(_data_matrix):
     """base matrix class for compressed row and column oriented matrices"""
 
-    def __init__(self, arg1, shape=None, dtype=None, copy=False, dims=None, nzmax=None):
+    def __init__(self, arg1, shape=None, dtype=None, copy=False):
         _data_matrix.__init__(self)
 
-        if dims is not None:
-            warn("dims= is deprecated, use shape= instead", DeprecationWarning)
-            shape=dims
-
-        if nzmax is not None:
-            warn("nzmax= is deprecated", DeprecationWarning)
-
 
         if isspmatrix(arg1):
             if arg1.format == self.format and copy:
@@ -57,16 +50,16 @@ class _cs_matrix(_data_matrix):
                     self.indptr  = np.array(indptr, copy=copy)
                     self.data    = np.array(data, copy=copy, dtype=getdtype(dtype, data))
                 else:
-                    raise ValueError, "unrecognized %s_matrix constructor usage" %\
-                            self.format
+                    raise ValueError("unrecognized %s_matrix constructor usage" %
+                            self.format)
 
         else:
             #must be dense
             try:
                 arg1 = np.asarray(arg1)
             except:
-                raise ValueError, "unrecognized %s_matrix constructor usage" % \
-                        self.format
+                raise ValueError("unrecognized %s_matrix constructor usage" %
+                        self.format)
             from coo import coo_matrix
             self._set_self( self.__class__(coo_matrix(arg1, dtype=dtype)) )
 
@@ -80,7 +73,7 @@ class _cs_matrix(_data_matrix):
                     major_dim = len(self.indptr) - 1
                     minor_dim = self.indices.max() + 1
                 except:
-                    raise ValueError,'unable to infer matrix dimensions'
+                    raise ValueError('unable to infer matrix dimensions')
                 else:
                     self.shape = self._swap((major_dim,minor_dim))
 
@@ -139,19 +132,17 @@ class _cs_matrix(_data_matrix):
 
         # check index pointer
         if (len(self.indptr) != major_dim + 1 ):
-            raise ValueError, \
-                "index pointer size (%d) should be (%d)" % \
-                 (len(self.indptr), major_dim + 1)
+            raise ValueError("index pointer size (%d) should be (%d)" %
+                                (len(self.indptr), major_dim + 1))
         if (self.indptr[0] != 0):
-            raise ValueError,"index pointer should start with 0"
+            raise ValueError("index pointer should start with 0")
 
         # check index and data arrays
         if (len(self.indices) != len(self.data)):
-            raise ValueError,"indices and data should have the same size"
+            raise ValueError("indices and data should have the same size")
         if (self.indptr[-1] > len(self.indices)):
-            raise ValueError, \
-                  "Last value of index pointer should be less than "\
-                  "the size of index and data arrays"
+            raise ValueError("Last value of index pointer should be less than "
+                                "the size of index and data arrays")
 
         self.prune()
 
@@ -159,14 +150,14 @@ class _cs_matrix(_data_matrix):
             #check format validity (more expensive)
             if self.nnz > 0:
                 if self.indices.max() >= minor_dim:
-                    raise ValueError, "%s index values must be < %d" % \
-                            (minor_name,minor_dim)
+                    raise ValueError("%s index values must be < %d" %
+                                        (minor_name,minor_dim))
                 if self.indices.min() < 0:
-                    raise ValueError, "%s index values must be >= 0" % \
-                            minor_name
+                    raise ValueError("%s index values must be >= 0" %
+                                        minor_name)
                 if np.diff(self.indptr).min() < 0:
-                    raise ValueError,'index pointer values must form a " \
-                                        "non-decreasing sequence'
+                    raise ValueError("index pointer values must form a "
+                                        "non-decreasing sequence")
 
         #if not self.has_sorted_indices():
         #    warn('Indices were not in sorted order.  Sorting indices.')
@@ -179,11 +170,11 @@ class _cs_matrix(_data_matrix):
         # First check if argument is a scalar
         if isscalarlike(other):
             # Now we would add this scalar to every element.
-            raise NotImplementedError, 'adding a scalar to a CSC or CSR ' \
-                  'matrix is not supported'
+            raise NotImplementedError('adding a scalar to a CSC or CSR '
+                                        'matrix is not supported')
         elif isspmatrix(other):
             if (other.shape != self.shape):
-                raise ValueError, "inconsistent shapes"
+                raise ValueError("inconsistent shapes")
 
             return self._binopt(other,'_plus_')
         elif isdense(other):
@@ -199,11 +190,11 @@ class _cs_matrix(_data_matrix):
         # First check if argument is a scalar
         if isscalarlike(other):
             # Now we would add this scalar to every element.
-            raise NotImplementedError, 'adding a scalar to a sparse ' \
-                  'matrix is not supported'
+            raise NotImplementedError('adding a scalar to a sparse '
+                                        'matrix is not supported')
         elif isspmatrix(other):
             if (other.shape != self.shape):
-                raise ValueError, "inconsistent shapes"
+                raise ValueError("inconsistent shapes")
 
             return self._binopt(other,'_minus_')
         elif isdense(other):
@@ -216,8 +207,8 @@ class _cs_matrix(_data_matrix):
         #note: this can't be replaced by other + (-self) for unsigned types
         if isscalarlike(other):
             # Now we would add this scalar to every element.
-            raise NotImplementedError, 'adding a scalar to a sparse ' \
-                  'matrix is not supported'
+            raise NotImplementedError('adding a scalar to a sparse '
+                                        'matrix is not supported')
         elif isdense(other):
             # Convert this matrix to a dense matrix and subtract them
             return other - self.todense()
@@ -307,10 +298,6 @@ class _cs_matrix(_data_matrix):
         return self.__class__((data,indices,indptr),shape=(M,N))
 
 
-    @np.deprecate
-    def getdata(self, ind):
-        return self.data[ind]
-
     def diagonal(self):
         """Returns the main diagonal of the matrix
         """
@@ -330,7 +317,7 @@ class _cs_matrix(_data_matrix):
             return self.data.sum()
         else:
             return spmatrix.sum(self,axis)
-            raise ValueError, "axis out of bounds"
+            raise ValueError("axis out of bounds")
 
     #######################
     # Getting and Setting #
@@ -361,7 +348,7 @@ class _cs_matrix(_data_matrix):
         elif isintlike(key):
             return self[key, :]
         else:
-            raise IndexError, "invalid index"
+            raise IndexError("invalid index")
 
 
     def _get_single_element(self,row,col):
@@ -395,9 +382,9 @@ class _cs_matrix(_data_matrix):
             [start:stop:string, i] for column-oriented matrices
         """
         if stride != 1:
-            raise ValueError, "slicing with step != 1 not supported"
+            raise ValueError("slicing with step != 1 not supported")
         if stop <= start:
-            raise ValueError, "slice width must be >= 1"
+            raise ValueError("slice width must be >= 1")
 
         #TODO make [i,:] faster
         #TODO implement [i,x:y:z]
@@ -445,9 +432,8 @@ class _cs_matrix(_data_matrix):
 
         def _in_bounds( i0, i1, num ):
             if not (0<=i0<num) or not (0<i1<=num) or not (i0<i1):
-                raise IndexError,\
-                      "index out of bounds: 0<=%d<%d, 0<=%d<%d, %d<%d" %\
-                      (i0, num, i1, num, i0, i1)
+                raise IndexError("index out of bounds: 0<=%d<%d, 0<=%d<%d, %d<%d" %
+                                    (i0, num, i1, num, i0, i1))
 
         i0, i1 = _process_slice( slice0, shape0 )
         j0, j1 = _process_slice( slice1, shape1 )
@@ -477,7 +463,7 @@ class _cs_matrix(_data_matrix):
             if (col < 0):
                 col += N
             if not (0<=row<M) or not (0<=col<N):
-                raise IndexError, "index out of bounds"
+                raise IndexError("index out of bounds")
 
             major_index, minor_index = self._swap((row,col))
 
@@ -486,12 +472,12 @@ class _cs_matrix(_data_matrix):
             indxs = np.where(minor_index == self.indices[start:end])[0]
 
             num_matches = len(indxs)
-    
-            
+
+
             if not np.isscalar(val):
                 raise ValueError('setting an array element with a sequence')
 
-            val = self.dtype.type(val) 
+            val = self.dtype.type(val)
 
             if num_matches == 0:
                 #entry not already present
@@ -519,12 +505,13 @@ class _cs_matrix(_data_matrix):
                 self.data[start:end][indxs[0]] = val
             else:
                 #entry appears more than once
-                raise ValueError,'nonzero entry (%d,%d) occurs more than once' % (row,col)
+                raise ValueError('nonzero entry (%d,%d) occurs more than once'
+                                % (row,col))
 
             self.check_format(full_check=True)
         else:
             # We should allow slices here!
-            raise IndexError, "invalid index"
+            raise IndexError("invalid index")
 
     ######################
     # Conversion methods #
@@ -632,19 +619,6 @@ class _cs_matrix(_data_matrix):
             fn( len(self.indptr) - 1, self.indptr, self.indices, self.data)
             self.has_sorted_indices = True
 
-    #TODO remove after 0.7
-    def ensure_sorted_indices(self, inplace=False):
-        """Return a copy of this matrix where the column indices are sorted
-        """
-        warn('ensure_sorted_indices is deprecated, ' \
-                'use sorted_indices() or sort_indices() instead', \
-                DeprecationWarning)
-
-        if inplace:
-            self.sort_indices()
-        else:
-            return self.sorted_indices()
-
     def prune(self):
         """Remove empty space after all non-zero elements.
         """
@@ -678,27 +652,19 @@ class _cs_matrix(_data_matrix):
             return self.__class__((data,self.indices,self.indptr), \
                                    shape=self.shape,dtype=data.dtype)
 
-    def _binopt(self, other, op, in_shape=None, out_shape=None):
+    def _binopt(self, other, op):
         """apply the binary operation fn to two sparse matrices"""
         other = self.__class__(other)
 
-        if in_shape is None:
-            in_shape = self.shape
-        if out_shape is None:
-            out_shape = self.shape
-
-        self.sort_indices()
-        other.sort_indices()
-
-        # e.g. csr_plus_csr, csr_mat_mat, etc.
+        # e.g. csr_plus_csr, csr_minus_csr, etc.
         fn = getattr(sparsetools, self.format + op + self.format)
 
-        maxnnz = self.nnz + other.nnz
+        maxnnz  = self.nnz + other.nnz
         indptr  = np.empty_like(self.indptr)
         indices = np.empty(maxnnz, dtype=np.intc)
         data    = np.empty(maxnnz, dtype=upcast(self.dtype,other.dtype))
 
-        fn(in_shape[0], in_shape[1], \
+        fn(self.shape[0], self.shape[1], \
                 self.indptr,  self.indices,  self.data,
                 other.indptr, other.indices, other.data,
                 indptr, indices, data)
@@ -706,11 +672,11 @@ class _cs_matrix(_data_matrix):
         actual_nnz = indptr[-1]
         indices = indices[:actual_nnz]
         data    = data[:actual_nnz]
-        if actual_nnz < maxnnz / 2:
+        if actual_nnz < maxnnz // 2:
             #too much waste, trim arrays
             indices = indices.copy()
             data    = data.copy()
 
-        A = self.__class__((data, indices, indptr), shape=out_shape)
-        A.has_sorted_indices = True
+        A = self.__class__((data, indices, indptr), shape=self.shape)
+
         return A
