@@ -8,7 +8,10 @@
 #include <Python.h>
 
 #define NO_IMPORT_ARRAY
+#define PY_ARRAY_UNIQUE_SYMBOL _scipy_sparse_superlu_ARRAY_API
+
 #include "_superluobject.h"
+#include "numpy/npy_3kcompat.h"
 #include <setjmp.h>
 #include <ctype.h>
 
@@ -38,7 +41,11 @@ static PyObject *
 SciPyLU_solve(SciPyLUObject *self, PyObject *args, PyObject *kwds) {
   PyArrayObject *b, *x=NULL;
   SuperMatrix B;
+#ifndef NPY_PY3K
   char itrans = 'N';
+#else
+  int itrans = 'N';
+#endif
   int info;
   trans_t trans;
   SuperLUStat_t stat;
@@ -50,7 +57,11 @@ SciPyLU_solve(SciPyLUObject *self, PyObject *args, PyObject *kwds) {
       return NULL;
   }
 
+#ifndef NPY_PY3K
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|c", kwlist,
+#else
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|C", kwlist,
+#endif
                                    &PyArray_Type, &b, 
                                    &itrans))
     return NULL;
@@ -157,7 +168,7 @@ SciPyLU_getattr(SciPyLUObject *self, char *name)
     PyObject *list = PyList_New(sizeof(members)/sizeof(char *));
     if (list != NULL) {
       for (i = 0; i < sizeof(members)/sizeof(char *); i ++)
-	PyList_SetItem(list, i, PyString_FromString(members[i]));
+	PyList_SetItem(list, i, PyUString_FromString(members[i]));
       if (PyErr_Occurred()) {
 	Py_DECREF(list);
 	list = NULL;
@@ -165,7 +176,17 @@ SciPyLU_getattr(SciPyLUObject *self, char *name)
     }
     return list;
   }
+#if PY_VERSION_HEX >= 0x03000000
+  if (1) {
+      PyObject *str, *ret;
+      str = PyUnicode_FromString(name);
+      ret = PyObject_GenericGetAttr((PyObject *)self, str);
+      Py_DECREF(str);
+      return ret;
+  }
+#else
   return Py_FindMethod(SciPyLU_methods, (PyObject *)self, name);
+#endif
 }
 
 
@@ -195,8 +216,12 @@ solve\n\
 ";
 
 PyTypeObject SciPySuperLUType = {
+#if defined(NPY_PY3K)
+  PyVarObject_HEAD_INIT(NULL, 0)
+#else
   PyObject_HEAD_INIT(NULL)
   0,
+#endif
   "factored_lu",
   sizeof(SciPyLUObject),
   0,
@@ -204,7 +229,7 @@ PyTypeObject SciPySuperLUType = {
   0,				/* tp_print */
   (getattrfunc)SciPyLU_getattr,  /* tp_getattr */
   0,				/* tp_setattr */
-  0,				/* tp_compare */
+  0,				/* tp_compare / tp_reserved */
   0,				/* tp_repr */
   0,				/* tp_as_number*/
   0,				/* tp_as_sequence*/
@@ -215,8 +240,36 @@ PyTypeObject SciPySuperLUType = {
   0,				/* tp_getattro */
   0,				/* tp_setattro */
   0,				/* tp_as_buffer */
-  0,				/* tp_flags */
+  Py_TPFLAGS_DEFAULT,		/* tp_flags */
   factored_lu_doc,		/* tp_doc */
+  0,                            /* tp_traverse */
+  0,                            /* tp_clear */
+  0,                            /* tp_richcompare */
+  0,                            /* tp_weaklistoffset */
+  0,                            /* tp_iter */
+  0,                            /* tp_iternext */
+  SciPyLU_methods,              /* tp_methods */
+  0,                            /* tp_members */
+  0,                            /* tp_getset */
+  0,                            /* tp_base */
+  0,                            /* tp_dict */
+  0,                            /* tp_descr_get */
+  0,                            /* tp_descr_set */
+  0,                            /* tp_dictoffset */
+  0,                            /* tp_init */
+  0,                            /* tp_alloc */
+  0,                            /* tp_new */
+  0,                            /* tp_free */
+  0,                            /* tp_is_gc */
+  0,                            /* tp_bases */
+  0,                            /* tp_mro */
+  0,                            /* tp_cache */
+  0,                            /* tp_subclasses */
+  0,                            /* tp_weaklist */
+  0,                            /* tp_del */
+#if PY_VERSION_HEX >= 0x02060000
+  0,                            /* tp_version_tag */
+#endif
 };
 
 
@@ -429,20 +482,31 @@ fail:
 #define ENUM_CHECK_INIT                         \
     long i = -1;                                \
     char *s = "";                               \
+    PyObject *tmpobj = NULL;                    \
     if (input == Py_None) return 1;             \
     if (PyString_Check(input)) {                \
         s = PyString_AS_STRING(input);          \
     }                                           \
-    if (PyInt_Check(input)) {                   \
+    else if (PyUnicode_Check(input)) {          \
+        tmpobj = PyUnicode_AsASCIIString(input);\
+        if (tmpobj == NULL) return 0;           \
+        s = PyString_AS_STRING(tmpobj);         \
+    }                                           \
+    else if (PyInt_Check(input)) {              \
         i = PyInt_AsLong(input);                \
     }
 
 #define ENUM_CHECK_FINISH(message)              \
+    Py_XDECREF(tmpobj);                         \
     PyErr_SetString(PyExc_ValueError, message); \
     return 0;
 
-#define ENUM_CHECK(name) \
-    if (my_strxcmp(s, #name) == 0 || i == (long)name) { *value = name; return 1; }
+#define ENUM_CHECK(name)                                \
+    if (my_strxcmp(s, #name) == 0 || i == (long)name) { \
+        *value = name;                                  \
+        Py_XDECREF(tmpobj);                             \
+        return 1;                                       \
+    }
 
 /*
  * Compare strings ignoring case, underscores and whitespace
@@ -583,6 +647,18 @@ static int droprule_cvt(PyObject *input, int *value)
         if (seq == NULL || !PySequence_Check(seq))
             goto fail;
     }
+    else if (PyUnicode_Check(input)) {
+        /* Comma-separated string */
+        PyObject *s;
+        int ret;
+        s = PyUnicode_AsASCIIString(input);
+        if (s == NULL) {
+            goto fail;
+        }
+        ret = droprule_cvt(s, value);
+        Py_DECREF(s);
+        return ret;
+    }
     else if (PySequence_Check(input)) {
         /* Sequence of strings or integers */
         seq = input;
