@@ -13,6 +13,7 @@ Run tests if sparse is not installed:
   python tests/test_sparse.py
 """
 
+import sys
 import warnings
 
 import numpy as np
@@ -133,9 +134,11 @@ class _TestCommon:
 
     def test_getrow(self):
         assert_array_equal(self.datsp.getrow(1).todense(), self.dat[1,:])
+        assert_array_equal(self.datsp.getrow(-1).todense(), self.dat[-1,:])
 
     def test_getcol(self):
         assert_array_equal(self.datsp.getcol(1).todense(), self.dat[:,1])
+        assert_array_equal(self.datsp.getcol(-1).todense(), self.dat[:,-1])
 
     def test_sum(self):
         """Does the matrix's .sum(axis=...) method work?
@@ -616,10 +619,10 @@ class _TestGetSet:
 
         for ij in [(0,4),(-1,4),(3,0),(3,4),(3,-1)]:
             assert_raises(IndexError, A.__setitem__, ij, 123.0)
-        
+
         for v in [[1,2,3], array([1,2,3])]:
             assert_raises(ValueError, A.__setitem__, (0,0), v)
-        
+
         for v in [3j]:
             assert_raises(TypeError, A.__setitem__, (0,0), v)
 
@@ -761,7 +764,7 @@ class _TestFancyIndexing:
             assert_almost_equal(A[i, j], 1)
 
         # [i,j]
-        for i, j in [(2, 3), (-1, 8), (-1, -2), (array(-1), -2), (-1, array(-2)), 
+        for i, j in [(2, 3), (-1, 8), (-1, -2), (array(-1), -2), (-1, array(-2)),
                      (array(-1), array(-2))]:
             _test_set(i, j, 1)
 
@@ -868,7 +871,7 @@ class _TestFancyIndexing:
         s = slice(int8(2),int8(4),None)
         assert_equal(A[s,:].todense(), B[2:4,:])
         assert_equal(A[:,s].todense(), B[:,2:4])
-    
+
     def test_fancy_indexing_randomized(self):
         random.seed(0) # make runs repeatable
 
@@ -881,9 +884,9 @@ class _TestFancyIndexing:
 
         I = np.random.random_integers(-M + 1, M - 1, size=NUM_SAMPLES)
         J = np.random.random_integers(-N + 1, N - 1, size=NUM_SAMPLES)
-        
+
         S = self.spmatrix(D)
-        
+
         assert_equal(S[I,J], D[I,J])
 
         I_bad = I + M
@@ -1300,7 +1303,7 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
 
     def test_resize(self):
         """A couple basic tests of the resize() method.
-        
+
         resize(shape) resizes the array in-place.
         """
         a = dok_matrix((5,5))
@@ -1320,12 +1323,106 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
         # This assert would fail, because the above assignment would
         # incorrectly call __set_item__ even though the value was 0.
         assert_((0,0) not in a.keys(), "Unexpected entry (0,0) in keys")
-        
+
         # Slice assignments were also affected.
         b = dok_matrix((3,3))
         b[:,0] = 0
         assert_(len(b.keys())==0, "Unexpected entries in keys")
 
+    # The following five tests are duplicates from _TestCommon, so they can be
+    # marked as knownfail for Python 2.4.  Once 2.4 is no longer supported,
+    # these duplicates can be removed again.
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_add_dense(self):
+        """ adding a dense matrix to a sparse matrix
+        """
+        sum1 = self.dat + self.datsp
+        assert_array_equal(sum1, 2*self.dat)
+        sum2 = self.datsp + self.dat
+        assert_array_equal(sum2, 2*self.dat)
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_radd(self):
+        a = self.dat.copy()
+        a[0,2] = 2.0
+        b = self.datsp
+        c = a + b
+        assert_array_equal(c,[[2,0,2,4],[6,0,2,0],[0,4,0,0]])
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_rsub(self):
+        assert_array_equal((self.dat - self.datsp),[[0,0,0,0],[0,0,0,0],[0,0,0,0]])
+        assert_array_equal((self.datsp - self.dat),[[0,0,0,0],[0,0,0,0],[0,0,0,0]])
+
+        A = self.spmatrix(matrix([[1,0,0,4],[-1,0,0,0],[0,8,0,-5]],'d'))
+        assert_array_equal((self.dat - A),self.dat - A.todense())
+        assert_array_equal((A - self.dat),A.todense() - self.dat)
+        assert_array_equal(A.todense() - self.datsp,A.todense() - self.dat)
+        assert_array_equal(self.datsp - A.todense(),self.dat - A.todense())
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_matmat_sparse(self):
+        a = matrix([[3,0,0],[0,1,0],[2,0,3.0],[2,3,0]])
+        a2 = array([[3,0,0],[0,1,0],[2,0,3.0],[2,3,0]])
+        b = matrix([[0,1],[1,0],[0,2]],'d')
+        asp = self.spmatrix(a)
+        bsp = self.spmatrix(b)
+        assert_array_almost_equal((asp*bsp).todense(), a*b)
+        assert_array_almost_equal( asp*b, a*b)
+        assert_array_almost_equal( a*bsp, a*b)
+        assert_array_almost_equal( a2*bsp, a*b)
+
+        # Now try performing cross-type multplication:
+        csp = bsp.tocsc()
+        c = b
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+        csp = bsp.tocsr()
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+        csp = bsp.tocoo()
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+
+        # Test provided by Andy Fraser, 2006-03-26
+        L = 30
+        frac = .3
+        random.seed(0) # make runs repeatable
+        A = zeros((L,2))
+        for i in xrange(L):
+            for j in xrange(2):
+                r = random.random()
+                if r < frac:
+                    A[i,j] = r/frac
+
+        A = self.spmatrix(A)
+        B = A*A.T
+        assert_array_almost_equal(B.todense(), A.todense() * A.T.todense())
+        assert_array_almost_equal(B.todense(), A.todense() * A.todense().T)
+
+        # check dimension mismatch  2x2 times 3x2
+        A = self.spmatrix( [[1,2],[3,4]] )
+        B = self.spmatrix( [[1,2],[3,4],[5,6]] )
+        assert_raises(ValueError, A.__mul__, B)
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_sub_dense(self):
+        """ subtracting a dense matrix to/from a sparse matrix
+        """
+        sum1 = 3*self.dat - self.datsp
+        assert_array_equal(sum1, 2*self.dat)
+        sum2 = 3*self.datsp - self.dat
+        assert_array_equal(sum2, 2*self.dat)
 
 
 class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
@@ -1340,7 +1437,7 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
     B[2,1] = 3
     B[3,0] = 10
 
-    
+
     @dec.knownfailureif(True, "Fancy indexing is known to be broken for LIL" \
                               " matrices")
     def test_fancy_indexing_set(self):
