@@ -1,32 +1,72 @@
 # Copyright Anne M. Archibald 2008
 # Released under the scipy license
+import sys
 import numpy as np
 from heapq import heappush, heappop
 import scipy.sparse
 
-def minkowski_distance_p(x,y,p=2):
-    """Compute the pth power of the L**p distance between x and y
+__all__ = ['minkowski_distance_p', 'minkowski_distance',
+           'distance_matrix',
+           'Rectangle', 'KDTree']
+
+
+def minkowski_distance_p(x, y, p=2):
+    """
+    Compute the p-th power of the L**p distance between x and y.
 
     For efficiency, this function computes the L**p distance but does
     not extract the pth root. If p is 1 or infinity, this is equal to
     the actual L**p distance.
+
+    Parameters
+    ----------
+    x : array_like, M by K
+
+    y : array_like, N by K
+
+    p : float, 1 <= p <= infinity
+        Which Minkowski p-norm to use.
+
+    Examples
+    --------
+    >>> minkowski_distance_p([[0,0],[0,0]], [[1,1],[0,1]])
+    array([2, 1])
+
     """
     x = np.asarray(x)
     y = np.asarray(y)
-    if p==np.inf:
-        return np.amax(np.abs(y-x),axis=-1)
-    elif p==1:
-        return np.sum(np.abs(y-x),axis=-1)
+    if p == np.inf:
+        return np.amax(np.abs(y-x), axis=-1)
+    elif p == 1:
+        return np.sum(np.abs(y-x), axis=-1)
     else:
-        return np.sum(np.abs(y-x)**p,axis=-1)
-def minkowski_distance(x,y,p=2):
-    """Compute the L**p distance between x and y"""
+        return np.sum(np.abs(y-x)**p, axis=-1)
+
+def minkowski_distance(x, y, p=2):
+    """
+    Compute the L**p distance between x and y.
+
+    Parameters
+    ----------
+    x : array_like, M by K
+
+    y : array_like, N by K
+
+    p : float, 1 <= p <= infinity
+        Which Minkowski p-norm to use.
+
+    Examples
+    --------
+    >>> minkowski_distance([[0,0],[0,0]], [[1,1],[0,1]])
+    array([ 1.41421356,  1.        ])
+
+    """
     x = np.asarray(x)
     y = np.asarray(y)
-    if p==np.inf or p==1:
-        return minkowski_distance_p(x,y,p)
+    if p == np.inf or p == 1:
+        return minkowski_distance_p(x, y, p)
     else:
-        return minkowski_distance_p(x,y,p)**(1./p)
+        return minkowski_distance_p(x, y, p)**(1./p)
 
 class Rectangle(object):
     """Hyperrectangle class.
@@ -111,17 +151,15 @@ class KDTree(object):
     sort of calculation.
 
     """
-
     def __init__(self, data, leafsize=10):
         """Construct a kd-tree.
 
-        Parameters:
-        ===========
-
-        data : array-like, shape (n,k)
+        Parameters
+        ----------
+        data : array_like, shape (n,k)
             The data points to be indexed. This array is not copied, and
             so modifying this data will result in bogus results.
-        leafsize : positive integer
+        leafsize : positive int
             The number of points at which the algorithm switches over to
             brute-force.
         """
@@ -136,11 +174,18 @@ class KDTree(object):
         self.tree = self.__build(np.arange(self.n), self.maxes, self.mins)
 
     class node(object):
-        pass
+        if sys.version_info[0] >= 3:
+            def __lt__(self, other): id(self) < id(other)
+            def __gt__(self, other): id(self) > id(other)
+            def __le__(self, other): id(self) <= id(other)
+            def __ge__(self, other): id(self) >= id(other)
+            def __eq__(self, other): id(self) == id(other)
+
     class leafnode(node):
         def __init__(self, idx):
             self.idx = idx
             self.children = len(idx)
+
     class innernode(node):
         def __init__(self, split_dim, split, less, greater):
             self.split_dim = split_dim
@@ -179,7 +224,8 @@ class KDTree(object):
                 greater_idx = np.nonzero(data>=split)[0]
             if len(less_idx)==0:
                 # _still_ zero? all must have the same value
-                assert np.all(data==data[0]), "Troublesome data array: %s" % data
+                if not np.all(data==data[0]):
+                    raise ValueError("Troublesome data array: %s" % data)
                 split = data[0]
                 less_idx = np.arange(len(data)-1)
                 greater_idx = np.array([len(data)-1])
@@ -276,12 +322,11 @@ class KDTree(object):
 
     def query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf):
         """
-        query the kd-tree for nearest neighbors
+        Query the kd-tree for nearest neighbors
 
         Parameters
         ----------
-
-        x : array-like, last dimension self.m
+        x : array_like, last dimension self.m
             An array of points to query.
         k : integer
             The number of nearest neighbors to return.
@@ -302,7 +347,6 @@ class KDTree(object):
 
         Returns
         -------
-
         d : array of floats
             The distances to the nearest neighbors.
             If x has shape tuple+(self.m,), then d has shape tuple if
@@ -317,7 +361,6 @@ class KDTree(object):
 
         Examples
         --------
-
         >>> from scipy.spatial import KDTree
         >>> x, y = np.mgrid[0:5, 2:8]
         >>> tree = KDTree(zip(x.ravel(), y.ravel()))
@@ -364,7 +407,10 @@ class KDTree(object):
             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
         retshape = np.shape(x)[:-1]
         if retshape!=():
-            if k>1:
+            if k is None:
+                dd = np.empty(retshape,dtype=np.object)
+                ii = np.empty(retshape,dtype=np.object)
+            elif k>1:
                 dd = np.empty(retshape+(k,),dtype=np.float)
                 dd.fill(np.inf)
                 ii = np.empty(retshape+(k,),dtype=np.int)
@@ -374,14 +420,14 @@ class KDTree(object):
                 dd.fill(np.inf)
                 ii = np.empty(retshape,dtype=np.int)
                 ii.fill(self.n)
-            elif k is None:
-                dd = np.empty(retshape,dtype=np.object)
-                ii = np.empty(retshape,dtype=np.object)
             else:
                 raise ValueError("Requested %s nearest neighbors; acceptable numbers are integers greater than or equal to one, or None")
             for c in np.ndindex(retshape):
                 hits = self.__query(x[c], k=k, p=p, distance_upper_bound=distance_upper_bound)
-                if k>1:
+                if k is None:
+                    dd[c] = [d for (d,i) in hits]
+                    ii[c] = [i for (d,i) in hits]
+                elif k>1:
                     for j in range(len(hits)):
                         dd[c+(j,)], ii[c+(j,)] = hits[j]
                 elif k==1:
@@ -390,13 +436,12 @@ class KDTree(object):
                     else:
                         dd[c] = np.inf
                         ii[c] = self.n
-                elif k is None:
-                    dd[c] = [d for (d,i) in hits]
-                    ii[c] = [i for (d,i) in hits]
             return dd, ii
         else:
             hits = self.__query(x, k=k, p=p, distance_upper_bound=distance_upper_bound)
-            if k==1:
+            if k is None:
+                return [d for (d,i) in hits], [i for (d,i) in hits]
+            elif k==1:
                 if len(hits)>0:
                     return hits[0]
                 else:
@@ -409,8 +454,6 @@ class KDTree(object):
                 for j in range(len(hits)):
                     dd[j], ii[j] = hits[j]
                 return dd, ii
-            elif k is None:
-                return [d for (d,i) in hits], [i for (d,i) in hits]
             else:
                 raise ValueError("Requested %s nearest neighbors; acceptable numbers are integers greater than or equal to one, or None")
 
@@ -419,62 +462,76 @@ class KDTree(object):
         R = Rectangle(self.maxes, self.mins)
 
         def traverse_checking(node, rect):
-            if rect.min_distance_point(x,p)>=r/(1.+eps):
+            if rect.min_distance_point(x, p) > r / (1. + eps):
                 return []
-            elif rect.max_distance_point(x,p)<r*(1.+eps):
+            elif rect.max_distance_point(x, p) < r * (1. + eps):
                 return traverse_no_checking(node)
             elif isinstance(node, KDTree.leafnode):
                 d = self.data[node.idx]
-                return node.idx[minkowski_distance(d,x,p)<=r].tolist()
+                return node.idx[minkowski_distance(d, x, p) <= r].tolist()
             else:
                 less, greater = rect.split(node.split_dim, node.split)
-                return traverse_checking(node.less, less)+traverse_checking(node.greater, greater)
+                return traverse_checking(node.less, less) + \
+                       traverse_checking(node.greater, greater)
+
         def traverse_no_checking(node):
             if isinstance(node, KDTree.leafnode):
-
                 return node.idx.tolist()
             else:
-                return traverse_no_checking(node.less)+traverse_no_checking(node.greater)
+                return traverse_no_checking(node.less) + \
+                       traverse_no_checking(node.greater)
 
         return traverse_checking(self.tree, R)
 
     def query_ball_point(self, x, r, p=2., eps=0):
-        """Find all points within r of x
+        """Find all points within distance r of point(s) x.
 
         Parameters
-        ==========
-
+        ----------
         x : array_like, shape tuple + (self.m,)
-            The point or points to search for neighbors of
+            The point or points to search for neighbors of.
         r : positive float
-            The radius of points to return
-        p : float 1<=p<=infinity
-            Which Minkowski p-norm to use
-        eps : nonnegative float
-            Approximate search. Branches of the tree are not explored
-            if their nearest points are further than r/(1+eps), and branches
-            are added in bulk if their furthest points are nearer than r*(1+eps).
+            The radius of points to return.
+        p : float, optional
+            Which Minkowski p-norm to use.  Should be in the range [1, inf].
+        eps : nonnegative float, optional
+            Approximate search. Branches of the tree are not explored if their
+            nearest points are further than ``r / (1 + eps)``, and branches are
+            added in bulk if their furthest points are nearer than
+            ``r * (1 + eps)``.
 
         Returns
-        =======
-
+        -------
         results : list or array of lists
-            If x is a single point, returns a list of the indices of the neighbors
-            of x. If x is an array of points, returns an object array of shape tuple
-            containing lists of neighbors.
+            If `x` is a single point, returns a list of the indices of the
+            neighbors of `x`. If `x` is an array of points, returns an object
+            array of shape tuple containing lists of neighbors.
 
+        Notes
+        -----
+        If you have many points whose neighbors you want to find, you may save
+        substantial amounts of time by putting them in a KDTree and using
+        query_ball_tree.
+
+        Examples
+        --------
+        >>> from scipy import spatial
+        >>> x, y = np.mgrid[0:4, 0:4]
+        >>> points = zip(x.ravel(), y.ravel())
+        >>> tree = spatial.KDTree(points)
+        >>> tree.query_ball_point([2, 0], 1)
+        [4, 8, 9, 12]
 
-        Note: if you have many points whose neighbors you want to find, you may save
-        substantial amounts of time by putting them in a KDTree and using query_ball_tree.
         """
         x = np.asarray(x)
-        if x.shape[-1]!=self.m:
-            raise ValueError("Searching for a %d-dimensional point in a %d-dimensional KDTree" % (x.shape[-1],self.m))
-        if len(x.shape)==1:
-            return self.__query_ball_point(x,r,p,eps)
+        if x.shape[-1] != self.m:
+            raise ValueError("Searching for a %d-dimensional point in a " \
+                             "%d-dimensional KDTree" % (x.shape[-1], self.m))
+        if len(x.shape) == 1:
+            return self.__query_ball_point(x, r, p, eps)
         else:
             retshape = x.shape[:-1]
-            result = np.empty(retshape,dtype=np.object)
+            result = np.empty(retshape, dtype=np.object)
             for c in np.ndindex(retshape):
                 result[c] = self.__query_ball_point(x[c], r, p=p, eps=eps)
             return result
@@ -586,12 +643,8 @@ class KDTree(object):
                 # This node pair will be visited in the other order
                 #return
                 pass
-            if rect1.min_distance_rectangle(rect2, p)>r/(1.+eps):
-                return
-            elif rect1.max_distance_rectangle(rect2, p)<r*(1.+eps):
-                traverse_no_checking(node1.less, node2)
-                traverse_no_checking(node1.greater, node2)
-            elif isinstance(node1, KDTree.leafnode):
+
+            if isinstance(node1, KDTree.leafnode):
                 if isinstance(node2, KDTree.leafnode):
                     d = self.data[node2.idx]
                     for i in node1.idx:
@@ -608,6 +661,11 @@ class KDTree(object):
                 less, greater = rect1.split(node1.split_dim, node1.split)
                 traverse_checking(node1.less,less,node2,rect2)
                 traverse_checking(node1.greater,greater,node2,rect2)
+            elif rect1.min_distance_rectangle(rect2, p)>r/(1.+eps):
+                return
+            elif rect1.max_distance_rectangle(rect2, p)<r*(1.+eps):
+                traverse_no_checking(node1.less, node2)
+                traverse_no_checking(node1.greater, node2)
             else:
                 less1, greater1 = rect1.split(node1.split_dim, node1.split)
                 less2, greater2 = rect2.split(node2.split_dim, node2.split)
@@ -773,26 +831,32 @@ class KDTree(object):
 
 
 def distance_matrix(x,y,p=2,threshold=1000000):
-    """Compute the distance matrix.
+    """
+    Compute the distance matrix.
 
-    Computes the matrix of all pairwise distances.
+    Returns the matrix of all pair-wise distances.
 
     Parameters
-    ==========
-
-    x : array-like, m by k
-    y : array-like, n by k
-    p : float 1<=p<=infinity
+    ----------
+    x : array_like, `M` by `K`
+        TODO: description needed
+    y : array_like, `N` by `K`
+        TODO: description needed
+    p : float, 1 <= p <= infinity
         Which Minkowski p-norm to use.
     threshold : positive integer
-        If m*n*k>threshold use a python loop instead of creating
-        a very large temporary.
+        If `M * N * K` > threshold, use a Python loop instead of creating
+        a very large temporary [what?  array?].
 
     Returns
-    =======
-
-    result : array-like, m by n
+    -------
+    result : array_like, `M` by `N`
 
+    Examples
+    --------
+    >>> distance_matrix([[0,0],[0,1]], [[1,0],[1,1]])
+    array([[ 1.        ,  1.41421356],
+           [ 1.41421356,  1.        ]])
 
     """
 
