@@ -1,16 +1,16 @@
-
 /* Cephes module version 1.5
  *  This module defines the functions in the cephes and amos libraries as
- *   Numerical python ufunc objects so that they can operate on arbitrary 
+ *   Numerical python ufunc objects so that they can operate on arbitrary
  *   NumPy arrays with broadcasting and typecasting rules implemented.
- *  
+ *
  *  Copyright 1999  Travis E. Oliphant
  * Revisions 2002 (added functions from cdflib)
  */
+#include <stdarg.h>
 
 #include "Python.h"
 #include "numpy/arrayobject.h"
-#include "numpy/ufuncobject.h" 
+#include "numpy/ufuncobject.h"
 #include "ufunc_extras.h"
 #include "abstract.h"
 #include "cephes.h"
@@ -22,23 +22,26 @@
 
 /* Defined in mtherr in the cephes library */
 extern int scipy_special_print_error_messages;
- 
+
 #include "cephes_doc.h"
 
-
+#if PY_VERSION_HEX >= 0x03000000
+#define PyInt_FromLong PyLong_FromLong
+#define PyUString_FromString PyUnicode_FromString
+#else
+#define PyUString_FromString PyString_FromString
+#endif
 static PyUFuncGenericFunction cephes1_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes1rc_functions[] = { NULL, NULL, NULL, NULL};
 static PyUFuncGenericFunction cephes1_2_functions[] = { NULL, NULL, NULL, NULL,};
 static PyUFuncGenericFunction cephes1_2c_functions[] = { NULL, NULL,};
 static PyUFuncGenericFunction cephes1c_4_functions[] = { NULL, NULL, NULL, NULL };
-static PyUFuncGenericFunction cephes1cp_4_functions[] = { NULL, NULL, NULL, NULL};
 static PyUFuncGenericFunction cephes1cpb_4_functions[] = { NULL, NULL,};
 static PyUFuncGenericFunction cephes2_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes2_2_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes2_4_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes2a_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes2c_functions[] = { NULL, NULL, NULL, NULL };
-static PyUFuncGenericFunction cephes2cp_functions[] = { NULL, NULL, NULL, NULL, };
 static PyUFuncGenericFunction cephes2cpp_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes3_functions[] = { NULL, NULL, NULL, NULL};
 static PyUFuncGenericFunction cephes3a_functions[] = { NULL, NULL, };
@@ -51,7 +54,7 @@ static PyUFuncGenericFunction cephes5_2_functions[] = { NULL, NULL, };
 static PyUFuncGenericFunction cephes1c_functions[] = { NULL, NULL, };
 
 static void * airy_data[] = { (void *)airy, (void *)airy, (void *)cairy_wrap, (void *)cairy_wrap,};
-static void * airye_data[] = { (void *)cairy_wrap_e, (void *)cairy_wrap_e, (void *)cairy_wrap_e, (void *)cairy_wrap_e, };
+static void * airye_data[] = { (void *)cairy_wrap_e_real, (void *)cairy_wrap_e_real, (void *)cairy_wrap_e, (void *)cairy_wrap_e, };
 static void * itairy_data[] = { (void *)itairy_wrap, (void *)itairy_wrap, };
 
 static void * kelvin_data[] = { (void *)kelvin_wrap, (void *)kelvin_wrap,};
@@ -67,7 +70,7 @@ static void * keip_data[] = { (void *)keip_wrap, (void *)keip_wrap,};
 static void * ellpj_data[] = { (void *)ellpj, (void *)ellpj,};
 
 static void * exp1_data[] = { (void *)exp1_wrap, (void *)exp1_wrap, (void *)cexp1_wrap, (void *)cexp1_wrap,};
-static void * expi_data[] = { (void *)expi_wrap, (void *)expi_wrap,};
+static void * expi_data[] = { (void *)expi_wrap, (void *)expi_wrap, (void *)cexpi_wrap, (void *)cexpi_wrap,};
 static void * expn_data[] = { (void *)expn, (void *)expn, };
 static void * kn_data[] = { (void *)kn, (void *)kn, };
 
@@ -110,7 +113,7 @@ static void * gdtr_data[] = { (void *)gdtr, (void *)gdtr, };
 static void * gdtri_data[] = { (void *)gdtri, (void *)gdtri, };
 */
 static void * hyp2f1_data[] = { (void *)hyp2f1, (void *)hyp2f1, (void *)chyp2f1_wrap, (void *)chyp2f1_wrap};
-static void * hyperg_data[] = { (void *)hyperg, (void *)hyperg, (void *)chyp1f1_wrap, (void *)chyp1f1_wrap};
+static void * hyp1f1_data[] = { (void *)hyp1f1_wrap, (void *)hyp1f1_wrap, (void *)chyp1f1_wrap, (void *)chyp1f1_wrap};
 static void * hypU_data[] = { (void *)hypU_wrap, (void *)hypU_wrap, };
 static void * hyp2f0_data[] = { (void *)hyp2f0, (void *)hyp2f0, };
 static void * threef0_data[] = { (void *)threef0, (void *)threef0, };
@@ -145,24 +148,26 @@ static void * i1e_data[] = { (void *)i1e, (void *)i1e, };
 static void * igamc_data[] = { (void *)igamc, (void *)igamc, };
 static void * igam_data[] = { (void *)igam, (void *)igam, };
 static void * igami_data[] = { (void *)igami, (void *)igami, };
+static void * gammaincinv_data[] = { (void *)gammaincinv,
+                                     (void *)gammaincinv, };
 
 static void * iv_data[] = { (void *)iv, (void *)iv, (void *)cbesi_wrap, (void *)cbesi_wrap,};
-static void * ive_data[] = { (void *)cbesi_wrap_e, (void *)cbesi_wrap_e, (void *)cbesi_wrap_e, (void *)cbesi_wrap_e, };
+static void * ive_data[] = { (void *)cbesi_wrap_e_real, (void *)cbesi_wrap_e_real, (void *)cbesi_wrap_e, (void *)cbesi_wrap_e, };
 static void * j0_data[] = { (void *)j0,  (void *)j0,  };
 static void * y0_data[] = { (void *)y0, (void *)y0, };
 static void * j1_data[] = { (void *)j1,  (void *)j1,  };
 static void * y1_data[] = { (void *)y1, (void *)y1, };
 static void * jv_data[] = { (void *)jv, (void *)jv, (void *)cbesj_wrap, (void *)cbesj_wrap,};
-static void * jve_data[] = { (void *)cbesj_wrap_e, (void *)cbesj_wrap_e, (void *)cbesj_wrap_e, (void *)cbesj_wrap_e, };
+static void * jve_data[] = { (void *)cbesj_wrap_e_real, (void *)cbesj_wrap_e_real, (void *)cbesj_wrap_e, (void *)cbesj_wrap_e, };
 static void * yv_data[] = { (void *)yv, (void *)yv, (void *)cbesy_wrap, (void *)cbesy_wrap,};
-static void * yve_data[] = { (void *)cbesy_wrap_e, (void *)cbesy_wrap_e, (void *)cbesy_wrap_e, (void *)cbesy_wrap_e, };
+static void * yve_data[] = { (void *)cbesy_wrap_e_real, (void *)cbesy_wrap_e_real, (void *)cbesy_wrap_e, (void *)cbesy_wrap_e, };
 
 static void * k0_data[] = { (void *)k0, (void *)k0, };
 static void * k0e_data[] = { (void *)k0e, (void *)k0e, };
 static void * k1_data[] = { (void *)k1, (void *)k1, };
 static void * k1e_data[] = { (void *)k1e, (void *)k1e, };
-static void * kv_data[] = { (void *)cbesk_wrap, (void *)cbesk_wrap, (void *)cbesk_wrap, (void *)cbesk_wrap,};
-static void * kve_data[] = { (void *)cbesk_wrap_e, (void *)cbesk_wrap_e, (void *)cbesk_wrap_e, (void *)cbesk_wrap_e,};
+static void * kv_data[] = { (void *)cbesk_wrap_real, (void *)cbesk_wrap_real, (void *)cbesk_wrap, (void *)cbesk_wrap,};
+static void * kve_data[] = { (void *)cbesk_wrap_e_real, (void *)cbesk_wrap_e_real, (void *)cbesk_wrap_e, (void *)cbesk_wrap_e,};
 static void * hankel1_data[] = { (void *)cbesh_wrap1, (void *)cbesh_wrap1,};
 static void * hankel1e_data[] = { (void *)cbesh_wrap1_e, (void *)cbesh_wrap1_e,};
 static void * hankel2_data[] = { (void *)cbesh_wrap2, (void *)cbesh_wrap2,};
@@ -306,7 +311,7 @@ static char cephes_1rc_types[] = { PyArray_FLOAT,  PyArray_FLOAT,  PyArray_DOUBL
 static char cephes_1c_types[] = { PyArray_CFLOAT, PyArray_CFLOAT, PyArray_CDOUBLE, PyArray_CDOUBLE, };
 
 
-/* Some functions needed from ufunc object, so that Py_complex's aren't being returned 
+/* Some functions needed from ufunc object, so that Py_complex's aren't being returned
 between code possibly compiled with different compilers.
 */
 
@@ -355,10 +360,6 @@ static void Cephes_InitOperators(PyObject *dictionary) {
         cephes1c_4_functions[1] = PyUFunc_d_dddd;
         cephes1c_4_functions[2] = PyUFunc_F_FFFF_As_D_DDDD;
         cephes1c_4_functions[3] = PyUFunc_D_DDDD;
-        cephes1cp_4_functions[0] = PyUFunc_f_ffff_As_D_DDDD;
-        cephes1cp_4_functions[1] = PyUFunc_d_dddd_As_D_DDDD;
-        cephes1cp_4_functions[2] = PyUFunc_F_FFFF_As_D_DDDD;
-        cephes1cp_4_functions[3] = PyUFunc_D_DDDD;
         cephes1cpb_4_functions[0] = PyUFunc_f_FFFF_As_d_DDDD;
         cephes1cpb_4_functions[1] = PyUFunc_d_DDDD;
         cephes2_functions[0] = PyUFunc_ff_f_As_dd_d;
@@ -371,10 +372,6 @@ static void Cephes_InitOperators(PyObject *dictionary) {
         cephes2c_functions[1] = PyUFunc_dd_d;
         cephes2c_functions[2] = PyUFunc_fF_F_As_dD_D;
         cephes2c_functions[3] = PyUFunc_dD_D;
-        cephes2cp_functions[0] = PyUFunc_ff_f_As_dD_D;
-        cephes2cp_functions[1] = PyUFunc_dd_d_As_dD_D;
-        cephes2cp_functions[2] = PyUFunc_fF_F_As_dD_D;
-        cephes2cp_functions[3] = PyUFunc_dD_D;
         cephes2cpp_functions[0] = PyUFunc_fF_F_As_dD_D;
         cephes2cpp_functions[1] = PyUFunc_dD_D;
         cephes2_4_functions[0] = PyUFunc_ff_ffff_As_dd_dddd;
@@ -397,7 +394,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
         cephes4a_2_functions[1] = PyUFunc_dddd_dd_As_dddi_dd;
         cephes5_2_functions[0] = PyUFunc_fffff_ff_As_ddddd_dd;
         cephes5_2_functions[1] = PyUFunc_ddddd_dd;
-	
+
 	/* Create function objects for each function call and insert
 	   them in the dictionary */
 	f = PyUFunc_FromFuncAndData(cephes3a_functions, bdtrc_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "bdtrc", bdtrc_doc, 0);
@@ -424,7 +421,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	Py_DECREF(f);
 	f = PyUFunc_FromFuncAndData(cephes3_functions, fdtri_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "fdtri", fdtri_doc, 0);
 	PyDict_SetItemString(dictionary, "fdtri", f);
- 	Py_DECREF(f);
+	Py_DECREF(f);
 
 	f = PyUFunc_FromFuncAndData(cephes3_functions, gdtrc_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "gdtrc", gdtrc_doc, 0);
 	PyDict_SetItemString(dictionary, "gdtrc", f);
@@ -441,7 +438,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	f = PyUFunc_FromFuncAndData(cephes4_functions, hyp2f1_data, cephes_5c2_types, 4, 4, 1, PyUFunc_None, "hyp2f1", hyp2f1_doc, 0);
 	PyDict_SetItemString(dictionary, "hyp2f1", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes3_functions, hyperg_data, cephes_4c_types, 4, 3, 1, PyUFunc_None, "hyp1f1", hyp1f1_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes3_functions, hyp1f1_data, cephes_4c_types, 4, 3, 1, PyUFunc_None, "hyp1f1", hyp1f1_doc, 0);
 	PyDict_SetItemString(dictionary, "hyp1f1", f);
 	Py_DECREF(f);
 
@@ -507,8 +504,8 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	f = PyUFunc_FromFuncAndData(cephes1_functions, ellpe_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "ellipe", ellipe_doc, 0);
 	PyDict_SetItemString(dictionary, "ellipe", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes1_functions, ellpk_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "ellipk", ellipk_doc, 0);
-	PyDict_SetItemString(dictionary, "ellipk", f);
+	f = PyUFunc_FromFuncAndData(cephes1_functions, ellpk_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "ellipkm1", ellipkm1_doc, 0);
+	PyDict_SetItemString(dictionary, "ellipkm1", f);
 	Py_DECREF(f);
 	f = PyUFunc_FromFuncAndData(cephes1_functions, exp10_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "exp10", exp10_doc, 0);
 	PyDict_SetItemString(dictionary, "exp10", f);
@@ -545,10 +542,16 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	f = PyUFunc_FromFuncAndData(cephes2_functions, igami_data, cephes_3_types, 2, 2, 1, PyUFunc_None, "gammainccinv", gammainccinv_doc, 0);
 	PyDict_SetItemString(dictionary, "gammainccinv", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2c_functions, iv_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "iv", iv_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2_functions, gammaincinv_data,
+                                    cephes_3_types, 2, 2, 1, PyUFunc_None,
+                                    "gammaincinv", gammaincinv_doc, 0);
+	PyDict_SetItemString(dictionary, "gammaincinv", f);
+	Py_DECREF(f);
+
+        f = PyUFunc_FromFuncAndData(cephes2c_functions, iv_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "iv", iv_doc, 0);
 	PyDict_SetItemString(dictionary, "iv", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2cp_functions, ive_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "ive", ive_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2c_functions, ive_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "ive", ive_doc, 0);
 	PyDict_SetItemString(dictionary, "ive", f);
 	Py_DECREF(f);
 
@@ -562,7 +565,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	f = PyUFunc_FromFuncAndData(cephes1rc_functions, exp1_data, cephes_1rc_types, 4, 1, 1, PyUFunc_None, "exp1", exp1_doc, 0);
 	PyDict_SetItemString(dictionary, "exp1", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes1_functions, expi_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "expi", expi_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes1rc_functions, expi_data, cephes_1rc_types, 4, 1, 1, PyUFunc_None, "expi", expi_doc, 0);
 	PyDict_SetItemString(dictionary, "expi", f);
 	Py_DECREF(f);
 
@@ -580,7 +583,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	PyDict_SetItemString(dictionary, "pdtri", f);
 	Py_DECREF(f);
         /*  Use the student t library from cdflib (it supports doubles for
-              degrees of freedom 
+              degrees of freedom
 	f = PyUFunc_FromFuncAndData(cephes2a_functions, stdtr_data, cephes_3_types, 2, 2, 1, PyUFunc_None, "stdtr", stdtr_doc, 0);
 	PyDict_SetItemString(dictionary, "stdtr", f);
 	Py_DECREF(f);
@@ -607,7 +610,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	Py_DECREF(f);
 
 
-	f = PyUFunc_FromFuncAndData(cephes1cp_4_functions, airye_data, cephes_5c_types, 4, 1, 4, PyUFunc_None, "airye", airye_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes1c_4_functions, airye_data, cephes_5c_types, 4, 1, 4, PyUFunc_None, "airye", airye_doc, 0);
 	PyDict_SetItemString(dictionary, "airye", f);
 	Py_DECREF(f);
 
@@ -655,13 +658,13 @@ static void Cephes_InitOperators(PyObject *dictionary) {
            accurate. So we alias jv to jn */
         PyDict_SetItemString(dictionary, "jn", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2cp_functions, jve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "jve", jve_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2c_functions, jve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "jve", jve_doc, 0);
 	PyDict_SetItemString(dictionary, "jve", f);
 	Py_DECREF(f);
 	f = PyUFunc_FromFuncAndData(cephes2c_functions, yv_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "yv", yv_doc, 0);
 	PyDict_SetItemString(dictionary, "yv", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2cp_functions, yve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "yve", yve_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2c_functions, yve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "yve", yve_doc, 0);
 	PyDict_SetItemString(dictionary, "yve", f);
 	Py_DECREF(f);
 
@@ -678,10 +681,10 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	f = PyUFunc_FromFuncAndData(cephes1_functions, k1e_data, cephes_2_types, 2, 1, 1, PyUFunc_None, "k1e", k1e_doc, 0);
 	PyDict_SetItemString(dictionary, "k1e", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2cp_functions, kv_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "kv", kv_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2c_functions, kv_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "kv", kv_doc, 0);
 	PyDict_SetItemString(dictionary, "kv", f);
 	Py_DECREF(f);
-	f = PyUFunc_FromFuncAndData(cephes2cp_functions, kve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "kve", kve_doc, 0);
+	f = PyUFunc_FromFuncAndData(cephes2c_functions, kve_data, cephes_3c_types, 4, 2, 1, PyUFunc_None, "kve", kve_doc, 0);
 	PyDict_SetItemString(dictionary, "kve", f);
 	Py_DECREF(f);
 
@@ -812,7 +815,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	PyDict_SetItemString(dictionary, "kolmogi", f);
 	Py_DECREF(f);
 
-	f = PyUFunc_FromFuncAndData(cephes1c_functions, wofz_data, cephes_1c_types, 2, 1, 1, PyUFunc_None, "wofz", wofz_doc, 0); 
+	f = PyUFunc_FromFuncAndData(cephes1c_functions, wofz_data, cephes_1c_types, 2, 1, 1, PyUFunc_None, "wofz", wofz_doc, 0);
 	PyDict_SetItemString(dictionary, "wofz", f);
 	Py_DECREF(f);
 
@@ -858,14 +861,14 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	Py_DECREF(f);
 	f = PyUFunc_FromFuncAndData(cephes3_functions, cdff2_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "fdtrix", fdtri_doc, 0);
 	PyDict_SetItemString(dictionary, "fdtrix", f);
- 	Py_DECREF(f);
+	Py_DECREF(f);
         */
-        
+
         /*  The Fortran code for this one seems not to be working properly.
 	f = PyUFunc_FromFuncAndData(cephes3_functions, cdff3_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "fdtridfn", "", 0);
 	PyDict_SetItemString(dictionary, "fdtridfn", f);
 	Py_DECREF(f);
-        */ 
+        */
 	f = PyUFunc_FromFuncAndData(cephes3_functions, cdff4_data, cephes_4_types, 2, 3, 1, PyUFunc_None, "fdtridfd", "", 0);
 	PyDict_SetItemString(dictionary, "fdtridfd", f);
 	Py_DECREF(f);
@@ -947,7 +950,7 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 	PyDict_SetItemString(dictionary, "tklmbda", f);
 	Py_DECREF(f);
 
-        
+
 	f = PyUFunc_FromFuncAndData(cephes2_functions, mathieu_a_data, cephes_3_types, 2, 2, 1, PyUFunc_None, "mathieu_a", mathieu_a_doc, 0);
 	PyDict_SetItemString(dictionary, "mathieu_a", f);
 	Py_DECREF(f);
@@ -1044,6 +1047,23 @@ static void Cephes_InitOperators(PyObject *dictionary) {
 
 }
 
+static PyObject *scipy_special_SpecialFunctionWarning = NULL;
+
+void scipy_special_raise_warning(char *fmt, ...)
+{
+    char msg[1024];
+    va_list ap;
+    NPY_ALLOW_C_API_DEF;
+
+    va_start(ap, fmt);
+    PyOS_vsnprintf(msg, 1024, fmt, ap);
+    va_end(ap);
+
+    NPY_ALLOW_C_API;
+    PyErr_Warn(scipy_special_SpecialFunctionWarning, msg);
+    NPY_DISABLE_C_API;
+}
+
 static char errprint_doc[] = \
 "errprint({flag}) sets the error printing flag for special functions\n" \
 "    (from the cephesmodule). The output is the previous state.\n" \
@@ -1059,25 +1079,72 @@ static PyObject *errprint_func(PyObject *self, PyObject *args)
   int oldflag = 0;
   if (!PyArg_ParseTuple ( args, "|i;cephes.errprint", &inflag)) return NULL;
 
-  oldflag = scipy_special_print_error_messages;  
+  oldflag = scipy_special_print_error_messages;
   if (inflag != -37) {
     scipy_special_print_error_messages = (inflag != 0);
   }
   return PyInt_FromLong((long) oldflag);
 }
 
-  
+
 static struct PyMethodDef methods[] = {
   {"errprint", errprint_func, METH_VARARGS, errprint_doc},
   {NULL,		NULL, 0}		/* sentinel */
 };
 
+#if PY_VERSION_HEX >= 0x03000000
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_cephes",
+    NULL,
+    -1,
+    methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
 
+PyObject *PyInit__cephes(void)
+{
+    PyObject *m, *s, *d;
+
+    m = PyModule_Create(&moduledef);
+    import_array();
+    import_ufunc();
+
+    /* Add some symbolic constants to the module */
+    d = PyModule_GetDict(m);
+
+    s = PyUString_FromString("2.0");
+    PyDict_SetItemString(d, "__version__", s);
+    Py_DECREF(s);
+
+    /* Add scipy_special_print_error_message global variable */
+    /*  No, instead acessible through errprint */
+
+    /* Load the cephes operators into the array module's namespace */
+    Cephes_InitOperators(d);
+
+    /* Register and add the warning type object */
+    scipy_special_SpecialFunctionWarning = PyErr_NewException(
+            "scipy.special._cephes.SpecialFunctionWarning",
+            PyExc_RuntimeWarning,
+            NULL);
+    PyModule_AddObject(m, "SpecialFunctionWarning",
+            scipy_special_SpecialFunctionWarning);
+
+    /* Check for errors */
+    if (PyErr_Occurred())
+        Py_FatalError("can't initialize module _cephes");
+    return m;
+}
+#else
 PyMODINIT_FUNC init_cephes(void) {
   PyObject *m, *d, *s;
-  
+
   /* Create the module and add the functions */
-  m = Py_InitModule("_cephes", methods); 
+  m = Py_InitModule("_cephes", methods);
 
   /* Import the ufunc objects */
   import_array();
@@ -1094,10 +1161,18 @@ PyMODINIT_FUNC init_cephes(void) {
   /*  No, instead acessible through errprint */
 
   /* Load the cephes operators into the array module's namespace */
-  Cephes_InitOperators(d); 
-  
+  Cephes_InitOperators(d);
+
+  /* Register and add the warning type object */
+  scipy_special_SpecialFunctionWarning = PyErr_NewException(
+      "scipy.special._cephes.SpecialFunctionWarning",
+      PyExc_RuntimeWarning,
+      NULL);
+  PyModule_AddObject(m, "SpecialFunctionWarning",
+                     scipy_special_SpecialFunctionWarning);
+
   /* Check for errors */
   if (PyErr_Occurred())
     Py_FatalError("can't initialize module _cephes");
 }
-
+#endif
