@@ -4,7 +4,7 @@
 
 from numpy.testing import TestCase, run_module_suite, assert_equal, \
     assert_array_equal, assert_almost_equal, assert_array_almost_equal, \
-    assert_, rand, dec
+    assert_allclose, assert_, rand, dec
 
 
 import numpy
@@ -195,6 +195,37 @@ class TestHypergeom(TestCase):
         assert_(isinstance(val, numpy.ndarray))
         assert_(val.dtype.char in typecodes['AllInteger'])
 
+    def test_precision(self):
+        # comparison number from mpmath
+        M = 2500
+        n = 50
+        N = 500
+        tot = M
+        good = n
+        hgpmf = stats.hypergeom.pmf(2, tot, good, N)
+        assert_almost_equal(hgpmf, 0.0010114963068932233, 11)
+
+    def test_precision2(self):
+        """Test hypergeom precision for large numbers.  See #1218."""
+        # Results compared with those from R.
+        oranges = 9.9e4
+        pears = 1.1e5
+        fruits_eaten = np.array([3, 3.8, 3.9, 4, 4.1, 4.2, 5]) * 1e4
+        quantile = 2e4
+        res = []
+        for eaten in fruits_eaten:
+            res.append(stats.hypergeom.sf(quantile, oranges + pears, oranges, eaten))
+        expected = np.array([0, 1.904153e-114, 2.752693e-66, 4.931217e-32,
+                             8.265601e-11, 0.1237904, 1])
+        assert_allclose(res, expected, atol=0, rtol=5e-7)
+
+        # Test with array_like first argument
+        quantiles = [1.9e4, 2e4, 2.1e4, 2.15e4]
+        res2 = stats.hypergeom.sf(quantiles, oranges + pears, oranges, 4.2e4)
+        expected2 = [1, 0.1237904, 6.511452e-34, 3.277667e-69]
+        assert_allclose(res2, expected2, atol=0, rtol=5e-7)
+
+
 class TestLogser(TestCase):
     def test_rvs(self):
         vals = stats.logser.rvs(0.75, size=(2, 50))
@@ -332,17 +363,15 @@ class TestSkellam(TestCase):
         assert_almost_equal(stats.skellam.cdf(k, mu1, mu2), skcdfR, decimal=5)
 
 
-class TestHypergeom(TestCase):
-    def test_precision(self):
-        # comparison number from mpmath
-        M = 2500
-        n = 50
-        N = 500
-        tot = M
-        good = n
-        hgpmf = stats.hypergeom.pmf(2, tot, good, N)
+class TestGamma(TestCase):
 
-        assert_almost_equal(hgpmf, 0.0010114963068932233, 11)
+    def test_pdf(self):
+        # a few test cases to compare with R
+        pdf = stats.gamma.pdf(90, 394, scale=1./5)
+        assert_almost_equal(pdf, 0.002312341)
+
+        pdf = stats.gamma.pdf(3, 10, scale=1./5)
+        assert_almost_equal(pdf, 0.1620358)
 
 
 class TestChi2(TestCase):
@@ -364,6 +393,12 @@ class TestDocstring(TestCase):
         if stats.bernoulli.__doc__ is not None:
             self.assertTrue("bernoulli" in stats.bernoulli.__doc__.lower())
 
+    def test_no_name_arg(self):
+        """If name is not given, construction shouldn't fail.  See #1508."""
+        stats.rv_continuous()
+        stats.rv_discrete()
+
+
 class TestEntropy(TestCase):
     def test_entropy_positive(self):
         """See ticket #497"""
@@ -491,7 +526,7 @@ class TestFrozen(TestCase):
         assert_equal(result_f, result)
 
         result_f = frozen.moment(2)
-        result = dist.moment(2)
+        result = dist.moment(2,loc=10.0, scale=3.0)
         assert_equal(result_f, result)
 
     def test_gamma(self):
@@ -667,5 +702,95 @@ def test_regression_ticket_1326():
     assert_almost_equal(stats.chi2.pdf(0.0, 2), 0.5, 14)
 
 
+def test_regression_tukey_lambda():
+    """ Make sure that Tukey-Lambda distribution correctly handles non-positive lambdas.
+    """
+    x = np.linspace(-5.0, 5.0, 101)
+
+    olderr = np.seterr(divide='ignore')
+    try:
+        for lam in [0.0, -1.0, -2.0, np.array([[-1.0], [0.0], [-2.0]])]:
+            p = stats.tukeylambda.pdf(x, lam)
+            assert_((p != 0.0).all())
+            assert_(~np.isnan(p).all())
+
+        lam = np.array([[-1.0], [0.0], [2.0]])
+        p = stats.tukeylambda.pdf(x, lam)
+    finally:
+        np.seterr(**olderr)
+
+    assert_(~np.isnan(p).all())
+    assert_((p[0] != 0.0).all())
+    assert_((p[1] != 0.0).all())
+    assert_((p[2] != 0.0).any())
+    assert_((p[2] == 0.0).any())
+
+
+def test_regression_ticket_1421():
+    """Regression test for ticket #1421 - correction discrete docs."""
+    assert_('pdf(x, mu, loc=0, scale=1)' not in stats.poisson.__doc__)
+    assert_('pmf(x,' in stats.poisson.__doc__)
+
+def test_nan_arguments_ticket_835():
+    assert_(np.isnan(stats.t.logcdf(np.nan)))
+    assert_(np.isnan(stats.t.cdf(np.nan)))
+    assert_(np.isnan(stats.t.logsf(np.nan)))
+    assert_(np.isnan(stats.t.sf(np.nan)))
+    assert_(np.isnan(stats.t.pdf(np.nan)))
+    assert_(np.isnan(stats.t.logpdf(np.nan)))
+    assert_(np.isnan(stats.t.ppf(np.nan)))
+    assert_(np.isnan(stats.t.isf(np.nan)))
+
+    assert_(np.isnan(stats.bernoulli.logcdf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.cdf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.logsf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.sf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.pmf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.logpmf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.ppf(np.nan, 0.5)))
+    assert_(np.isnan(stats.bernoulli.isf(np.nan, 0.5)))
+
+
+
+def test_frozen_fit_ticket_1536():
+    np.random.seed(5678)
+    true = np.array([0.25, 0., 0.5])
+    x = stats.lognorm.rvs(true[0], true[1], true[2], size=100)
+
+    olderr = np.seterr(divide='ignore')
+    try:
+        params = np.array(stats.lognorm.fit(x, floc=0.))
+    finally:
+        np.seterr(**olderr)
+
+    assert_almost_equal(params, true, decimal=2)
+
+    params = np.array(stats.lognorm.fit(x, fscale=0.5, loc=0))
+    assert_almost_equal(params, true, decimal=2)
+
+    params = np.array(stats.lognorm.fit(x, f0=0.25, loc=0))
+    assert_almost_equal(params, true, decimal=2)
+
+    params = np.array(stats.lognorm.fit(x, f0=0.25, floc=0))
+    assert_almost_equal(params, true, decimal=2)
+
+    np.random.seed(5678)
+    loc = 1
+    floc = 0.9
+    x = stats.norm.rvs(loc, 2., size=100)
+    params = np.array(stats.norm.fit(x, floc=floc))
+    expected = np.array([floc, np.sqrt(((x-floc)**2).mean())])
+    assert_almost_equal(params, expected, decimal=4)
+
+def test_regression_ticket_1530():
+    """Check the starting value works for Cauchy distribution fit."""
+    np.random.seed(654321)
+    rvs = stats.cauchy.rvs(size=100)
+    params = stats.cauchy.fit(rvs)
+    expected = (0.045, 1.142)
+    assert_almost_equal(params, expected, decimal=1)
+
+
 if __name__ == "__main__":
     run_module_suite()
+
