@@ -1,11 +1,11 @@
 import sys
 import os
 
-
-from numpy.testing import *
+from numpy.testing import TestCase, assert_
 
 from scipy.weave import catalog
-from weave_test_utils import *
+from weave_test_utils import clear_temp_catalog, restore_temp_catalog, \
+        empty_temp_dir, cleanup_temp_dir
 
 
 class TestDefaultDir(TestCase):
@@ -19,47 +19,56 @@ class TestDefaultDir(TestCase):
             test_file.close()
             os.remove(name)
 
+
 class TestOsDependentCatalogName(TestCase):
     pass
 
+
 class TestCatalogPath(TestCase):
+
     def test_default(self):
         in_path = catalog.default_dir()
         path = catalog.catalog_path(in_path)
         d,f = os.path.split(path)
-        assert(d == in_path)
-        assert(f == catalog.os_dependent_catalog_name())
+        assert_(d == in_path)
+        assert_(f == catalog.os_dependent_catalog_name())
+
     def test_current(self):
         in_path = '.'
         path = catalog.catalog_path(in_path)
         d,f = os.path.split(path)
-        assert(d == os.path.abspath(in_path))
-        assert(f == catalog.os_dependent_catalog_name())
+        assert_(d == os.path.abspath(in_path))
+        assert_(f == catalog.os_dependent_catalog_name())
+
     def test_user(path):
         if sys.platform != 'win32':
             in_path = '~'
             path = catalog.catalog_path(in_path)
             d,f = os.path.split(path)
-            assert(d == os.path.expanduser(in_path))
-            assert(f == catalog.os_dependent_catalog_name())
+            assert_(d == os.path.expanduser(in_path))
+            assert_(f == catalog.os_dependent_catalog_name())
+
     def test_module(self):
         # hand it a module and see if it uses the parent directory
         # of the module.
         path = catalog.catalog_path(os.__file__)
         d,f = os.path.split(os.__file__)
         d2,f = os.path.split(path)
-        assert (d2 == d)
+        assert_(d2 == d)
+
     def test_path(self):
         # use os.__file__ to get a usable directory.
         in_path,f = os.path.split(os.__file__)
         path = catalog.catalog_path(in_path)
         d,f = os.path.split(path)
-        assert (d == in_path)
+        assert_(d == in_path)
+
     def test_bad_path(self):
         # stupid_path_name
         in_path = 'stupid_path_name'
         path = catalog.catalog_path(in_path)
-        assert (path is None)
+        assert_(path is None)
+
 
 class TestGetCatalog(TestCase):
     """ This only tests whether new catalogs are created correctly.
@@ -67,6 +76,7 @@ class TestGetCatalog(TestCase):
         Putting catalogs in the right place is all tested with
         catalog_dir tests.
     """
+
     def get_test_dir(self,erase = 0):
         # make sure tempdir catalog doesn't exist
         import tempfile, glob
@@ -80,6 +90,7 @@ class TestGetCatalog(TestCase):
             for cat_file in cat_files:
                 os.remove(cat_file)
         return pardir
+
     def remove_dir(self,d):
         import distutils.dir_util
         distutils.dir_util.remove_tree(d)
@@ -88,12 +99,15 @@ class TestGetCatalog(TestCase):
         pardir = self.get_test_dir(erase=1)
         cat = catalog.get_catalog(pardir,'r')
         self.remove_dir(pardir)
-        assert(cat is None)
+        assert_(cat is None)
+
     def test_create_catalog(self):
         pardir = self.get_test_dir(erase=1)
         cat = catalog.get_catalog(pardir,'c')
+        assert_(cat is not None)
+        cat.close()
         self.remove_dir(pardir)
-        assert(cat is not None)
+
 
 class TestCatalog(TestCase):
 
@@ -103,12 +117,15 @@ class TestCatalog(TestCase):
             del os.environ['PYTHONCOMPILED']
         else:
             self.old_PYTHONCOMPILED = None
+
     def reset_environ(self):
         if self.old_PYTHONCOMPILED:
             os.environ['PYTHONCOMPILED'] = self.old_PYTHONCOMPILED
             self.old_PYTHONCOMPILED = None
+
     def setUp(self):
         self.clear_environ()
+
     def tearDown(self):
         self.reset_environ()
 
@@ -116,58 +133,66 @@ class TestCatalog(TestCase):
         q = catalog.catalog()
         q.set_module_directory('bob')
         r = q.get_module_directory()
-        assert (r == 'bob')
+        assert_(r == 'bob')
+
     def test_clear_module_directory(self):
         q = catalog.catalog()
         r = q.get_module_directory()
-        assert (r is None)
+        assert_(r is None)
         q.set_module_directory('bob')
         r = q.clear_module_directory()
-        assert (r is None)
+        assert_(r is None)
+
     def test_get_environ_path(self):
         if sys.platform == 'win32': sep = ';'
         else: sep = ':'
         os.environ['PYTHONCOMPILED'] = sep.join(('path1','path2','path3'))
         q = catalog.catalog()
         path = q.get_environ_path()
-        assert(path == ['path1','path2','path3'])
+        assert_(path == ['path1','path2','path3'])
+
     def test_build_search_order1(self):
         """ MODULE in search path should be replaced by module_dir.
         """
         q = catalog.catalog(['first','MODULE','third'])
         q.set_module_directory('second')
         order = q.build_search_order()
-        assert(order == ['first','second','third',catalog.default_dir()])
+        assert_(order == ['first','second','third',catalog.default_dir()])
+
     def test_build_search_order2(self):
         """ MODULE in search path should be removed if module_dir==None.
         """
         q = catalog.catalog(['first','MODULE','third'])
         order = q.build_search_order()
-        assert(order == ['first','third',catalog.default_dir()])
+        assert_(order == ['first','third',catalog.default_dir()])
+
     def test_build_search_order3(self):
         """ If MODULE is absent, module_dir shouldn't be in search path.
         """
         q = catalog.catalog(['first','second'])
         q.set_module_directory('third')
         order = q.build_search_order()
-        assert(order == ['first','second',catalog.default_dir()])
+        assert_(order == ['first','second',catalog.default_dir()])
+
     def test_build_search_order4(self):
         """ Make sure environment variable is getting used.
         """
         q = catalog.catalog(['first','second'])
-        if sys.platform == 'win32': sep = ';'
-        else: sep = ':'
+        if sys.platform == 'win32':
+            sep = ';'
+        else:
+            sep = ':'
         os.environ['PYTHONCOMPILED'] = sep.join(('MODULE','fourth','fifth'))
         q.set_module_directory('third')
         order = q.build_search_order()
-        assert(order == ['first','second','third','fourth','fifth',catalog.default_dir()])
+        assert_(order == ['first','second','third','fourth','fifth',catalog.default_dir()])
 
     def test_catalog_files1(self):
         """ Be sure we get at least one file even without specifying the path.
         """
         q = catalog.catalog()
         files = q.get_catalog_files()
-        assert(len(files) == 1)
+        assert_(len(files) == 1)
 
     def test_catalog_files2(self):
         """ Ignore bad paths in the path.
@@ -175,7 +200,7 @@ class TestCatalog(TestCase):
         q = catalog.catalog()
         os.environ['PYTHONCOMPILED'] = '_some_bad_path_'
         files = q.get_catalog_files()
-        assert(len(files) == 1)
+        assert_(len(files) == 1)
 
     def test_get_existing_files1(self):
         """ Shouldn't get any files when temp doesn't exist and no path set.
@@ -184,7 +209,8 @@ class TestCatalog(TestCase):
         q = catalog.catalog()
         files = q.get_existing_files()
         restore_temp_catalog()
-        assert(len(files) == 0)
+        assert_(len(files) == 0)
+
     def test_get_existing_files2(self):
         """ Shouldn't get a single file from the temp dir.
         """
@@ -197,7 +223,7 @@ class TestCatalog(TestCase):
         q = catalog.catalog()
         files = q.get_existing_files()
         restore_temp_catalog()
-        assert(len(files) == 1)
+        assert_(len(files) == 1)
 
     def test_access_writable_file(self):
         """ There should always be a writable file -- even if it is in temp
@@ -210,12 +236,15 @@ class TestCatalog(TestCase):
         finally:
             f.close()
             os.remove(file)
+
     def test_writable_with_bad_path(self):
         """ There should always be a writable file -- even if search paths contain
             bad values.
         """
-        if sys.platform == 'win32': sep = ';'
-        else: sep = ':'
+        if sys.platform == 'win32':
+            sep = ';'
+        else:
+            sep = ':'
         os.environ['PYTHONCOMPILED'] = sep.join(('_bad_path_name_'))
         q = catalog.catalog()
         file = q.get_writable_file()
@@ -225,6 +254,7 @@ class TestCatalog(TestCase):
         finally:
             f.close()
         os.remove(file)
+
     def test_writable_dir(self):
         """ Check that we can create a file in the writable directory
         """
@@ -237,14 +267,15 @@ class TestCatalog(TestCase):
         finally:
             f.close()
             os.remove(file)
+
     def test_unique_module_name(self):
         """ Check that we can create a file in the writable directory
         """
         q = catalog.catalog()
         file = q.unique_module_name('bob')
         cfile1 = file+'.cpp'
-        assert(not os.path.exists(cfile1))
-        #make sure it is writable
+        assert_(not os.path.exists(cfile1))
+        # Make sure it is writable
         try:
             f = open(cfile1,'w')
             f.write('bob')
@@ -253,8 +284,9 @@ class TestCatalog(TestCase):
         # try again with same code fragment -- should get unique name
         file = q.unique_module_name('bob')
         cfile2 = file+'.cpp'
-        assert(not os.path.exists(cfile2+'.cpp'))
+        assert_(not os.path.exists(cfile2+'.cpp'))
         os.remove(cfile1)
+
     def test_add_function_persistent1(self):
         """ Test persisting a function in the default catalog
         """
@@ -269,7 +301,7 @@ class TestCatalog(TestCase):
         # any way to clean modules???
         restore_temp_catalog()
         for i in funcs:
-            assert(i in pfuncs)
+            assert_(i in pfuncs)
 
     def test_add_function_ordered(self):
         clear_temp_catalog()
@@ -315,13 +347,13 @@ class TestCatalog(TestCase):
         # been read in from a prior catalog file (such as the defualt one).
         # the test should really be made so that these aren't read in, but
         # until I get this figured out...
-        #assert(funcs1 == [string.lower,string.upper])
-        #assert(funcs2 == [os.chdir,os.abort,string.replace,string.find])
-        #assert(funcs3 == [re.purge,re.match,os.open,
+        #assert_(funcs1 == [string.lower,string.upper])
+        #assert_(funcs2 == [os.chdir,os.abort,string.replace,string.find])
+        #assert_(funcs3 == [re.purge,re.match,os.open,
         #                  os.access,string.atoi,string.atof])
-        assert(funcs1[:2] == [string.lower,string.upper]),`funcs1`
-        assert(funcs2[:4] == [os.chdir,os.abort,string.replace,string.find])
-        assert(funcs3[:6] == [re.purge,re.match,os.open,
+        assert_(funcs1[:2] == [string.lower,string.upper]),`funcs1`
+        assert_(funcs2[:4] == [os.chdir,os.abort,string.replace,string.find])
+        assert_(funcs3[:6] == [re.purge,re.match,os.open,
                           os.access,string.atoi,string.atof])
         cleanup_temp_dir(user_dir)
         cleanup_temp_dir(env_dir)
