@@ -3,7 +3,7 @@
 
 import sys
 
-from numpy.testing import *
+from numpy.testing import TestCase, dec, assert_, assert_raises
 
 from scipy.weave import inline_tools
 
@@ -13,7 +13,6 @@ class TestDictConstruct(TestCase):
     # Check that construction from basic types is allowed and have correct
     # reference counts
     #------------------------------------------------------------------------
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_empty(self):
         # strange int value used to try and make sure refcount is 2.
@@ -22,12 +21,12 @@ class TestDictConstruct(TestCase):
                return_val = val;
                """
         res = inline_tools.inline(code)
-        assert sys.getrefcount(res) == 2
-        assert res == {}
+        assert_(sys.getrefcount(res) == 2)
+        assert_(res == {})
 
 
 class TestDictHasKey(TestCase):
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_obj(self):
         class Foo:
@@ -39,8 +38,8 @@ class TestDictHasKey(TestCase):
                return_val =  a.has_key(key);
                """
         res = inline_tools.inline(code,['a','key'])
-        assert res
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(res)
+
     @dec.slow
     def test_int(self):
         a = {}
@@ -49,8 +48,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key(1234);
                """
         res = inline_tools.inline(code,['a'])
-        assert res
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(res)
+
     @dec.slow
     def test_double(self):
         a = {}
@@ -59,8 +58,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key(1234.);
                """
         res = inline_tools.inline(code,['a'])
-        assert res
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(res)
+
     @dec.slow
     def test_complex(self):
         a = {}
@@ -70,9 +69,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key(key);
                """
         res = inline_tools.inline(code,['a','key'])
-        assert res
+        assert_(res)
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_string(self):
         a = {}
@@ -81,8 +79,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key("b");
                """
         res = inline_tools.inline(code,['a'])
-        assert res
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(res)
+
     @dec.slow
     def test_std_string(self):
         a = {}
@@ -92,8 +90,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key(key_name);
                """
         res = inline_tools.inline(code,['a','key_name'])
-        assert res
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(res)
+
     @dec.slow
     def test_string_fail(self):
         a = {}
@@ -102,7 +100,8 @@ class TestDictHasKey(TestCase):
                return_val = a.has_key("c");
                """
         res = inline_tools.inline(code,['a'])
-        assert not res
+        assert_(not res)
+
 
 class TestDictGetItemOp(TestCase):
 
@@ -111,9 +110,8 @@ class TestDictGetItemOp(TestCase):
         a['b'] = 12345
 
         res = inline_tools.inline(code,args)
-        assert res == a['b']
+        assert_(res == a['b'])
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_char(self):
         self.generic_get('return_val = a["b"];')
@@ -123,18 +121,13 @@ class TestDictGetItemOp(TestCase):
     def test_char_fail(self):
         # We can't through a KeyError for dicts on RHS of
         # = but not on LHS.  Not sure how to deal with this.
-        try:
-            self.generic_get('return_val = a["c"];')
-        except KeyError:
-            pass
+        assert_raises(KeyError, self.generic_get, 'return_val = a["c"];')
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_string(self):
         self.generic_get('return_val = a[std::string("b")];')
 
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_obj(self):
         code = """
@@ -148,16 +141,15 @@ class TestDictGetItemOp(TestCase):
     def test_obj_fail(self):
         # We can't through a KeyError for dicts on RHS of
         # = but not on LHS.  Not sure how to deal with this.
-        try:
-            code = """
-                   py::object name = "c";
-                   return_val = a[name];
-                   """
-            self.generic_get(code,['a'])
-        except KeyError:
-            pass
+        code = """
+               py::object name = "c";
+               return_val = a[name];
+               """
+        assert_raises(KeyError, self.generic_get, code, ['a'])
+
 
 class TestDictSetOperator(TestCase):
+
     def generic_new(self,key,val):
         # test that value is set correctly and that reference counts
         # on dict, key, and val are being handled correctly.
@@ -165,12 +157,13 @@ class TestDictSetOperator(TestCase):
         # call once to handle mysterious addition of one ref count
         # on first call to inline.
         inline_tools.inline("a[key] = val;",['a','key','val'])
-        assert a[key] == val
+        assert_(a[key] == val)
         before = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
         inline_tools.inline("a[key] = val;",['a','key','val'])
-        assert a[key] == val
+        assert_(a[key] == val)
         after = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
-        assert before == after
+        assert_(before == after)
+
     def generic_overwrite(self,key,val):
         a = {}
         overwritten = 1
@@ -179,36 +172,35 @@ class TestDictSetOperator(TestCase):
         # on first call to inline.
         before_overwritten = sys.getrefcount(overwritten)
         inline_tools.inline("a[key] = val;",['a','key','val'])
-        assert a[key] == val
+        assert_(a[key] == val)
         before = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
         inline_tools.inline("a[key] = val;",['a','key','val'])
-        assert a[key] == val
+        assert_(a[key] == val)
         after = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
         after_overwritten = sys.getrefcount(overwritten)
-        assert before == after
-        assert before_overwritten == after_overwritten
+        assert_(before == after)
+        assert_(before_overwritten == after_overwritten)
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_new_int_int(self):
         key,val = 1234,12345
         self.generic_new(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_new_double_int(self):
         key,val = 1234.,12345
         self.generic_new(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_new_std_string_int(self):
         key,val = "hello",12345
         self.generic_new(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_new_complex_int(self):
         key,val = 1+1j,12345
         self.generic_new(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_new_obj_int(self):
         class Foo:
@@ -216,27 +208,26 @@ class TestDictSetOperator(TestCase):
         key,val = Foo(),12345
         self.generic_new(key,val)
 
-    @dec.knownfailureif(sys.platform=='win32')
     @dec.slow
     def test_overwrite_int_int(self):
         key,val = 1234,12345
         self.generic_overwrite(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_overwrite_double_int(self):
         key,val = 1234.,12345
         self.generic_overwrite(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_overwrite_std_string_int(self):
         key,val = "hello",12345
         self.generic_overwrite(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_overwrite_complex_int(self):
         key,val = 1+1j,12345
         self.generic_overwrite(key,val)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_overwrite_obj_int(self):
         class Foo:
@@ -244,7 +235,9 @@ class TestDictSetOperator(TestCase):
         key,val = Foo(),12345
         self.generic_overwrite(key,val)
 
+
 class TestDictDel(TestCase):
+
     def generic(self,key):
         # test that value is set correctly and that reference counts
         # on dict, key, are being handled correctly. after deletion,
@@ -252,35 +245,35 @@ class TestDictDel(TestCase):
         a = {}
         a[key] = 1
         inline_tools.inline("a.del(key);",['a','key'])
-        assert key not in a
+        assert_(key not in a)
         a[key] = 1
         before = sys.getrefcount(a), sys.getrefcount(key)
         inline_tools.inline("a.del(key);",['a','key'])
-        assert key not in a
+        assert_(key not in a)
         after = sys.getrefcount(a), sys.getrefcount(key)
-        assert before[0] == after[0]
-        assert before[1] == after[1] + 1
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(before[0] == after[0])
+        assert_(before[1] == after[1] + 1)
+
     @dec.slow
     def test_int(self):
         key = 1234
         self.generic(key)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_double(self):
         key = 1234.
         self.generic(key)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_std_string(self):
         key = "hello"
         self.generic(key)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_complex(self):
         key = 1+1j
         self.generic(key)
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_obj(self):
         class Foo:
@@ -288,43 +281,45 @@ class TestDictDel(TestCase):
         key = Foo()
         self.generic(key)
 
+
 class TestDictOthers(TestCase):
-    @dec.knownfailureif(sys.platform=='win32')
+
     @dec.slow
     def test_clear(self):
         a = {}
         a["hello"] = 1
         inline_tools.inline("a.clear();",['a'])
-        assert not a
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(not a)
+
     @dec.slow
     def test_items(self):
         a = {}
         a["hello"] = 1
         items = inline_tools.inline("return_val = a.items();",['a'])
-        assert items == a.items()
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(items == a.items())
+
     @dec.slow
     def test_values(self):
         a = {}
         a["hello"] = 1
         values = inline_tools.inline("return_val = a.values();",['a'])
-        assert values == a.values()
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(values == a.values())
+
     @dec.slow
     def test_keys(self):
         a = {}
         a["hello"] = 1
         keys = inline_tools.inline("return_val = a.keys();",['a'])
-        assert keys == a.keys()
-    @dec.knownfailureif(sys.platform=='win32')
+        assert_(keys == a.keys())
+
     @dec.slow
     def test_update(self):
         a,b = {},{}
         a["hello"] = 1
         b["hello"] = 2
         inline_tools.inline("a.update(b);",['a','b'])
-        assert a == b
+        assert_(a == b)
+
 
 if __name__ == "__main__":
     import nose
