@@ -1,19 +1,21 @@
 """ Test refcounting and behavior of SCXX.
 """
-import time
-import os,sys
 
-from numpy.testing import *
-set_package_path()
-from weave import inline_tools
-restore_path()
+import sys
 
-class test_object_construct(NumpyTestCase):
+from numpy.testing import TestCase, dec, assert_equal, assert_, assert_raises
+
+from scipy.weave import inline_tools
+
+
+class TestObjectConstruct(TestCase):
     #------------------------------------------------------------------------
     # Check that construction from basic types is allowed and have correct
     # reference counts
     #------------------------------------------------------------------------
-    def check_int(self,level=5):
+
+    @dec.slow
+    def test_int(self):
         # strange int value used to try and make sure refcount is 2.
         code = """
                py::object val = 1001;
@@ -22,7 +24,9 @@ class test_object_construct(NumpyTestCase):
         res = inline_tools.inline(code)
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,1001)
-    def check_float(self,level=5):
+
+    @dec.slow
+    def test_float(self):
         code = """
                py::object val = (float)1.0;
                return_val = val;
@@ -30,7 +34,9 @@ class test_object_construct(NumpyTestCase):
         res = inline_tools.inline(code)
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,1.0)
-    def check_double(self,level=5):
+
+    @dec.slow
+    def test_double(self):
         code = """
                py::object val = 1.0;
                return_val = val;
@@ -38,7 +44,9 @@ class test_object_construct(NumpyTestCase):
         res = inline_tools.inline(code)
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,1.0)
-    def check_complex(self,level=5):
+
+    @dec.slow
+    def test_complex(self):
         code = """
                std::complex<double> num = std::complex<double>(1.0,1.0);
                py::object val = num;
@@ -47,7 +55,9 @@ class test_object_construct(NumpyTestCase):
         res = inline_tools.inline(code)
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,1.0+1.0j)
-    def check_string(self,level=5):
+
+    @dec.slow
+    def test_string(self):
         code = """
                py::object val = "hello";
                return_val = val;
@@ -56,7 +66,8 @@ class test_object_construct(NumpyTestCase):
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,"hello")
 
-    def check_std_string(self,level=5):
+    @dec.slow
+    def test_std_string(self):
         code = """
                std::string s = std::string("hello");
                py::object val = s;
@@ -66,18 +77,13 @@ class test_object_construct(NumpyTestCase):
         assert_equal(sys.getrefcount(res),2)
         assert_equal(res,"hello")
 
-class test_object_print(NumpyTestCase):
+class TestObjectPrint(TestCase):
     #------------------------------------------------------------------------
     # Check the object print protocol.
     #------------------------------------------------------------------------
-    def check_stdout(self,level=5):
-        code = """
-               py::object val = "how now brown cow";
-               val.print(stdout);
-               """
-        res = inline_tools.inline(code)
-        # visual check on this one.
-    def check_stringio(self,level=5):
+
+    @dec.slow
+    def test_stringio(self):
         import cStringIO
         file_imposter = cStringIO.StringIO()
         code = """
@@ -88,7 +94,8 @@ class test_object_print(NumpyTestCase):
         print file_imposter.getvalue()
         assert_equal(file_imposter.getvalue(),"'how now brown cow'")
 
-##    def check_failure(self,level=5):
+##    @dec.slow
+##    def test_failure(self):
 ##        code = """
 ##               FILE* file = 0;
 ##               py::object val = "how now brown cow";
@@ -101,41 +108,52 @@ class test_object_print(NumpyTestCase):
 ##            pass
 
 
-class test_object_cast(NumpyTestCase):
-    def check_int_cast(self,level=5):
+class TestObjectCast(TestCase):
+
+    @dec.slow
+    def test_int_cast(self):
         code = """
                py::object val = 1;
-               int raw_val = val;
-               """
-        inline_tools.inline(code)
-    def check_double_cast(self,level=5):
-        code = """
-               py::object val = 1.0;
-               double raw_val = val;
-               """
-        inline_tools.inline(code)
-    def check_float_cast(self,level=5):
-        code = """
-               py::object val = 1.0;
-               float raw_val = val;
-               """
-        inline_tools.inline(code)
-    def check_complex_cast(self,level=5):
-        code = """
-               std::complex<double> num = std::complex<double>(1.0,1.0);
-               py::object val = num;
-               std::complex<double> raw_val = val;
-               """
-        inline_tools.inline(code)
-    def check_string_cast(self,level=5):
-        code = """
-               py::object val = "hello";
-               std::string raw_val = val;
+               int raw_val __attribute__ ((unused)) = val;
                """
         inline_tools.inline(code)
 
+    @dec.slow
+    def test_double_cast(self):
+        code = """
+               py::object val = 1.0;
+               double raw_val __attribute__ ((unused)) = val;
+               """
+        inline_tools.inline(code)
+
+    @dec.slow
+    def test_float_cast(self):
+        code = """
+               py::object val = 1.0;
+               float raw_val __attribute__ ((unused)) = val;
+               """
+        inline_tools.inline(code)
+
+    @dec.slow
+    def test_complex_cast(self):
+        code = """
+               std::complex<double> num = std::complex<double>(1.0, 1.0);
+               py::object val = num;
+               std::complex<double> raw_val __attribute__ ((unused)) = val;
+               """
+        inline_tools.inline(code)
+
+    @dec.slow
+    def test_string_cast(self):
+        code = """
+               py::object val = "hello";
+               std::string raw_val __attribute__ ((unused)) = val;
+               """
+        inline_tools.inline(code)
+
+
 # test class used for testing python class access from C++.
-class foo:
+class Foo:
     def bar(self):
         return "bar results"
     def bar2(self,val1,val2):
@@ -143,40 +161,49 @@ class foo:
     def bar3(self,val1,val2,val3=1):
         return val1, val2, val3
 
-class str_obj:
-    def __str__(self):
-        return "b"
+#class StrObj:
+#    def __str__(self):
+#        return "b"
 
-class test_object_hasattr(NumpyTestCase):
-    def check_string(self,level=5):
-        a = foo()
+
+class TestObjectHasattr(TestCase):
+
+    @dec.slow
+    def test_string(self):
+        a = Foo()
         a.b = 12345
         code = """
                return_val = a.hasattr("b");
                """
         res = inline_tools.inline(code,['a'])
-        assert res
-    def check_std_string(self,level=5):
-        a = foo()
+        assert_(res)
+
+    @dec.slow
+    def test_std_string(self):
+        a = Foo()
         a.b = 12345
         attr_name = "b"
         code = """
                return_val = a.hasattr(attr_name);
                """
         res = inline_tools.inline(code,['a','attr_name'])
-        assert res
-    def check_string_fail(self,level=5):
-        a = foo()
+        assert_(res)
+
+    @dec.slow
+    def test_string_fail(self):
+        a = Foo()
         a.b = 12345
         code = """
                return_val = a.hasattr("c");
                """
         res = inline_tools.inline(code,['a'])
-        assert not res
-    def check_inline(self,level=5):
+        assert_(not res)
+
+    @dec.slow
+    def test_inline(self):
         """ THIS NEEDS TO MOVE TO THE INLINE TEST SUITE
         """
-        a = foo()
+        a = Foo()
         a.b = 12345
         code = """
                throw_error(PyExc_AttributeError,"bummer");
@@ -194,19 +221,21 @@ class test_object_hasattr(NumpyTestCase):
             print 'before, after, after2:', before, after, after2
             pass
 
-    def check_func(self,level=5):
-        a = foo()
+    @dec.slow
+    def test_func(self):
+        a = Foo()
         a.b = 12345
         code = """
                return_val = a.hasattr("bar");
                """
         res = inline_tools.inline(code,['a'])
-        assert res
+        assert_(res)
 
-class test_object_attr(NumpyTestCase):
+
+class TestObjectAttr(TestCase):
 
     def generic_attr(self,code,args=['a']):
-        a = foo()
+        a = Foo()
         a.b = 12345
 
         before = sys.getrefcount(a.b)
@@ -216,43 +245,41 @@ class test_object_attr(NumpyTestCase):
         after = sys.getrefcount(a.b)
         assert_equal(after,before)
 
-    def check_char(self,level=5):
+    @dec.slow
+    def test_char(self):
         self.generic_attr('return_val = a.attr("b");')
 
-    def check_char_fail(self,level=5):
-        try:
-            self.generic_attr('return_val = a.attr("c");')
-        except AttributeError:
-            pass
+    @dec.slow
+    def test_char_fail(self):
+        assert_raises(AttributeError, self.generic_attr, 'return_val = a.attr("c");')
 
-    def check_string(self,level=5):
+    @dec.slow
+    def test_string(self):
         self.generic_attr('return_val = a.attr(std::string("b"));')
 
-    def check_string_fail(self,level=5):
-        try:
-            self.generic_attr('return_val = a.attr(std::string("c"));')
-        except AttributeError:
-            pass
+    @dec.slow
+    def test_string_fail(self):
+        assert_raises(AttributeError, self.generic_attr, 'return_val = a.attr(std::string("c"));')
 
-    def check_obj(self,level=5):
+    @dec.slow
+    def test_obj(self):
         code = """
                py::object name = "b";
                return_val = a.attr(name);
                """
         self.generic_attr(code,['a'])
 
-    def check_obj_fail(self,level=5):
-        try:
-            code = """
-                   py::object name = "c";
-                   return_val = a.attr(name);
-                   """
-            self.generic_attr(code,['a'])
-        except AttributeError:
-            pass
+    @dec.slow
+    def test_obj_fail(self):
+        code = """
+               py::object name = "c";
+               return_val = a.attr(name);
+               """
+        assert_raises(AttributeError, self.generic_attr, code, ['a'])
 
-    def check_attr_call(self,level=5):
-        a = foo()
+    @dec.slow
+    def test_attr_call(self):
+        a = Foo()
         res = inline_tools.inline('return_val = a.attr("bar").call();',['a'])
         first = sys.getrefcount(res)
         del res
@@ -261,42 +288,56 @@ class test_object_attr(NumpyTestCase):
         assert_equal(res,"bar results")
         assert_equal(first,second)
 
-class test_object_set_attr(NumpyTestCase):
+
+class TestObjectSetAttr(TestCase):
 
     def generic_existing(self, code, desired):
         args = ['a']
-        a = foo()
+        a = Foo()
         a.b = 12345
         res = inline_tools.inline(code,args)
         assert_equal(a.b,desired)
 
     def generic_new(self, code, desired):
         args = ['a']
-        a = foo()
+        a = Foo()
         res = inline_tools.inline(code,args)
         assert_equal(a.b,desired)
 
-    def check_existing_char(self,level=5):
+    @dec.slow
+    def test_existing_char(self):
         self.generic_existing('a.set_attr("b","hello");',"hello")
-    def check_new_char(self,level=5):
+
+    @dec.slow
+    def test_new_char(self):
         self.generic_new('a.set_attr("b","hello");',"hello")
-    def check_existing_string(self,level=5):
+
+    @dec.slow
+    def test_existing_string(self):
         self.generic_existing('a.set_attr("b",std::string("hello"));',"hello")
-    def check_new_string(self,level=5):
+
+    @dec.slow
+    def test_new_string(self):
         self.generic_new('a.set_attr("b",std::string("hello"));',"hello")
-    def check_existing_object(self,level=5):
+
+    @dec.slow
+    def test_existing_object(self):
         code = """
                py::object obj = "hello";
                a.set_attr("b",obj);
                """
         self.generic_existing(code,"hello")
-    def check_new_object(self,level=5):
+
+    @dec.slow
+    def test_new_object(self):
         code = """
                py::object obj = "hello";
                a.set_attr("b",obj);
                """
         self.generic_new(code,"hello")
-    def check_new_fail(self,level=5):
+
+    @dec.slow
+    def test_new_fail(self):
         try:
             code = """
                    py::object obj = 1;
@@ -306,103 +347,146 @@ class test_object_set_attr(NumpyTestCase):
         except:
             pass
 
-    def check_existing_int(self,level=5):
+    @dec.slow
+    def test_existing_int(self):
         self.generic_existing('a.set_attr("b",1);',1)
-    def check_existing_double(self,level=5):
+
+    @dec.slow
+    def test_existing_double(self):
         self.generic_existing('a.set_attr("b",1.0);',1.0)
-    def check_existing_complex(self,level=5):
+
+    @dec.slow
+    def test_existing_complex(self):
         code = """
                std::complex<double> obj = std::complex<double>(1,1);
                a.set_attr("b",obj);
                """
         self.generic_existing(code,1+1j)
-    def check_existing_char1(self,level=5):
+
+    @dec.slow
+    def test_existing_char1(self):
         self.generic_existing('a.set_attr("b","hello");',"hello")
-    def check_existing_string1(self,level=5):
+
+    @dec.slow
+    def test_existing_string1(self):
         code = """
                std::string obj = std::string("hello");
                a.set_attr("b",obj);
                """
         self.generic_existing(code,"hello")
 
-class test_object_del(NumpyTestCase):
+
+class TestObjectDel(TestCase):
+
     def generic(self, code):
         args = ['a']
-        a = foo()
+        a = Foo()
         a.b = 12345
         res = inline_tools.inline(code,args)
-        assert not hasattr(a,"b")
+        assert_(not hasattr(a,"b"))
 
-    def check_char(self,level=5):
+    @dec.slow
+    def test_char(self):
         self.generic('a.del("b");')
-    def check_string(self,level=5):
+
+    @dec.slow
+    def test_string(self):
         code = """
                std::string name = std::string("b");
                a.del(name);
                """
         self.generic(code)
-    def check_object(self,level=5):
+
+    @dec.slow
+    def test_object(self):
         code = """
                py::object name = py::object("b");
                a.del(name);
                """
         self.generic(code)
 
-class test_object_cmp(NumpyTestCase):
-    def check_equal(self,level=5):
+
+class TestObjectCmp(TestCase):
+
+    @dec.slow
+    def test_equal(self):
         a,b = 1,1
         res = inline_tools.inline('return_val = (a == b);',['a','b'])
         assert_equal(res,(a == b))
-    def check_equal_objects(self,level=5):
-        class foo:
+
+    @dec.slow
+    def test_equal_objects(self):
+        class Foo:
             def __init__(self,x):
                 self.x = x
             def __cmp__(self,other):
                 return cmp(self.x,other.x)
-        a,b = foo(1),foo(2)
+        a,b = Foo(1),Foo(2)
         res = inline_tools.inline('return_val = (a == b);',['a','b'])
         assert_equal(res,(a == b))
-    def check_lt(self,level=5):
+
+    @dec.slow
+    def test_lt(self):
         a,b = 1,2
         res = inline_tools.inline('return_val = (a < b);',['a','b'])
         assert_equal(res,(a < b))
-    def check_gt(self,level=5):
+
+    @dec.slow
+    def test_gt(self):
         a,b = 1,2
         res = inline_tools.inline('return_val = (a > b);',['a','b'])
         assert_equal(res,(a > b))
-    def check_gte(self,level=5):
+
+    @dec.slow
+    def test_gte(self):
         a,b = 1,2
         res = inline_tools.inline('return_val = (a >= b);',['a','b'])
         assert_equal(res,(a >= b))
-    def check_lte(self,level=5):
+
+    @dec.slow
+    def test_lte(self):
         a,b = 1,2
         res = inline_tools.inline('return_val = (a <= b);',['a','b'])
         assert_equal(res,(a <= b))
-    def check_not_equal(self,level=5):
+
+    @dec.slow
+    def test_not_equal(self):
         a,b = 1,2
         res = inline_tools.inline('return_val = (a != b);',['a','b'])
         assert_equal(res,(a != b))
-    def check_int(self,level=5):
+
+    @dec.slow
+    def test_int(self):
         a = 1
         res = inline_tools.inline('return_val = (a == 1);',['a'])
         assert_equal(res,(a == 1))
-    def check_int2(self,level=5):
+
+    @dec.slow
+    def test_int2(self):
         a = 1
         res = inline_tools.inline('return_val = (1 == a);',['a'])
         assert_equal(res,(a == 1))
-    def check_unsigned_long(self,level=5):
+
+    @dec.slow
+    def test_unsigned_long(self):
         a = 1
         res = inline_tools.inline('return_val = (a == (unsigned long)1);',['a'])
         assert_equal(res,(a == 1))
-    def check_double(self,level=5):
+
+    @dec.slow
+    def test_double(self):
         a = 1
         res = inline_tools.inline('return_val = (a == 1.0);',['a'])
         assert_equal(res,(a == 1.0))
-    def check_char(self,level=5):
+
+    @dec.slow
+    def test_char(self):
         a = "hello"
         res = inline_tools.inline('return_val = (a == "hello");',['a'])
         assert_equal(res,(a == "hello"))
-    def check_std_string(self,level=5):
+
+    @dec.slow
+    def test_std_string(self):
         a = "hello"
         code = """
                std::string hello = std::string("hello");
@@ -411,14 +495,17 @@ class test_object_cmp(NumpyTestCase):
         res = inline_tools.inline(code,['a'])
         assert_equal(res,(a == "hello"))
 
-class test_object_repr(NumpyTestCase):
-    def check_repr(self,level=5):
-        class foo:
+
+class TestObjectRepr(TestCase):
+
+    @dec.slow
+    def test_repr(self):
+        class Foo:
             def __str__(self):
                 return "str return"
             def __repr__(self):
                 return "repr return"
-        a = foo()
+        a = Foo()
         res = inline_tools.inline('return_val = a.repr();',['a'])
         first = sys.getrefcount(res)
         del res
@@ -427,14 +514,17 @@ class test_object_repr(NumpyTestCase):
         assert_equal(first,second)
         assert_equal(res,"repr return")
 
-class test_object_str(NumpyTestCase):
-    def check_str(self,level=5):
-        class foo:
+
+class TestObjectStr(TestCase):
+
+    @dec.slow
+    def test_str(self):
+        class Foo:
             def __str__(self):
                 return "str return"
             def __repr__(self):
                 return "repr return"
-        a = foo()
+        a = Foo()
         res = inline_tools.inline('return_val = a.str();',['a'])
         first = sys.getrefcount(res)
         del res
@@ -444,15 +534,19 @@ class test_object_str(NumpyTestCase):
         print res
         assert_equal(res,"str return")
 
-class test_object_unicode(NumpyTestCase):
+
+class TestObjectUnicode(TestCase):
+
     # This ain't going to win awards for test of the year...
-    def check_unicode(self,level=5):
-        class foo:
+
+    @dec.slow
+    def test_unicode(self):
+        class Foo:
             def __repr__(self):
                 return "repr return"
             def __str__(self):
                 return "unicode"
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.unicode();',['a'])
         first = sys.getrefcount(res)
         del res
@@ -461,42 +555,54 @@ class test_object_unicode(NumpyTestCase):
         assert_equal(first,second)
         assert_equal(res,"unicode")
 
-class test_object_is_callable(NumpyTestCase):
-    def check_true(self,level=5):
-        class foo:
+
+class TestObjectIsCallable(TestCase):
+
+    @dec.slow
+    def test_true(self):
+        class Foo:
             def __call__(self):
                 return 0
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.is_callable();',['a'])
-        assert res
-    def check_false(self,level=5):
-        class foo:
-            pass
-        a= foo()
-        res = inline_tools.inline('return_val = a.is_callable();',['a'])
-        assert not res
+        assert_(res)
 
-class test_object_call(NumpyTestCase):
-    def check_noargs(self,level=5):
-        def foo():
+    @dec.slow
+    def test_false(self):
+        class Foo:
+            pass
+        a= Foo()
+        res = inline_tools.inline('return_val = a.is_callable();',['a'])
+        assert_(not res)
+
+
+class TestObjectCall(TestCase):
+
+    @dec.slow
+    def test_noargs(self):
+        def Foo():
             return (1,2,3)
-        res = inline_tools.inline('return_val = foo.call();',['foo'])
+        res = inline_tools.inline('return_val = Foo.call();',['Foo'])
         assert_equal(res,(1,2,3))
-        assert_equal(sys.getrefcount(res),2)
-    def check_args(self,level=5):
-        def foo(val1,val2):
+        assert_equal(sys.getrefcount(res),3) # should be 2?
+
+    @dec.slow
+    def test_args(self):
+        def Foo(val1,val2):
             return (val1,val2)
         code = """
                py::tuple args(2);
                args[0] = 1;
                args[1] = "hello";
-               return_val = foo.call(args);
+               return_val = Foo.call(args);
                """
-        res = inline_tools.inline(code,['foo'])
+        res = inline_tools.inline(code,['Foo'])
         assert_equal(res,(1,"hello"))
         assert_equal(sys.getrefcount(res),2)
-    def check_args_kw(self,level=5):
-        def foo(val1,val2,val3=1):
+
+    @dec.slow
+    def test_args_kw(self):
+        def Foo(val1,val2,val3=1):
             return (val1,val2,val3)
         code = """
                py::tuple args(2);
@@ -504,37 +610,42 @@ class test_object_call(NumpyTestCase):
                args[1] = "hello";
                py::dict kw;
                kw["val3"] = 3;
-               return_val = foo.call(args,kw);
+               return_val = Foo.call(args,kw);
                """
-        res = inline_tools.inline(code,['foo'])
+        res = inline_tools.inline(code,['Foo'])
         assert_equal(res,(1,"hello",3))
         assert_equal(sys.getrefcount(res),2)
-    def check_noargs_with_args(self,level=5):
+
+    @dec.slow
+    def test_noargs_with_args(self):
         # calling a function that does take args with args
         # should fail.
-        def foo():
+        def Foo():
             return "blah"
         code = """
                py::tuple args(2);
                args[0] = 1;
                args[1] = "hello";
-               return_val = foo.call(args);
+               return_val = Foo.call(args);
                """
         try:
-            first = sys.getrefcount(foo)
-            res = inline_tools.inline(code,['foo'])
+            first = sys.getrefcount(Foo)
+            res = inline_tools.inline(code,['Foo'])
         except TypeError:
-            second = sys.getrefcount(foo)
+            second = sys.getrefcount(Foo)
             try:
-                res = inline_tools.inline(code,['foo'])
+                res = inline_tools.inline(code,['Foo'])
             except TypeError:
-                third = sys.getrefcount(foo)
+                third = sys.getrefcount(Foo)
         # first should == second, but the weird refcount error
         assert_equal(second,third)
 
-class test_object_mcall(NumpyTestCase):
-    def check_noargs(self,level=5):
-        a = foo()
+
+class TestObjectMcall(TestCase):
+
+    @dec.slow
+    def test_noargs(self):
+        a = Foo()
         res = inline_tools.inline('return_val = a.mcall("bar");',['a'])
         assert_equal(res,"bar results")
         first = sys.getrefcount(res)
@@ -543,8 +654,10 @@ class test_object_mcall(NumpyTestCase):
         assert_equal(res,"bar results")
         second = sys.getrefcount(res)
         assert_equal(first,second)
-    def check_args(self,level=5):
-        a = foo()
+
+    @dec.slow
+    def test_args(self):
+        a = Foo()
         code = """
                py::tuple args(2);
                args[0] = 1;
@@ -554,8 +667,10 @@ class test_object_mcall(NumpyTestCase):
         res = inline_tools.inline(code,['a'])
         assert_equal(res,(1,"hello"))
         assert_equal(sys.getrefcount(res),2)
-    def check_args_kw(self,level=5):
-        a = foo()
+
+    @dec.slow
+    def test_args_kw(self):
+        a = Foo()
         code = """
                py::tuple args(2);
                args[0] = 1;
@@ -567,8 +682,10 @@ class test_object_mcall(NumpyTestCase):
         res = inline_tools.inline(code,['a'])
         assert_equal(res,(1,"hello",3))
         assert_equal(sys.getrefcount(res),2)
-    def check_std_noargs(self,level=5):
-        a = foo()
+
+    @dec.slow
+    def test_std_noargs(self):
+        a = Foo()
         method = "bar"
         res = inline_tools.inline('return_val = a.mcall(method);',['a','method'])
         assert_equal(res,"bar results")
@@ -578,8 +695,10 @@ class test_object_mcall(NumpyTestCase):
         assert_equal(res,"bar results")
         second = sys.getrefcount(res)
         assert_equal(first,second)
-    def check_std_args(self,level=5):
-        a = foo()
+
+    @dec.slow
+    def test_std_args(self):
+        a = Foo()
         method = "bar2"
         code = """
                py::tuple args(2);
@@ -590,8 +709,10 @@ class test_object_mcall(NumpyTestCase):
         res = inline_tools.inline(code,['a','method'])
         assert_equal(res,(1,"hello"))
         assert_equal(sys.getrefcount(res),2)
-    def check_std_args_kw(self,level=5):
-        a = foo()
+
+    @dec.slow
+    def test_std_args_kw(self):
+        a = Foo()
         method = "bar3"
         code = """
                py::tuple args(2);
@@ -604,10 +725,12 @@ class test_object_mcall(NumpyTestCase):
         res = inline_tools.inline(code,['a','method'])
         assert_equal(res,(1,"hello",3))
         assert_equal(sys.getrefcount(res),2)
-    def check_noargs_with_args(self,level=5):
+
+    @dec.slow
+    def test_noargs_with_args(self):
         # calling a function that does take args with args
         # should fail.
-        a = foo()
+        a = Foo()
         code = """
                py::tuple args(2);
                args[0] = 1;
@@ -626,109 +749,135 @@ class test_object_mcall(NumpyTestCase):
         # first should == second, but the weird refcount error
         assert_equal(second,third)
 
-class test_object_hash(NumpyTestCase):
-    def check_hash(self,level=5):
-        class foo:
+
+class TestObjectHash(TestCase):
+
+    @dec.slow
+    def test_hash(self):
+        class Foo:
             def __hash__(self):
                 return 123
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.hash(); ',['a'])
         print 'hash:', res
         assert_equal(res,123)
 
-class test_object_is_true(NumpyTestCase):
-    def check_true(self,level=5):
-        class foo:
+
+class TestObjectIsTrue(TestCase):
+
+    @dec.slow
+    def test_true(self):
+        class Foo:
             pass
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.is_true();',['a'])
         assert_equal(res,1)
-    def check_false(self,level=5):
+
+    @dec.slow
+    def test_false(self):
         a= None
         res = inline_tools.inline('return_val = a.is_true();',['a'])
         assert_equal(res,0)
 
-class test_object_is_true(NumpyTestCase):
-    def check_false(self,level=5):
-        class foo:
-            pass
-        a= foo()
-        res = inline_tools.inline('return_val = a.mcall("not");',['a'])
-        assert_equal(res,0)
-    def check_true(self,level=5):
-        a= None
-        res = inline_tools.inline('return_val = a.mcall("not");',['a'])
-        assert_equal(res,1)
 
-class test_object_type(NumpyTestCase):
-    def check_type(self,level=5):
-        class foo:
+class TestObjectType(TestCase):
+
+    @dec.slow
+    def test_type(self):
+        class Foo:
             pass
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.type();',['a'])
         assert_equal(res,type(a))
 
-class test_object_size(NumpyTestCase):
-    def check_size(self,level=5):
-        class foo:
+
+class TestObjectSize(TestCase):
+
+    @dec.slow
+    def test_size(self):
+        class Foo:
             def __len__(self):
                 return 10
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.size();',['a'])
         assert_equal(res,len(a))
-    def check_len(self,level=5):
-        class foo:
+
+    @dec.slow
+    def test_len(self):
+        class Foo:
             def __len__(self):
                 return 10
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.len();',['a'])
         assert_equal(res,len(a))
-    def check_length(self,level=5):
-        class foo:
+
+    @dec.slow
+    def test_length(self):
+        class Foo:
             def __len__(self):
                 return 10
-        a= foo()
+        a= Foo()
         res = inline_tools.inline('return_val = a.length();',['a'])
         assert_equal(res,len(a))
 
+
 from UserList import UserList
-class test_object_set_item_op_index(NumpyTestCase):
-    def check_list_refcount(self,level=5):
+
+
+class TestObjectSetItemOpIndex(TestCase):
+
+    @dec.slow
+    def test_list_refcount(self):
         a = UserList([1,2,3])
         # temporary refcount fix until I understand why it incs by one.
         inline_tools.inline("a[1] = 1234;",['a'])
         before1 = sys.getrefcount(a)
         after1 = sys.getrefcount(a)
         assert_equal(after1,before1)
-    def check_set_int(self,level=5):
+
+    @dec.slow
+    def test_set_int(self):
         a = UserList([1,2,3])
         inline_tools.inline("a[1] = 1234;",['a'])
         assert_equal(sys.getrefcount(a[1]),2)
         assert_equal(a[1],1234)
-    def check_set_double(self,level=5):
+
+    @dec.slow
+    def test_set_double(self):
         a = UserList([1,2,3])
         inline_tools.inline("a[1] = 123.0;",['a'])
         assert_equal(sys.getrefcount(a[1]),2)
         assert_equal(a[1],123.0)
-    def check_set_char(self,level=5):
+
+    @dec.slow
+    def test_set_char(self):
         a = UserList([1,2,3])
         inline_tools.inline('a[1] = "bubba";',['a'])
         assert_equal(sys.getrefcount(a[1]),2)
         assert_equal(a[1],'bubba')
-    def check_set_string(self,level=5):
+
+    @dec.slow
+    def test_set_string1(self):
         a = UserList([1,2,3])
         inline_tools.inline('a[1] = std::string("sissy");',['a'])
         assert_equal(sys.getrefcount(a[1]),2)
         assert_equal(a[1],'sissy')
-    def check_set_string(self,level=5):
+
+    @dec.slow
+    def test_set_string2(self):
         a = UserList([1,2,3])
         inline_tools.inline('a[1] = std::complex<double>(1,1);',['a'])
         assert_equal(sys.getrefcount(a[1]),2)
         assert_equal(a[1],1+1j)
 
+
 from UserDict import UserDict
-class test_object_set_item_op_key(NumpyTestCase):
-    def check_key_refcount(self,level=5):
+
+
+class TestObjectSetItemOpKey(TestCase):
+
+    @dec.slow
+    def test_key_refcount(self):
         a = UserDict()
         code =  """
                 py::object one = 1;
@@ -762,7 +911,8 @@ class test_object_set_item_op_key(NumpyTestCase):
         assert_equal(val[0] + 1, val[1])
         assert_equal(val[1], val[2])
 
-    def check_set_double_exists(self,level=5):
+    @dec.slow
+    def test_set_double_exists(self):
         a = UserDict()
         key = 10.0
         a[key] = 100.0
@@ -775,34 +925,41 @@ class test_object_set_item_op_key(NumpyTestCase):
         assert_equal(sys.getrefcount(key),5)
         assert_equal(sys.getrefcount(a[key]),2)
         assert_equal(a[key],123.0)
-    def check_set_double_new(self,level=5):
+
+    @dec.slow
+    def test_set_double_new(self):
         a = UserDict()
         key = 1.0
         inline_tools.inline('a[key] = 123.0;',['a','key'])
         assert_equal(sys.getrefcount(key),4) # should be 3
         assert_equal(sys.getrefcount(a[key]),2)
         assert_equal(a[key],123.0)
-    def check_set_complex(self,level=5):
+
+    @dec.slow
+    def test_set_complex(self):
         a = UserDict()
         key = 1+1j
         inline_tools.inline("a[key] = 1234;",['a','key'])
-        assert_equal(sys.getrefcount(key),3)
+        assert_equal(sys.getrefcount(key),4) # should be 3
         assert_equal(sys.getrefcount(a[key]),2)
         assert_equal(a[key],1234)
-    def check_set_char(self,level=5):
+
+    @dec.slow
+    def test_set_char(self):
         a = UserDict()
         inline_tools.inline('a["hello"] = 123.0;',['a'])
         assert_equal(sys.getrefcount(a["hello"]),2)
         assert_equal(a["hello"],123.0)
 
-    def check_set_class(self,level=5):
+    @dec.slow
+    def test_set_class(self):
         a = UserDict()
-        class foo:
+        class Foo:
             def __init__(self,val):
                 self.val = val
             def __hash__(self):
                 return self.val
-        key = foo(4)
+        key = Foo(4)
         inline_tools.inline('a[key] = "bubba";',['a','key'])
         first = sys.getrefcount(key)
         inline_tools.inline('a[key] = "bubba";',['a','key'])
@@ -813,15 +970,16 @@ class test_object_set_item_op_key(NumpyTestCase):
         assert_equal(sys.getrefcount(key),4)
         assert_equal(sys.getrefcount(a[key]),2)
         assert_equal(a[key],'bubba')
-    def check_set_from_member(self,level=5):
+
+    @dec.slow
+    def test_set_from_member(self):
         a = UserDict()
         a['first'] = 1
         a['second'] = 2
         inline_tools.inline('a["first"] = a["second"];',['a'])
         assert_equal(a['first'],a['second'])
 
+
 if __name__ == "__main__":
-    import sys
-    if len(sys.argv) == 1:
-        sys.argv.extend(["--level=5"])
-    NumpyTest().run()
+    import nose
+    nose.run(argv=['', __file__])
